
function [phi phik H] = dyadicrecursion2scaleeq(h,J,a);

% dyadicrecursion2scaleeq - finds father function from refinement filter, using
%                           dyadic recursion
% Usage
%   phi = dyadicrecursion2scaleeq(h,J,a);
%   [phi phik H] = dyadicrecursion2scaleeq(h,J,a);
% Input
%   h    real vector with sum(h)=2; refinement filter (default h = [1 1])
%   J    integer scalar; number of resolution levels (default J=1)
%   a    integer scalar or real vector: indicates which solution must be taken
%        in cases where the refinement has more than one solution
%        if a is an integer, then the a-th solution is taken
%        if a is a real vector, then a linear combination is taken with
%        coefficients a
%        (default a = 1);
% Output
%   phi  father function, evaluated in the dyadic points 2^(-J)*(0:2^J*(s-1)) 
%        with s = length(h)
%   phik matrix with phi(k) in integer k, for all linerly independent solutions
%   H    square sxs matrix, so that H*phik = phik

if nargin<3, a = 1; end
if nargin<2, J = 1; end
if nargin<1, h = [1 1]; end
tol = 1.e-8;
h = 2*h/sum(h);

% first, find the values of phi(k) with k integers between 0 and length(h)-1
s = length(h); H = zeros(s,s);
for r=1:s,
   cl = max(1,2*r-s);
   cr = min(s,2*r-1);
   hl = min(s,2*r-1);
   hr = hl-cr+cl;
   H(r,cl:cr) = h(hl:-1:hr);
end
[E Lambda] = eig(H);
spectrum = diag(Lambda);
l1 = find(abs(spectrum-1)<tol);
phik = E(1:s,l1);
sumphik = sum(phik);
sumphik(abs(sumphik)<tol) = 1;
phik = phik*diag(1./sumphik);

if length(a) == 1 & abs(a(1)-round(a(1)))<tol,
   if size(phik,2) >= 1,
      phij = phik(1:s,a);
   else
      phij = zeros(s,1)*NaN;
   end
else
   m = size(phik,2);
   a = [column(a); zeros(m,1)]; a = a(1:m);
   phij = phik*a;
end

hj = h;
nj = s;
for j = 0:J-1,
   Hj = zeros(2*nj-1,nj);
   for r=1:nj,
      Hj = Hj + [zeros(r-1,nj); diag(hj(r)*ones(nj,1)); zeros(nj-r,nj)];
   end
   phij = Hj*phij;
   nj = 2*nj-1;
   hj0 = hj;
   hj = zeros(nj,1); hj(1:2:nj) = hj0;
end
phi = phij;
