
function [Q R P] = minQTAplusBitvec(A,B,W,types,maxjmini);

% minQTAplusBitvec -- orthogonal composite rotations and reflections Q that 
%                     reduce norm(W.*(Q'*A+B),'fro') (vector version)
%  Usage
%    [Q R P] = minQTAplusBitvec(A,B);
%    [Q R P] = minQTAplusBitvec(A,B,W);
%    [Q R P] = minQTAplusBitvec(A,B,types);
%    [Q R P] = minQTAplusBitvec(A,types);
%    [Q R P] = minQTAplusBitvec(A,B,W,types);
%    [Q R P] = minQTAplusBitvec(A,B,W,types,maxjmini);
%  Inputs
%    A        rectangular or squared matrix; let [n m] = size(A)
%    B        matrix, size(A) = size(B), default B = zeros(size(A))
%    W        matrix, size(W) = size(A), default W = ones(size(A))
%    types    scalar, 0 for rotations only, 1 for reflections only, 2 for both
%    maxjmini integer: maximum distance between two columns involved in a
%                      rotation or reflection
%  Outputs
%    Q      orthogonal matrix of size n x n, with n the # rows in A
%    R      R = Q'*A+B
%    P      cell array of Givens rotations or reflections, so that
%           Q = [I+P{1}]*[I+P{2}]*...*[I+P{nn}]
%           where nn = n*(n+1)/2
%  Description
%    Each P{k} is an orthogonal transform (rotation or reflection) involving
%    and affecting two lines of A. Each combination of two lines in A is
%    treated by exactly one P{k}.
%    The routine guarantees that norm(R,'fro') <= norm(I*A+B,'fro')
%  Note
%  See also
%    help minQTAplusBvec
%    help zerossincosx2xvec

tol = 1.e-10;
maxit = 10;
[n m] = size(A);
if nargin == 1, W = ones(size(A)); B = zeros(size(A)); types = 2; end
if nargin == 2, 
   if max(size(B)) == 1, 
      types = B; B = zeros(size(A)); W = ones(size(A));
   else
      W = ones(size(A)); types = 2;
   end
end
if nargin == 3,
   if max(size(W)) == 1, 
      types = W; W = ones(size(A));
   else
      types = 2;
   end
end
if nargin < 5,
   maxjmini = ceil(n/2);
end
maxjmini = min(maxjmini,ceil(n/2));

if any(size(B)~=size(A)), error('dimension mismatch A and B'), end

I = sparse(eye(n)); Q=eye(n); P = cell(1,n);
k = 1;
% if n is odd, then the subsequent lines miss the rotations involving row n and
% rows 1,2,3,4,5,....
for jmini = 1:maxjmini, for bb = 0:min(1,n-2*jmini),
   i = find(mod(bb*jmini+(0:n-jmini-1+bb),2*jmini)<jmini);
   ni = length(i);
   j = i+jmini; j(j>n) = j(j>n)-n;
   ai = A(i,1:m)'; aj = A(j,1:m)'; bi = B(i,1:m)'; bj = B(j,1:m)';
   wi = W(i,1:m)'; wj = W(j,1:m)';
   aiwi = ai.*wi; aiwj = ai.*wj; ajwi = aj.*wi; ajwj = aj.*wj;
   biwi = bi.*wi; biwj = bi.*wj; bjwi = bj.*wi; bjwj = bj.*wj;
   alfa = zeros(ni,2); 
   F = Inf*ones(ni,2);
   if types == 0 | types == 2, % find optimal rotation
      a = sum(ajwi.*ajwi+aiwj.*aiwj-aiwi.*aiwi-ajwj.*ajwj);
      b =  2*sum(aiwj.*ajwj-aiwi.*ajwi);
      c = -2*sum(aiwi.*biwi+ajwj.*bjwj);
      d =  2*sum(aiwj.*bjwj-ajwi.*biwi);
      [alfa0 D2 it] = zerossincosx2xvec(a,b,c,d,maxit,tol,false);
      F0 = zeros(ni,8);
      for r=1:8,
         c0 = cos(alfa0(1:ni,r))*ones(1,m); s0 = sin(alfa0(1:ni,r))*ones(1,m);
         ri = (c0'.*ai-s0'.*aj+bi).*wi; rj = (s0'.*ai+c0'.*aj+bj).*wj;
         F0(1:ni,r) = sum(ri.^2)+sum(rj.^2);
      end
      [F0 r] = min(F0,[],2);
      alfa0 = alfa0((1:ni)+ni*(row(r)-1));
      alfa(1:ni,1) = alfa0;
      F(1:ni,1) = F0;
      c0 = cos(alfa0); s0 = sin(alfa0);
   end
   if types == 1 | types == 2, % find optimal reflection 
                               % (for this, switch roles of bi and bj and of wi
                               % and wj)
      a = sum(ajwj.*ajwj+aiwi.*aiwi-aiwj.*aiwj-ajwi.*ajwi);
      b =  2*sum(aiwi.*ajwi-aiwj.*ajwj);
      c = -2*sum(aiwj.*bjwj+ajwi.*biwi);
      d =  2*sum(aiwi.*biwi-ajwj.*bjwj);
      [alfa1 D2] = zerossincosx2xvec(a,b,c,d,maxit,tol,false);
      F1 = zeros(ni,8);
      for r=1:8,
         c1 = cos(alfa1(1:ni,r))*ones(1,m); s1 = sin(alfa1(1:ni,r))*ones(1,m);
         ri = (s1'.*ai+c1'.*aj+bi).*wi; rj = (c1'.*ai-s1'.*aj+bj).*wj;
         F1(1:ni,r) = sum(ri.^2)+sum(rj.^2);
      end
      [F1 r] = min(F1,[],2);
      alfa1 = alfa1((1:ni)+ni*(row(r)-1));
      alfa(1:ni,2) = alfa1;
      F(1:ni,2) = F1;
      c1 = cos(alfa1); s1 = sin(alfa1);
   end
   [F b] = min(F,[],2);
   alfa = alfa((1:ni)+ni*(row(b)-1));
   Pk = I;
   diagPk = diag(Pk);
   diagPk(i) = c0.*row(b==1)+s1.*row(b==2);
   diagPk(j) = c0.*row(b==1)-s1.*row(b==2);
   Pk = diag(diagPk);
   diagPk = diag(I,1);
   Pk(n*(j-1)+i) = -s0.*row(b==1)+c1.*row(b==2);
   Pk(n*(i-1)+j) =  s0.*row(b==1)+c1.*row(b==2);
   A = Pk*A; Q = Q*Pk'; P{k} = Pk-I; k=k+1;
end, end
R = A+B;
