
function [xopt fopt] = optfunc1Dfibon(f,varargin)

% optfunc1Dfibon -- ThreshLab's optimization routine for 1D functions f(x)
%  Usage
%    [xopt fopt] = ...
%  optfunc1Dfibon(f,'x0',x0,'lo',xlo,'up',xup,'relerr',relerr,...
%                 'component',c,'argpos',argpos,par1,par2,...)
%  Inputs
%    f      : string of function name
%    xlo    : lower bound of search interval [xlo xup] (default 0)
%    xup    : upper bound of search interval [xlo xup] (default 1)
%    relerr : relative precision of output
%    maxit  : maximum number of iterations
%    argpos : integer; position of variable in parameter list (default is 1)
%    c      : integer; index into vector xx so that x=xx(c) is optimized, 
%             while other components of xx are fixed (default is 1)
%    'max' or 'min' : maximization or minimization (default is 'min')
%    par1, par2, ...: fixed parameters of f(x;par1,par2,...), OTHER than
%                     components of xx
%    x0     : starting values of all components of xx, including the component
%             that is optimized over (x0 is indispensible if xx is multivariate,
%             for the sake of the values to be kept fixed)
%  Outputs
%    xopt   : arg min/max f(x) for x in [xlo xup]
%    fopt   : f(xopt)
%  Description
%    This is a simple (unconstrained) 1-d optimization procedure of the
%    function f(x;par1,par2,...), where par1, par2, ... are fixed parameter
%    values and x is a 1-d variable over which the optimization takes place.
%    x may be x=xx(c). If x0 is not specified, xx is taken to be 1d, i.e., x=xx
%
%    if argpos>1, then the function can be optimized w.r.t. other than its
%    first argument. E.g., if argpos = 3, then the routine optimizes
%    f(par1,par2,x,par3,...). Argpos must be smaller than input arguments of f
% 
%    The routine is based on Fibonacci optimization, it does not require
%    information on the derivative (gradient) but the object function (f)
%    should be (at least approximately) convex.
%  Example
%    edit testfuncforoptfunc1Dfibon.m:
%    ---------------------------------------------
%    function f = testfuncforoptfunc1Dfibon01(x,y)
%
%    f = y*(x(1)-0.4).^2+(x(2)-0.9).^2;
%    ---------------------------------------------
%    then 
%    y = 0.5;
%    x0 = [1 1];
%    f = 'testfuncforoptfunc1Dfibon';
%    xlo = 0;
%    xhi = 1;
%    relerr = 1.e-5;
%    x1min = optfunc1Dfibon(f,'x0',x0,'lo',xlo,'hi',xhi,'relerr',relerr,...
%                           'comp',1,'argpos',1,y);
%    has result
%    x1min = 0.400
% 
%  Notes
%    (1) Users of the optim(ization) or funfun toolboxes may wish to replace
%    the code by a simple call of the functions 
%    fminbnd (previously fmin), fmincon (previously constr), fminsearch.
%    (2) This version of optfunc1Dfibon.m disregards the starting value of 
%        xx(c)
%    (but needs starting values of other components of x if x is multivariate)
%  See also
%    help minfunc1Dfibon
%    help maxfunc1Dfibon

x0 = 0;
xlo = 0; xup = 1; relerr = 1.e-4; maxit = Inf;
argpos = 1;
c = 1;
nvarargin = length(varargin);
inputsinvarargin = (1:nvarargin);
minimize = 1;
for k = 1:nvarargin, if ischar(varargin{k}), 
   varname = varargin{k};
   switch varname,
   case {'lo','low','xlo','xlow'},
      xlo = varargin{k+1};
      inputsinvarargin(k:k+1) = 0;
   case {'hi','high','xhi','xhigh','up','xup'},
      xup = varargin{k+1};
      inputsinvarargin(k:k+1) = 0;
   case {'relerr','eps','tol'},
      relerr = varargin{k+1};
      inputsinvarargin(k:k+1) = 0;
   case {'argpos'},
      argpos = varargin{k+1};
      inputsinvarargin(k:k+1) = 0;
   case {'comp','c','component'},
      c = varargin{k+1};
      inputsinvarargin(k:k+1) = 0;
   case {'max'}
      minimize = 0;
      inputsinvarargin(k) = 0;
   case {'maxit'}
      maxit = varargin{k+1};
      inputsinvarargin(k:k+1) = 0;
   case {'min'}
      inputsinvarargin(k) = 0;
   case {'x0','start','starting','x'}
      x0 = varargin{k+1};
      inputsinvarargin(k:k+1) = 0;
   end
end, end

xx = x0;
if xup < xlo, xuplo = xup; xup = xlo; xlo = xuplo; end


inputsinvarargin = inputsinvarargin(find(inputsinvarargin));
parinf = varargin(inputsinvarargin);
narginf = length(parinf)+1;
arginf = cell(1,narginf);
for i = 1:argpos-1, arginf{i} = parinf{i}; end
for i = argpos+1:narginf, arginf{i} = parinf{i-1}; end

%initialisation Fibonacci numbers
F0 = 1;
F1 = 1;
F2 = 2;
Niter = 1;

% computation of number of iteration steps in optimisation
while (xup - xlo) / F2 > relerr
   F0 = F1;
   F1 = F2;
   F2 = F0 + F1;
   Niter = Niter + 1;
end

Niter = min(Niter,maxit);

% optimization

a = xlo;
b = xup;

v = a + (F1/F2) * (b - a); xx(c) = v; arginf{argpos} = xx;
fv = feval(f,arginf{:});
u = a + (F0/F2) * (b - a); xx(c) = u; arginf{argpos} = xx;
fu = feval(f,arginf{:});
if ~minimize, fu = -fu; fv = -fv; end

for k = 1:Niter - 1
   if (fu > fv)
      a = u;
      u = v;
      fu = fv;
      v = b-u+a; xx(c) = v; arginf{argpos} = xx;
      fv = feval(f,arginf{:});
      if ~minimize, fv = -fv; end
   else
      b = v;
      v = u;
      fv = fu;
      u = a+b-v; xx(c) = u; arginf{argpos} = xx;
      fu = feval(f,arginf{:});
      if ~minimize, fu = -fu; end
   end
end

xopt = (a+b)/2; xx(c) = xopt; arginf{argpos} = xx;
fopt = feval(f,arginf{:});

% Copyright (c) Maarten Jansen
%
% This software is part of ThreshLab and is copyrighted material. 

