
% This test illustrates that minimum MSE thresholds, outperform minimax,
% universal or FDR thresholds when we have the choice between soft- and hard
% thresholding
%
% In spite of ThreshLab's intelligent and fast routine for marginal 
% log-likelihood maximization in EBayesThresh approach, this routine is still
% the bottleneck in the simulation study when the zero inflated Laplace prior
% (with two parameters) is used. Simulations run much faster if only one
% parameter needs to be tested, so it is possible to make one parameter (a) to
% be "known". This speeds up the study, but also untruely favors EBayesThresh
% We can estimate a from the unobserved noise-free data and plug that value
% into ebayesthresh. Set afromnoisefree = 1; to run this option
%
% setup of Johnstone and Silverman, Ann.Stat., 2004, p. 1609
%
% alternatively, the simulation may adopt a true zero-inflated Laplacian prior 
% model for the generation of the noise-free components. Set truelaplace = 1;
% for this option

if exist('n') ~= 1, n = 10000; end
if exist('p') ~= 1, p = 0.005; end
K = p*n;
p = K/n;
if exist('mu0') ~= 1, mu0 = 7; end
if exist('nsimul') ~= 1, nsimul = 100; end
if exist('simul1') ~= 1, simul1 = 1; end
if exist('afromnoisefree') ~= 1, afromnoisefree = 0; end
if exist('truelaplace') ~= 1, truelaplace = 0; end
if exist('stdev') ~= 1, stdev = 1; end

M = 500;

alfa = 0.05;

for simul= simul1:nsimul,
   rand('state',M*simul);
   randn('state',M*simul);
   Z = randn(n,1);

   if ~truelaplace,
      % next 2 lines are setup johnstone04:needles
      v = zeros(size(Z));
      x = randperm(n); x = x(1:K); v(x) = mu0;
   else
      % next 3 lines are exact zero inflated Laplacian prior
      a = 1/mu0;
      r = rand(n,1);
      v = randlaplace(n,1,a).*(r<p);
   end

   Y = v+stdev*Z;


   stdevest = MAD1(Y)/invcumgauss(3/4);
   Xoracle = find(abs(v)>stdev);
   vhatoracle = zeros(size(Y)); vhatoracle(Xoracle) = Y(Xoracle);
   vhatMSEsoft = minmsesoft(v,Y);
   vhatSUREsoft = minsuresoft(Y,stdevest);
   vhatGCVsoft = mingcvsoft(Y);

   vhatMSEhard = minmsehard(v,Y);
   vhatSUREhard = minsurehard(Y,stdevest);
% GCVhard is a nonsmooth function, whose optimization requires special
% attention. Using a basic minimization routine (based on Fibonacci),
% we provide the routine with an upperbound
   tmax = sqrt(2*log(n))*stdevest;
   vhatGCVhard = mingcvhard(Y,'maxthr',tmax);


   if afromnoisefree,
      a = 1/mu0;
   else
      a = -1;
   end
   vhatEBthr = ebayesthresh(Y,'prior','laplace','a',a,'thresrule','median');
   thrFDR = FDRthreshold(Y,1,alfa);
   vhatFDR = HT(Y,thrFDR);

   mseoracle(simul) = norm(vhatoracle - v)^2/n;
   mseSUREsoft(simul) = norm(vhatSUREsoft - v)^2/n;
   mseGCVsoft(simul) = norm(vhatGCVsoft - v)^2/n;
   mseMSEsoft(simul) = norm(vhatMSEsoft - v)^2/n;

   mseSUREhard(simul) = norm(vhatSUREhard - v)^2/n;
   mseGCVhard(simul) = norm(vhatGCVhard - v)^2/n;
   mseMSEhard(simul) = norm(vhatMSEhard - v)^2/n;


   mseEBthr(simul) = norm(vhatEBthr - v)^2/n;
   mseFDRthr(simul) = norm(vhatFDR - v)^2/n;

   XMSEsoft = find(abs(vhatMSEsoft)>eps);
   XSUREsoft = find(abs(vhatMSEsoft)>eps);
   XGCVsoft = find(abs(vhatGCVsoft)>eps);
   XMSEhard = find(abs(vhatMSEhard)>eps);
   XSUREhard = find(abs(vhatMSEhard)>eps);
   XGCVhard = find(abs(vhatGCVhard)>eps);
   XEBthr = find(abs(vhatEBthr)>eps);
   XFDRthr = find(abs(vhatFDR)>eps);

   ntruepos = length(Xoracle);
   falseposrateMSEsoft(simul) = length(setdiff(XMSEsoft,Xoracle))/(n-ntruepos);
   falseposrateSUREsoft(simul) = length(setdiff(XSUREsoft,Xoracle))/...
                                 (n-ntruepos);
   falseposrateGCVsoft(simul) = length(setdiff(XGCVsoft,Xoracle))/(n-ntruepos);
   falseposrateMSEhard(simul) = length(setdiff(XMSEhard,Xoracle))/(n-ntruepos);
   falseposrateSUREhard(simul) = length(setdiff(XSUREhard,Xoracle))/...
                                 (n-ntruepos);
   falseposrateGCVhard(simul) = length(setdiff(XGCVhard,Xoracle))/(n-ntruepos);
   falseposrateEBthr(simul) = length(setdiff(XEBthr,Xoracle))/(n-ntruepos);
   falseposrateFDRthr(simul) = length(setdiff(XFDRthr,Xoracle))/(n-ntruepos);

   fdrMSEsoft(simul) = length(setdiff(XMSEsoft,Xoracle))/length(XMSEsoft);
   fdrSUREsoft(simul) = length(setdiff(XSUREsoft,Xoracle))/length(XSUREsoft);
   fdrGCVsoft(simul) = length(setdiff(XGCVsoft,Xoracle))/length(XGCVsoft);
   fdrMSEhard(simul) = length(setdiff(XMSEhard,Xoracle))/length(XMSEhard);
   fdrSUREhard(simul) = length(setdiff(XSUREhard,Xoracle))/length(XSUREhard);
   fdrGCVhard(simul) = length(setdiff(XGCVhard,Xoracle))/length(XGCVhard);
   fdrEBthr(simul) = length(setdiff(XEBthr,Xoracle))/length(XEBthr);
   fdrFDRthr(simul) = length(setdiff(XFDRthr,Xoracle))/length(XFDRthr);

   falsenegrateMSEsoft(simul) = length(setdiff(Xoracle,XMSEsoft))/ntruepos;
   falsenegrateSUREsoft(simul) = length(setdiff(Xoracle,XSUREsoft))/...
                                 (n-ntruepos);
   falsenegrateGCVsoft(simul) = length(setdiff(Xoracle,XGCVsoft))/ntruepos;
   falsenegrateMSEhard(simul) = length(setdiff(Xoracle,XMSEhard))/ntruepos;
   falsenegrateSUREhard(simul) = length(setdiff(Xoracle,XSUREhard))/...
                                 (n-ntruepos);
   falsenegrateGCVhard(simul) = length(setdiff(Xoracle,XGCVhard))/ntruepos;
   falsenegrateEBthr(simul) = length(setdiff(Xoracle,XEBthr))/ntruepos;
   falsenegrateFDRthr(simul) = length(setdiff(Xoracle,XFDRthr))/ntruepos;


simul

end

relmseSUREsoft = mseSUREsoft./mseoracle;
relmseGCVsoft = mseGCVsoft./mseoracle;
relmseMSEsoft = mseMSEsoft./mseoracle;
relmseFDRthr = mseFDRthr./mseoracle;
relmseEBthr = mseEBthr./mseoracle;
relmseSUREhard = mseSUREhard./mseoracle;
relmseGCVhard = mseGCVhard./mseoracle;
relmseMSEhard = mseMSEhard./mseoracle;

pos = (0:7)*0.6;
pos = (0:7);

verticalboxplot(relmseSUREsoft,'linewidth',2,'color','b','pos',pos(1),'shaded')
hold on
verticalboxplot(relmseGCVsoft,'linewidth',2,'color','r','pos',pos(2),'shaded')
verticalboxplot(relmseMSEsoft,'linewidth',2,'color','g','pos',pos(3),'shaded')

verticalboxplot(relmseFDRthr,'linewidth',2,'color','k','pos',pos(4))
verticalboxplot(relmseEBthr,'linewidth',2,'color','m','pos',pos(5))

verticalboxplot(relmseSUREhard,'linewidth',2,'color','b','pos',pos(6))
verticalboxplot(relmseGCVhard,'linewidth',2,'color','r','pos',pos(7))
verticalboxplot(relmseMSEhard,'linewidth',2,'color','g','pos',pos(8))

hold off

set(gca,'fontsize',14,'fontweight','b')

xtics = pos;
xlabs = {'SURE' 'GCV' 'MSE' 'FDR' 'EB' 'SURE' 'GCV' 'MSE'};
set(gca,'XTick',xtics)
set(gca,'XTickLabel',xlabs)
set(gcf,'position',[0 0 1000 500])
set(gcf, 'PaperPosition', [0 0 30 15]);

disp('Shaded boxplots are for SOFT thresholding, others are for hard')
disp('thresholding, or, in case of EB, for the EBayesThresh posterior median')
disp('threshold')
