
% This routine illustrates the mirror effect in Cp and AIC in sparse variable
% selection in a (sparse) signal-plus-noise model
%
% This routine plots results of SUREhard, MSEhard. See also GCVhard

% Plot and font settings
fontopt = {'fontsize',16,'fontweight','bold'};
rightal = {'horizontalalignment','right'};
topal = {'verticalalignment','top'};
figsize = [800 420];
papersize = figsize/96; figpaperpos = [0 0 papersize];


if exist('samplesize') ~= 1, samplesize = 2000; end
if exist('degreeofsparsity') ~= 1, degreeofsparsity = 0.05; end
n = samplesize;
p = degreeofsparsity;
stdev = 1;

if exist('equidistantk') ~= 1, equidistantk = 0; end
% The binary value equidistantk decides whether the plots are made for
% equidistant k (number of selected components) or equidistant threshold
% values.
m = 1000; % number of values k0 for which AIC/Cp is evaluated

if exist('realrandom') ~= 1, realrandom = false; end

if realrandom==false,
   randn('state',2000);
   rand('state',2000);
end



Z = randn(1,n);

a = 1/5;
r = rand(1,n);
beta = randlaplace(1,n,a).*(r<p);
[betasort i] = sort(abs(beta)); betasort = beta(i); 
beta = betasort(length(beta):-1:1);


Y = beta+stdev*Z;

if equidistantk,
   s = floor(n/m);
   kk0 = (1:s:n); % vector of values k0 = n-k; k is number of selected
                  % components
   m = length(kk0); 
   % Rounding effects may cause length(kk0) to be slightly different from the
   % original m
   t = abs(Ysort(kk0)); % empirical threshold corresponding to values of k0
else
   tmax = max(abs(Y));
   t = (1:m)/m*tmax;
   kk0 = zeros(1,m);
   for i = 1:m, kk0(i) = sum(abs(Y)<t(i)); end
end
kk = n - kk0;

gcvsoft = GCV1soft(Y,t);
gcvhard = GCV1softhard(Y,t);
msesoft = MSE1(beta,Y,t,1);
msehard = MSE1(beta,Y,t,0);
STrisk = softrisk(t,beta,stdev);
HTrisk = hardrisk(t,beta,stdev);

suresoft = SURE1soft(Y,t,stdev);
surehard = SURE1hard(Y,t,stdev);

% surehardsoft is min(Cp; k components) as function of k
surehardsoft = zeros(size(surehard));
surehardsoft1 = surehardsoft;
for i = 1:m
   Ythr = HT(Y,t(i));
   k = kk(i);
   surehardsoft(i) = norm(Ythr-Y)^2/n +2*k/n*stdev^2 - stdev^2;
   surehardsoft1(i) = norm(Ythr-Y)^2/n;
end
correction = 4*stdev^2*t.*gauss(t,stdev);

% Next is linear selection on oracle-ordered data
oracleCp = zeros(1,m); SPE = zeros(1,m); % SPE = squared prediction error
for i = 1:m
   k = kk(i);
   Ylin = zeros(size(Y));
   Ylin(1:k) = Y(1:k);
   oracleCp(i) = norm(Ylin-Y)^2/n +2*k/n*stdev^2 - stdev^2;
   SPE(i) = norm(Ylin-beta)^2/n;
end

figure(1)
plot(kk,SPE,'m','linewidth',2)
hold on
plot(kk,oracleCp,'b','linewidth',2)
plot(kk,surehardsoft,'k','linewidth',2)
plot(kk,msehard,'g','linewidth',2)
plot(kk,surehardsoft+correction,'r--','linewidth',2)
legend('SPE oracle','Cp oracle','Cp (SURE) HT','Cp + mirror corr.',...
        'SPE - HT',fontopt{:})
hold off
axy = axis; 
axis off
figpos = get(gcf,'position'); figpos(3:4) = figsize;
xticks = bestticks(axy(2));
drawaxes('xtick',xticks,fontopt{:})
text(axy(2),axy(4)*0.05,'model size',fontopt{:},rightal{:})


% Now the same for AIC vs R (Kulback-Leibler)

AIClin = zeros(1,m); AICthr0 = zeros(1,m);
Rlin = zeros(1,m); Rthr = zeros(1,m);
sigma2hatthr = zeros(1,m);
sigma2hatlin = zeros(1,m);
for i = 1:m
   k = kk(i);
   Ylin = zeros(size(Y));
   Ylin(1:k) = Y(1:k);
   sigma2hatlin(i) = norm(Y-Ylin)^2/(n-k);
   AIClin(i) = -(n-k)/n - log(2*pi*sigma2hatlin(i)) - 2*k/n;
   Rlin(i) = -norm(Ylin-beta)^2/sigma2hatlin(i)/n-log(2*pi*sigma2hatlin(i)) ...
             -stdev^2/sigma2hatlin(i);

   Ythr = HT(Y,t(i));
   sigma2hatthr(i) = norm(Y-Ythr)^2/(n-k);
   AICthr0(i) = -(n-k)/n - log(2*pi*sigma2hatthr(i)) - 2*k/n;
   Rthr(i) = -norm(Ythr-beta)^2/sigma2hatthr(i)/n-log(2*pi*sigma2hatthr(i)) ...
             -stdev^2/sigma2hatthr(i);
end

stdevestGCV = sqrt(varestGCV(Y));
k0tildestGCV = n*(cumgauss(t,stdevestGCV)-cumgauss(-t,stdevestGCV));
AICthrcorrGCV = -(n-kk)/n - log(2*pi*sigma2hatthr) ...
              -2*k0tildestGCV/n.*(kk/n+2*t.*gauss(t,stdevestGCV))./...
                                 (k0tildestGCV/n-2*t.*gauss(t,stdevestGCV));

stdevestMAD = MAD1(Y)/invcumgauss(3/4);
k0tildestMAD = n*(cumgauss(t,stdevestMAD)-cumgauss(-t,stdevestMAD));
AICthrcorrMAD = -(n-kk)/n - log(2*pi*sigma2hatthr) ...
              -2*k0tildestMAD/n.*(kk/n+2*t.*gauss(t,stdevestMAD))./...
                                 (k0tildestMAD/n-2*t.*gauss(t,stdevestMAD));


figure(2)
plot(kk,Rlin,'m','linewidth',2)
hold on
plot(kk,AIClin,'b','linewidth',2)
plot(kk,AICthr0,'k','linewidth',2)
plot(kk,Rthr,'g','linewidth',2)
plot(kk,AICthrcorrGCV,'r--','linewidth',2)
plot(kk,AICthrcorrMAD,'m--')
legend('R oracle','AIC oracle','AICthr','Rthr','AIC with mirror corr.',...
fontopt{:},'location','SE')
hold off
axis([0,n,-10,1])
axis off
figpos = get(gcf,'position'); figpos(3:4) = figsize;
xticks = bestticks(axy(2));
drawaxes('xtick',xticks,fontopt{:})
text(axy(2),axy(4)*0.2,'model size',fontopt{:},rightal{:})


