
function [l,r,z] = findnewoptimalunbalancedhaar(y,k,lmax,rmax,pdf);

% findnewoptimalunbalancedhaar - ThreshLab2/ContinuousUnbalancedHaar
%   Usage
%     [l,r,z] = findnewoptimalunbalancedhaar(y,k,lmax,rmax,pdf);
%   Inputs
%     y       observed (noisy) values
%     k       location around which unbalanced analysis will be renewed
%     lmax    maximum scale of analysis on the left hand side of k
%     rmax    maximum scale of analysis on the right hand side of k
%     pdf     string: 'normal' or 'poisson' for Haar or Haar-Fisz analysis
%             respectively
%   Outputs
%     l       left scale of unbalanced Haar basis
%     r       right scale of unbalanced Haar basis
%     z       signficance/Haar(or Haar-Fisz) wavelet coefficient
%   Description
%     finds optimal values for l and r for an unbalanced Haar or
%     Haar-Fisz transform:
%     (mean(y(k+1:k+r))-mean(y(k-l+1:k)))/sqrt(nn)*sqrt(l*r);
%     where nn = sum(y(k-l+1:k+r)); in the case of Haar-Fisz
%     and   nn = r+l                in the case of Haar
%   Background
%     This routine is called when optimal location within [lmax,rmax] has to be
%     reconsidered. This occurs when a neighbouring change point has been
%     fixed. Future candidates should then be re-evaluated in the sense that
%     their scales should be limited within the intervals between fixed change
%     points, and no longer across the locations of fixed changed points.
%   See also
%     help findoptimalunbalancedhaar i.e., the unconstraint search

N = length(y);
maxzu = 0;
lmin = 1;
rmin = 1;
lstep = ceil((lmax-lmin)/512);
rstep = ceil((rmax-rmin)/512);

r = rmax;
maxr = r;
l = lmax;
maxl = l;
% NOTE: rmax is maximal value for r; maxr is value of r that maximizes object
% funtion. Same for lmax and maxl!!
while lstep > 0 | rstep > 0,
% if one of the steps equals zero, matlab will not go through the iteration (so
% there is no danger for inifinte loops from this point of view)
%lstep
%rstep

   for l = lmin:lstep:lmax,
      if strcmp(pdf,'poisson') | strcmp(pdf,'Poisson') | strcmp(pdf,'pois'),
         nn = sum(y(k-l+1:k+r));
      else
         nn = r+l;
      end
      if nn > 0 & r*l >0,
         zu = (mean(y(k+1:k+r))-mean(y(k-l+1:k)))/sqrt(nn)*sqrt(l*r);
         if abs(zu) > abs(maxzu),
            maxl = l;
            maxzu = zu;
         end
      end
   end
   l = maxl;
   for r = rmin:rstep:rmax,
      if strcmp(pdf,'poisson') | strcmp(pdf,'Poisson') | strcmp(pdf,'pois'),
         nn = sum(y(k-l+1:k+r));
      else
         nn = r+l;
      end
      if nn > 0 & r*l >0,
         zu = (mean(y(k+1:k+r))-mean(y(k-l+1:k)))/sqrt(nn)*sqrt(l*r);
         if abs(zu) > abs(maxzu),
            maxr = r;
            maxzu = zu;
         end
      end
   end
   r = maxr;
   for l = lmin:lstep:lmax,
      if strcmp(pdf,'poisson') | strcmp(pdf,'Poisson') | strcmp(pdf,'pois'),
         nn = sum(y(k-l+1:k+r));
      else
         nn = r+l;
      end
      if nn > 0 & r*l >0,
         zu = (mean(y(k+1:k+r))-mean(y(k-l+1:k)))/sqrt(nn)*sqrt(l*r);
         if abs(zu) > abs(maxzu),
            maxl = l;
            maxzu = zu;
         end
      end
   end
   l = maxl;
   lmin = max(l-lstep,lmin);
   lmax = min(l+lstep,lmax);
   rmin = max(r-rstep,rmin);
   rmax = min(r+rstep,rmax);
   lstep = max(0,min(lstep-1,ceil((lmax-lmin)/512)));
   rstep = max(0,min(rstep-1,ceil((rmax-rmin)/512)));

end

z = maxzu;
