
function [Phi, HH] = DDbasisbyrefinement(x,knots,ptilde);

% DDbasisbyrefinement - ThreshLab2/Irregular/1D/ --- 
%              construct Deslauriers-Dubuc basis using a two-scale equation
% Usage
%    [Phi, HH] = DDbasisbyrefinement(x,knots,ptilde);
% Inputs
%    x      points in which basis functions are evaluated, vector of length n
%    knots   knots of the Deslauriers-Dubuc basis, vector of length nk
%    ptilde  number of vanishing moments (ptilde-1 is degree of polynomials)
% Outputs
%    Phi     matrix of size n x nk
%    HH      (optional) cell array comtaining the (sparse) refinement matrices
% Description
%    Constructs a Deslauriers-Dubuc basis using refinement/subdivision. 
%    This means that the grid of knots is refined up to a fine scale grid that
%    covers all points in x, so that all these points can be attributed
%    function values.
% Notes
%    (1) In order to maintain the order ptilde at all steps of the refinement, 
%    this refinement must insert a refinement point between each pair of coarse
%    scale points (NB.:order = vanishing moments)
%    Therefore the refinement must be dyadic. Part of the code is devoted to
%    construct a dyadic scheme that refines the grid knots into a grid xx that
%    contains the grid x
% See Also
%    help Bsplinebasisbyrefinement

tol = 1.e-6;

% First find, eliminate and keep track of doubles in vector x
[x duplicates] = uniquewithduplicates(x);   

nk = length(knots); n = length(x);
[x xorder] = sort(x); x = column(x); knots = column(sort(knots));
% eliminate points from x that are also in knots (up to tolerance tol)
% These points will be moved in front.
knotsroundtol = round(knots/tol); xroundtol = round(x/tol);
knotinx = find(ismember(knotsroundtol,xroundtol));
% We want knotinx to contain one entry for coinciding knots that are also in x
% (reason: we will use knotinx to reconstruct x, where we assume that x
%  contains no coinciding values; so we do not want doubles in the
%  reconstruction, as this would cause conflicts of sizes)
  [dummy uniqueknotinx] = unique(knotsroundtol(knotinx));
  knotinx = knotinx(uniqueknotinx);
noknot = find(~ismember(xroundtol,knotsroundtol));
% make x contain non-knots only, so that these points can be used for
% refinement purproses
x = x(noknot); n = length(x);
[xx e] = sort([knots;x]);
e = find(e<nk+1);
% e now contains the position of the knots in xx
% Add points to make a dyadic refinement possible
de = e(2:nk)-e(1:nk-1); nr = max(de); J = ceil(log2(nr)); nr = 2^J;
for k=1:nk-1,
   % propose intermediate points xk between knots(k) and knots(k+1)
   xk = knots(k) + (1:nr-1)'/nr*(knots(k+1)-knots(k));
   % for each point already in xx, take out one point from xk (the closest)
   for l=e(k)+1:e(k+1)-1,
      [dm m] = min(abs(xk-xx(l)));
      xk = setdiff(xk,xk(m));
   end
   % add the remaining points of xk to x
   x = [x;xk];
end
% sort the grid with the newly added values
[xx sorted] = sort([knots;x]); nn = length(xx);
e = zeros(nn,1); e(sorted) = (1:nn); 
% e contains the position of the unsorted [knots;x] in the sorted xx
% The first nk elements of e are *in principle* the knots. Exceptions occur
% if knots coincide, therefore e will be redefined for that puprose. But first
% it is used for in the definition of ii
% ii contains indices into the extended vector xx, containing the original
% points x
ii = [knotinx;(nk+1:nk+n)']; ii = e(ii); ii = sort(ii);
% Now define e as the position of the knots in xx
% e = e(1:nk); --> fails if knots coincide
e1 = e(1);
e = (e1:nr:nk*nr);

e2 = e(nk);
nPhi = nk; 
Phi = eye(nPhi);
HH = cell(1,J);
for j=1:J,
   ne = length(e);
   o = (e(2:ne)+e(1:ne-1))/2;
   ne = length(e); no = length(o); nf = ne+no;
   f = zeros(nf,1); f(1:2:nf) = e; f(2:2:nf) = o;
   xe = xx(e); xo = xx(o); xf = xx(f); Ie = eye(ne);
   P = interpolynomialprediction(xe,Ie,xo,ptilde);
   UP = {P,Ie};
   Hj = refinementfromliftingsteps(UP);
   HH{j} = Hj;
   Phi = Hj*Phi;
   e = f;
end
Phi = [zeros(e1-1,nPhi); Phi; zeros(nn-e2,nPhi)];
Phi = Phi(ii,1:nPhi);
Phi(xorder,1:nPhi) = Phi;
Phi = Phi(duplicates,1:nPhi);
