
function [w, nL, odds] = FWT2G1D(x,varargin);

%  FWT2G1D -- ThreshLab/ThreshLab2/Irregular/1D/ 2nd generation frwd.wav.transf.
%   Usage
%     [w, nL, odds] = FWT2G1D(x,y,method,dualpar,primalpar,nlevels);
%     [w, nL, odds] = FWT2G1D(y,method,dualpar,primalpar,nlevels);
%   Inputs
%     x         ordered real vector, length n; covariate values
%     y         real vector or matrix, size n x m or m x n; response values
%     method    char; describing the refinement method (see below)
%     dualpar   integer, matrix or cell array; parameters of refinement method
%     primalpar integer, integer vector, cell, char; parameters of detail basis
%     nlevels   integer; number of resolution levels
%   Outputs
%     w       vector of size(y) with wavelet coefficients
%     nL      integer, number of scaling coefficients at coarsest scale
%     odds    array or cell array of indices into x
%   Description
%     Fast 2nd generation wavelet transform on irregular point sets
%     Applies repeated lifting filter bank.
%
%     possible values of method
%       (1) 'spline', 'Bspline', 'CDF'
%       (2) 'Deslauriers-Dubuc', 'DD'
%       (3) 'brokenbasis', 'BB'
%       (4) 'avint' (Deslauriers-Dubuc with average interpolation)
%
%     if dualpar is an integer, it represents the dual number of vanishing
%     moments
%
%     if primalpar is an integer vector, its first component has the primal
%     number of vanishing moments
%
%   Note
%   See also
%     help IWT2G1D (inverse transform)

%%% Parsing input %%%
n = length(x);
y = [];
primalpar = {};
dualpar = {};
M = []; % number of resolution levels
refinement = 'none';
dualtransform = false;
infoforIWT = false;
middleknots = false;
extendwith0 = false;
nvarargin = length(varargin);
varname = {};
moments = [];
for v=1:nvarargin,
   varv = varargin{v};
   if ~isempty(varname), eval([varname '= varv;']); varname = {};
   elseif ischar(varv), switch varv
   case {'dualtransform','dual','adjoint'},
      dualtransform = true;
   case {'infoforIWT'},
      infoforIWT = true;
   case {'middleknots'},
      middleknots = true;
   case {'addzeros','extendwithzeros','extendwith0'}
      extendwith0 = true;
   otherwise
      if strcmp(refinement,'none'), refinement = varv;
      elseif isempty(primalpar), primalpar = {varv};
      else, varname = varv;
      end
   end
   elseif iscell(varv),
      if isempty(dualpar), dualpar = varv;
      % if iscell(dualpar), then it is typically set Omega for broken basis
      elseif isempty(primalpar), primalpar = varv;
      end
   else
      [rv cv] = size(varv);
      % if ismember(n,[rv cv]) 
      if max(rv,cv)>n-1, % y may be a bit longer than x (boundary effects)
         if isempty(y), y = varv;
         elseif isempty(dualpar), dualpar = varv;
         % if dualpar is a large matrix, then it typically contains the values
         % of Omega for the construction of a broken basis
         elseif isempty(moments), moments = varv;
         else, primalpar = {primalpar{:},varv};
         end
      elseif max(rv,cv)==1 & isempty(dualpar), dualpar = varv;
      elseif min(rv,cv)==1 & isempty(primalpar), primalpar = {varv};
      elseif max(rv,cv)==1 & isempty(M), M = varv;
      else, primalpar = {primalpar{:},varv};
      end
   end
end
if isempty(y), y=x; x=(1:n)/n; end
if isempty(dualpar), dualpar = 1; end
if isempty(M), % check if the value of M has been attributed to primalpar
   M = 1; j = NaN; npp = length(primalpar);
   for i = 1:npp, if max(size(primalpar{i}))==1, j=i; end, end
   if ~isnan(j),
      M = primalpar{j}; notj = setdiff((1:npp),j); primalpar = primalpar(notj);
   end
end
primalpar = {primalpar{:},moments};

transposex = false; if size(x,1)==1, x = x'; transposex = true; end
if size(x,2)>1, error('1D wavelet requires 1D vector of knots; x has 2D'), end
transposey = false; if size(y,1) < size(x,1), y = y'; transposey = true; end
ny = size(y,2);

%%% Preprocessing refinement
refinement = fixrefinement(refinement);
if strcmp(refinement,'BB'), 
   [OmegaL Omega] = smoothsetmatrix(x,dualpar);
   Omega = {OmegaL,Omega,'LR'};
   y = extendresponse(x,y,Omega,extendwith0);
   [PhiB aa LR] = brokenbasis(x,x,Omega);
   %%%% dualpar = {size(Omega,2),PhiB,LR};
   %%%% dualpar = LR;
   dualpar = {LR,Omega};
   % LR contains the left- and right limits of the highest order derivatives of
   % the broken basis in the knots. These limits will be used in the
   % construction of the refinement matrices.
else
   Omega = 'none';
end
nn = size(y,1);
if strcmp(refinement,'avint') & n<nn+1,
   % avint refinement requires the number of knots to be the number of basis
   % functions plus one
   x = extendknots(x,middleknots); n=n+1;
end

%%% Preprocessing detail coefficient calculation
primalpar = fixprimal(primalpar);
mom = primalpar{2}; primaldof = primalpar{1};
if isempty(mom),
   sumdof = primaldof(1)+primaldof(3);
   qq = (0:sumdof-1);
   if strcmp(refinement,'avint'), qq = (1:sumdof-1); end
   mom = finescalemoments(x,qq,refinement,dualpar);
end
if n>nn & size(mom,1)>nn, mom = mom(1:nn,:); end 
   % i.e., if #knots exceeds #scaling functions (avint) 
C = 1;
if isinf(primaldof(2)), C = finescalegram(x,dualpar,primaldof(3)); end,
if ismember(refinement,{'DDUP'}), dualpar = {dualpar,mom}; end
primalpar = {C,1,1};
% reuse primalpar for further parameters to keep track of during the multiscale
% transform, for the sake of the final update
%%% Prepare additional information for output (for use in inverse WT)
if infoforIWT,
   splits = cell(M,2);
   liftedrefinements = cell(M,1);
   boundaryrefinements = cell(M,1);
   finalupdates = cell(M,1);
else
   odds = (1:nn);
end

%%% Initialise mutiscale  transform
w = zeros(nn,ny);
s = y;

%%%%%%%%% Main part: Run the multiscale transform %%%%%%%%%%
for j = 1:M
%%% resolution level j
   % (1) SUBSAMPLING
   [e o c] = splitwithboundaries(n,nn-n); ne = length(e);
   % (2) REFINEMENT
   [UP Hbb] = constructrefinement(x,refinement,dualpar,e,o);
   % (3) COARSE SCALE APPROXIMATION
   if ~infoforIWT, [s d] = FWT1stepscale(s,UP,Hbb,dualtransform); end
   if strcmp(refinement,'BB'),
      % in BB refinement, we need to reconstruct the refinable basis at coarse
      % scale, so we have to compute the dual parameters at coarse scale
      dualpar{1} = coarsescalejumps(dualpar{1},UP,Hbb,c,dualtransform);
   end
   % (4) MOMENTS
   [mome momo] = coarsescalemoments(mom,UP,Hbb,dualtransform);
   % (5) FINAL UPDATE
   [U primalpar] = constructfinalupdate(UP,mome,momo,primaldof,primalpar,e,o);
   % mome and momo are kept out of primalpar, because they have to be updated
   % outside constructfinalupdate. e,o are kept out of primalpar, because they
   % are not updated and they are not specific for the final update
   if ~infoforIWT & ~isnan(U), s = s + U*d; end

   if infoforIWT,
      splits{j,1} = e; splits{j,2} = o;
      liftedrefinements{j} = UP;
      boundaryrefinements{j} = Hbb;
      finalupdates{j} = U;
   else
      odds(1:nn) = [odds(e) odds(o)];
   end
   nn = ne; n = length(c); x = x(c); mom = mome;
   no = length(o); if ~infoforIWT, w(nn+1:nn+no,1:ny) = d; end
   
end
nL = nn; if ~infoforIWT, w(1:nL,1:ny) = s; end
if infoforIWT,
   if transposey, y=y'; end
   w = {y,splits,liftedrefinements,boundaryrefinements,finalupdates,...
        dualtransform};
elseif transposey, w = w';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function operation = fixrefinement(operation)

switch operation,
   case {'spline','splines','Bspline','Bsplines','CDF','cdf'}
      operation = 'Bspline';
   case {'BB','Broken','broken','BrokenBasis','brokenbasis'},
      operation = 'BB';
   case {'DD','Deslauriers','Dubuc','PpolintU','Deslauriers-Dubuc'},
      operation = 'DD';
   case {'PavintU','avint'},
      operation = 'avint';
   case {'UPpolint','DDUP'},
      operation = 'DDUP';
end

function xx = extendknots(x,middleknots,ny);

nx = length(x);
if nargin<3, ny = nx; end
if middleknots==true,
   xx = [x(1); (x(1:nx-1)+x(2:nx))/2; x(nx)];
elseif nx==ny,
   xx = [x; 2*x(nx)-x(nx-1)];
else
   xx = x(1:ny+1);
end

function y = extendresponse(x,y,Omega,addzeros)

n = length(x);
nt = size(Omega{2},2);
nn = size(y,1);
ny = size(y,2);
pt = ceil(nt/2)-1;
if nt>2 & nn==n,
   % we need n+nt-2 fine scaling coefficients
   nn = n+nt-2;
   if addzeros,
      y = [zeros(pt,ny);y;zeros(nt-2-pt,ny)];
   else,
      % on both sides we add nt-1 coefficients, in such a way that if
      % the interior coefficients represent a linear combination of the
      % smooth basis Omega (i.e., Phi(:,int)*y = Omega*c), then the boundary
      % coefficients are an extension of that function.
      % This way, linear combinations of Omega will have zero detail
      % functions.
      l = min(n,2*nt); r = n-l+1;
      A = broken2smooth(x,Omega);
      Ain = A(pt+1:pt+n,1:nt);
      Al = A(1:pt,1:nt); Ar = A(pt+n+1:nn,1:nt);
      y = [Al*pinv(Ain(1:l,1:nt))*y(1:l,1:ny)
           y
           Ar*pinv(Ain(r:n,1:nt))*y(r:n,1:ny)];
      % y = [zeros(pt,ny);y;zeros(nt-pt,ny)];
   end
end

function primalpar = fixprimal(nu);

% create cell of primal parameters, for the construction of the final update
% primalpar{1} has the degrees of freedom dof
%     dof(1) is the number of primal vanishing moments to be provided in the
%            detail basis Psi
%     dof(2) is the number of degrees of freedom to be used in making the
%            transform as close as possible to orthogonal
% primalpar{2} has the scaling function moments (for use in the construction of
%              a detail basis Psi with vanishing moments)

dof = [NaN 0 0]; i = 0; 
semiorth = false; initgram = false; moments = [];
if ischar(nu) | isreal(nu), nu = {nu}; end
if iscell(nu),
   ncells = length(nu);
   for k = 1:ncells, nuk = nu{k};
      if isnumeric(nuk), 
         n = min(size(nuk)); m = max(size(nuk));
         if n==1 & m<5, for l=1:m, i=i+1; dof(i) = nuk(l); end
         else
            moments = nuk;
         end
      end
      if ischar(nuk), switch(nuk),
         case{'semiorth','semi-orth'},
            semiorth = true;
         case{'gram','initgram','Gram','initGram'},
            initgram = true;
      end, end
   end
   if semiorth, dof = [0 Inf 0]; end
   if initgram, dof = [0 Inf 1]; end
   if isnan(dof(1)),
      if ~isempty(moments), dof(1) = min(size(moments)); else, dof(1) = 1; end
   end
   primalpar = {dof,moments};
end

function [UP Hboundary] = constructrefinement(xf,refinement,dualpar,e,o);

% output: lifting scheme UP and boundary handling information Hbb

nf = length(xf);
nc = ceil(nf/2);
ne = length(e);
no = length(o);
nn = ne+no;
nb = nn-nf;
findliftingfromrefinement = false;
if strcmp(refinement,'BB')
   findliftingfromrefinement = true;,
   % (1) find the refinement matrix and fix the subsampling
   LR = dualpar{1}; Omega = dualpar{2}; dualpar = size(Omega{2},2);
   H = refinementbrokenbasis(LR); Hii = H;
end
if nb>0, % boundary treatment (assumes H to be defined at the boundaries)
   iif = (o(1)-1:o(1)+nf-2); iic = (o(1)-1:o(1)+nc-2); % inner points
   bf = [(1:o(1)-2) (o(1)+nf-1:nn)]; % fine scale boundary
   bc = [(1:o(1)-2) (o(1)+nc-1:nc+nb)]; % coarse scale boundary
   Hbb = H(bf,bc); Hib = H(iif,bc); Hbi = H(bf,iic); Hii = H(iif,iic);
   Hboundary = {iif,bf,iic,bc,Hbb,Hib,Hbi,e,o};
else
   Hboundary = {}; 
end
if findliftingfromrefinement,
   UP = liftingstepsfromrefinement(Hii);
   if isnumeric(dualpar), if dualpar==0, UP = {zeros(No,Ne),UP{1}}; end, end
else switch refinement,
   case {'Bspline'},
      UP = liftingstepsBspline(xf,dualpar);
   case {'DD'},
      xe = xf(e); xo = xf(o);
      if iscell(dualpar),
         predictionmethod = dualpar{1}; dualpar = dualpar(2:end);
         P = sparse(feval(predictionmethod,xe,eye(ne),xo,dualpar{:}));
      else
         P = interpolynomialprediction(xe,eye(ne),xo,dualpar);
      end
      UP = {P,eye(ne)};
   case {'DDUP'},
      xe = xf(e); xo = xf(o);
      mom = dualpar{2}; nmom = size(mom,2); dualpar = dualpar{1};
      Me = mom(e,1:nmom); Mo = mom(o,1:nmom);
      U = updateprimalmoments(Mo,Me);
      D = diag(1./(ones(size(xe))+updateprimalmoments(Mo,Me)*ones(size(xo))));
      P = interpolynomialprediction(xe,eye(ne),xo,dualpar);
      UP = {D*U,P,D};
   case {'avint'},
      nn = nf-1; o = o(find(o<nn+1));
      xe = xf(e); xo = xf(o);
      Deltax = xf(2:nn+1)-xf(1:nn);
      Deltaxcoarse = Deltax;
      Deltaxcoarse(o-1) = Deltaxcoarse(o-1)+Deltaxcoarse(o);
      H = sparse(zeros(nn,nn));
      H(e,e) = eye(ne);
      H(o-1,o-1) = diag(Deltax(o-1)./Deltaxcoarse(o-1));
      H(o-1,o) = diag(Deltax(o)./Deltaxcoarse(o-1));
      D = H(e,e);
      U = H(e,o);
      I = sparse(eye(ne));
      P = averinterpolynomialprediction(xe,eye(ne),xo,dualpar);
      % The matrices above have been constructed to define the transform
      % scoarse = D*sfine(e) + U*sfine(o)
      % d = sfine(o) - P*scoarse
      % The scoarse line can be rewritten as
      % scoarse = D*(sfine(e) + D\U*sfine(o))
      % The scheme of frwliftingsteps.m is
      % scoarse = UP{3}\(sfine(e) + UP{1}*sfine(o))
      % d = sfine(o) - UP{2}*scoarse
      % Identification leads to
      % UP{3} = inv(D)
      % UP{1} = D\U
      % UP{2} = P 
      UP = {D\U,P,inv(D)};
   otherwise
      error(['No implementation for refinement ' refinement])
   end
end

function [ye yo] = FWT1stepscale(yf,UP,Hboundary,dualtransform)

ny = size(yf,2); nn = length(yf);
if ~isempty(Hboundary),
   iif = Hboundary{1}; bf = Hboundary{2};
   iic = Hboundary{3}; bc = Hboundary{4};
   Hbb = Hboundary{5}; Hib = Hboundary{6}; Hbi = Hboundary{7};
end
% Hbi is supposed to be zero, meaning that the refinement of the interior basis
% proceeds without intervention from the boundary (i.e., the interior basis is
% refinable on its own.)
if dualtransform == true, UP = duallifting(UP); end
if isempty(Hboundary),
   yiif = yf;
else
   yiif = yf(iif,1:ny);
   if dualtransform == true,
      ybc = Hbb'*yf(bf,1:ny)+Hib'*yiif;
   else
      ybc = Hbb\yf(bf,1:ny); yiif = yiif-Hib*ybc;
   end
end
[yiic yo] = frwliftingsteps(yiif,UP);
if isempty(Hboundary),
   ye = yiic;
else
   ne = length(iic)+length(bc);
   ye = zeros(ne,ny); ye(iic,1:ny) = yiic; ye(bc,1:ny) = ybc;
end

function mom = finescalemoments(x,qq,refinement,dualpar);

% computes the moments of x.^qq at finest scale
% i.e., mom = int_0^1 x^qq phi_J(x) dx

if length(qq)==0, qq=NaN; end
if ~isnan(qq), switch refinement,
case {'BB'},
   mom = brokenbasismoments(x,dualpar{2},qq);
case {'DD','avint','DDUP'},
   mom = initmom(x,qq);
case {'Bspline'},
   mom = Bsplinemoments(x,dualpar,qq,false);
end
else, mom = NaN; end

function C = finescalegram(x,nutilde,initgram);

if iscell(nutilde), nutilde = nutilde{1}; end
if max(size(nutilde))>1, nutilde = nutilde(1,1); end
n = length(x);
if initgram==true,
   C = grammatrixBsplines(x,nutilde);
   l = ceil(nutilde/2)-1; C = C(l+1:l+n,l+1:l+n);
else
   C = eye(n);
end

function [Me Mo] = coarsescalemoments(M,UP,Hbb,dualtransform);

if nargin<4, dualtransform=false; end
if ~isnan(M(1,1)),
   [Me Mo] = FWT1stepscale(M,UP,Hbb,~dualtransform);
else
   Me = NaN; Mo = NaN;
end

function LRe = coarsescalejumps(LRf,UP,Hbb,c,dualtransform);

LRfT = LRf';
LReT = FWT1stepscale(LRfT,UP,Hbb,~dualtransform);
LRe = LReT';
nne = size(LRe,2);
% LRf is a (2n) x nn matrix
% LRf contains left- and right limits of higher order derivatives, defining the
% broken basis Phi (see help brokenbasis)
% We have: Phicoarse = Phifine*H => LRcoarse = LRfine*H,
%    hence LRcoarse' = H'*LRfine', 
% H' is the dual transform of Ht'
% Now keep only the derivatives in the even knots (this operation has to work
% with the knots subdivision, not with the basis function subdivsion, hence it
% should not take the boundaries into accoun)
nc = length(c); cc = staircase(1:2*nc,c,c)*2-1 + mod(0:2*nc-1,2);
LRe = LRe(cc,1:nne);


function [U par] = constructfinalupdate(UP,Me,Mo,dof,par,e,o)

U = NaN; ne = length(e); no = length(o); nn = ne+no;
p = 0; m = 0; r = 0;
if length(dof)>0, p = dof(1); end
if length(dof)>1, m = dof(2); end
if length(dof)>2, r = dof(3); end
nmom = size(Me,2);
if p > 0 & m == 0, % all free parameters for VANISHING MOMENTS
   U = updateprimalmoments(o,e,Mo,Me,[p r]);
end
if isinf(m), % i.e., semiorth==true, make semi-orthogonal transform
   C = par{1};
   [U, C] = updatesemiorth(UP,C);
   par{1} = C;
   % Note: updatesemiorth(UP,I) (i.e., with initgram = false) is equivalent to
   % updateLSapprxprimmom with nu = [0 Inf];
elseif m > 0, % make transform as close as possible to ortho.
   W = par{1}; Wt = par{2};
   if any(size(W)~=[nn nn]), W = eye(nn); end
   if any(size(Wt)~=[nn nn]), Wt = eye(nn); end
   [U, W, Wt] = updateLSapprxprimmom(UP,p+m,Me,Mo,W,Wt);
   par{1} = W; par{2} = Wt;
   % Although filters Ht, Gt are computed explicitly for optimisation of
   % the update, Wt is NOT used for calculation of the actual transform, as
   % Wt*input requires more computations than the lifting scheme
end
