
function w = FWT2Gpacket(x,y,nt,n,tree,varargin);

%  FWT2GPpacket -- 2nd gen. frwd. wav. packet transf.; 
%   Usage
%     w = FWT2Gpacket(x,y,nt,n,M,tree,<arguments>);
%   Inputs
%     x      grid-locations of observations (time, space)
%     y      observed values
%     nt     dual vanishing moments -- polynomial prediction order
%     n      integer vector; # free parameters in final update (see note (1))
%     tree   binary tree of subbands to be decomposed (several data
%            structures are possible, see below)
%     'spline'   invokes spline wavelet tranform
%     'DD','Deslauriers','PpolintU'   invokes Deslauriers-Dubuc transform
%   Outputs
%     w       vector of size(y) with wavelet packet coefficients
%   Description
%     Fast 2nd generation wavelet packet transform on irregular point sets
%     Applies repeated lifting filter bank.
%     The binary tree defining the subbands to be decomposed can be specified
%     in several ways
%     (1)
%     as an ordered vector of integers, which are indices into a full binary
%     tree (root=1; first level = {2,3}; offspring of 2 = {4,5} and so on)
%     defining the subtree of that full binary tree corresponding to the
%     subbands to be decomposed. As the root must be part of the subtree, the
%     first element of the vector is a 1.
%     (2)
%     as matrix or cell of two rows or two columns, where the first row/column
%     is an ordered vector of integers giving for each node/vertex in the tree
%     its parent node, while the second row/column is a binary vector
%     indicating whether this node is the scaling or detail offspring of that
%     parent node.
%     (3)
%     as a matrix of two rows or two columns, giving for each node the zero,
%     one or two offsprings (children) of that node.
%   Note
%   See also
%     help IWT2Gpacket (inverse transform)
%     help FWT2G1D

if isnumeric(tree),
   sizetree = size(tree);
   if min(sizetree)==1,
      if length(tree)==1, 
         % tree is just a scalar; this is then interpreted as the number of
         % levels in a standard 2nd generation wavelet transform.
         M = tree; tree = 2.^(0:M-1);
      end
      tree = sort(tree); nl = tree(end); active = zeros(1,nl); 
      % nl: number of leaves
      % find parent of each active node in the tree. Active nodes are nodes
      % that appear in the tree and whose parent is active as well.
      % The variable active is zero if the node is non-active and it has the
      % index into the new tree if it is active.
      if tree(1) == 1, parent = 0; detail = 0; active(1) = 1; ll = 1; end
      for i=2:length(tree), l = tree(i);
         parentl = floor(l/2); d = l-2*parentl;
         parentll = active(parentl);
         if parentll>0,
            ll = ll+1; active(l) = ll; parent = [parent; parentll];
            detail = [detail; d];
         end
      end
      children = [];
   elseif min(sizetree)==2,
      if size(tree,2)>2, tree=tree'; end
      if tree(1,1)==0, 
         parent = tree(:,1); detail = tree(:,2); children = [];
      else,
         children = tree;
      end
   end
elseif iscell(tree),
   parent = tree{1}; detail = tree{2}; children = [];
end

if isempty(children),
   M = length(parent);
   children = ones(M,2)*(M+1); % sentinel child M+1
   d0 = find(detail==0); d0 = d0(d0>1);
   d1 = find(detail==1); d1 = d1(d1>1);
   children(parent(d0),1) = d0;
   children(parent(d1),2) = d1;
end

w = y; ny = size(y,1); C = size(y,2);
f = cell(M+1,1); v = cell(M+1,1);
f{1} = (1:ny); v{1} = (1:ny);
for m = 1:M,
   fm = f{m}; vm = v{m}; nf = length(fm);
   w(vm,1:C) = FWT2G1D(x(fm),w(vm,1:C),nt,n,1,varargin{:});
   [e o] = splitevenodd(fm); ne = length(e); em = fm(e); om = fm(o);
   f{children(m,1)} = em; f{children(m,2)} = om;
   v{children(m,1)} = vm(1:ne); v{children(m,2)} = vm(ne+1:nf);
end

