
function y = IWT2Gpacket(x,w,nt,n,tree,varargin);

%  IWT2GPpacket -- 2nd gen. frwd. wav. packet transf.; 
%   Usage
%     y = IWT2Gpacket(x,w,nt,n,M,tree,<arguments>);
%   Inputs
%     x      grid-locations of observations (time, space)
%     w      wavelet packet coefficients
%     nt     dual vanishing moments -- polynomial prediction order
%     n      integer vector; # free parameters in final update (see note (1))
%     tree   binary tree of subbands to be decomposed (several data
%            structures are possible, see help FWT2Gpacket)
%     'spline'   invokes spline wavelet tranform
%     'DD','Deslauriers','PpolintU'   invokes Deslauriers-Dubuc transform
%   Outputs
%     y       vector of size(w) with signal reconstruction
%   Description
%   Note
%   See also
%     help FWT2Gpacket (forward transform)
%     help IWT2G1D

if isnumeric(tree),
   sizetree = size(tree);
   if min(sizetree)==1,
      if length(tree)==1, 
         % tree is just a scalar; this is then interpreted as the number of
         % levels in a standard 2nd generation wavelet transform.
         M = tree; tree = 2.^(0:M-1);
      end
      tree = sort(tree); nl = tree(end); active = zeros(1,nl); 
      % nl: number of leaves
      % find parent of each active node in the tree. Active nodes are nodes
      % that appear in the tree and whose parent is active as well.
      % The variable active is zero if the node is non-active and it has the
      % index into the new tree if it is active.
      if tree(1) == 1, parent = 0; detail = 0; active(1) = 1; ll = 1; end
      for i=2:length(tree), l = tree(i);
         parentl = floor(l/2); d = l-2*parentl;
         parentll = active(parentl);
         if parentll>0,
            ll = ll+1; active(l) = ll; parent = [parent; parentll];
            detail = [detail; d];
         end
      end
      children = [];
   elseif min(sizetree)==2,
      if size(tree,2)>2, tree=tree'; end
      if tree(1,1)==0, 
         parent = tree(:,1); detail = tree(:,2); children = [];
      else,
         children = tree;
      end
   end
elseif iscell(tree),
   parent = tree{1}; detail = tree{2}; children = [];
end

if isempty(children),
   M = length(parent);
   children = ones(M,2)*(M+1); % sentinel child M+1
   d0 = find(detail==0); d0 = d0(d0>1);
   d1 = find(detail==1); d1 = d1(d1>1);
   children(parent(d0),1) = d0;
   children(parent(d1),2) = d1;
end

nw = size(w,1);
f = cell(M+1,1); v = cell(M+1,1);
f{1} = (1:nw); v{1} = (1:nw);
for m = 1:M,
   fm = f{m}; vm = v{m}; nf = length(fm);
   [e o] = splitevenodd(fm); ne = length(e); em = fm(e); om = fm(o);
   f{children(m,1)} = em; f{children(m,2)} = om;
   v{children(m,1)} = vm(1:ne); v{children(m,2)} = vm(ne+1:nf);
end

y = w; C = size(w,2);
for m = M:-1:1,
   fm = f{m}; vm = v{m};
   y(vm,1:C) = IWT2G1D(x(fm),y(vm,1:C),nt,n,1,varargin{:});
end

