function w = RWT_2GPpolintU(x,y,nt,n,M,varargin);

%  RWT_2GpolintU -- 1D, 2nd generation, non-decimated frwd. wav. transf.; 
%                   polynomial interpol. prediction; prediction first
%   Usage
%     w       = RWT_2GpolintU(x,y,nt,n,M,<other arguments>);
%   Inputs
%     x      grid-locations of observations (time, space)
%     y      fine scaling coefficients
%     nt     dual vanishing moments -- polynomial prediction order
%     n      primal vanishing moments -- number of taps in update filter
%     M      number of resulution levels
%   Optional inputs
%     'outputvector'  reshapes the output as a single vector or a single matrix
%   Outputs
%     w      by default:
%             N x (M+1) or (M+1) x N matrix, where N=length(y) 
%             containing non-decimated wavelet coefficients
%             if y is a row-vector, then w has M+1 rows of length N
%             if y is a column vector, then w has M+1 columns of length N
%             if y is a matrix, then w is a cell array of M+1 matrices with
%                                    size equal to that of y
%            if 'outputvector'
%             row or column vector of length N*(M+1)
%             matrix of size N*(M+1) x K, where [N K] is size of input
%   Description
%     Non decimated wavelet transform, type Deslauriers-Dubuc on
%     irregular point sets
%     Applies repeated lifting filter bank with one prediction step and one
%     update step. The prediction is based on polynomial interpolation, the
%     update uses n taps (i.e., n nonzeros, i.e., n degrees of freedom)
%   Note
%   See also
%     help FWT_2GPpolintU (fast, decimated alternative)
%     help IRT_2GPpolintU (reconstruction, i.e., inverse transform)

% Parsing varargin
nvarargin = length(varargin);
listofoptions = {};
addoutputinfo = false;
vectoroutput = false;
for k = 1:nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'addoutputinfo'},
     addoutputinfo = true;
   case {'vector','vectorout','vectoroutput','outvector','outputvector'},
     vectoroutput = true;
   case {'matrix','matrixout','matrixoutput','outmatrix','outputmatrix'},
     vectoroutput = true;
   case {'todo'}
     warning(['no implementation for ' vark])
   otherwise
     warning(['no implementation for ' vark])
end, end, end

transposex = false;
if size(x,2) > size(x,1),
   x = x'; transposex = true;
end
transposey = false;
if size(y,1) ~= size(x,1),
   y = y'; transposey = true;
end

if n > 0,
   Mj = initmom(x,(0:n-1));
end

if addoutputinfo == true,
   finalupdates = cell(1,M);
end

N = length(x);
w = cell(1,M+1);
for j = 1:M
%%%%%%%  scale = j
  skip = 2^(j-1);
  wj = zeros(size(y));
  if addoutputinfo == true,
     finalupdatesj = cell(1,2*skip);
  end
  for k=1:skip,
     xk = x(k:skip:N);
     yk = y(k:skip:N,:);
     if n>0, Mf = Mj(k:skip:N,1:n); end
     [e o] = splitevenodd(xk);
     for eo=0:1,
        yo = yk(o,:);
        ye = yk(e,:);
        xo = xk(o);
        xe = xk(e);
        Ne = length(e); No = length(o);
        if n>0,
           Me = Mf(e,1:n); Mo = Mf(o,1:n);
        end
        % PREDICTION MATRIX %
        I = sparse(eye(Ne));
        P = sparse(interpolynomialprediction(xe,I,xo,nt));
        % PREDICTION %
        yo = yo - P*ye;
        % MOMENTS %
        if n > 0,
           Me = Me + P'*Mo; 
        end
        % UPDATE for primal vanishing moments (wavelet functions) %
        if n > 0,
           xo = xk(o); xe = xk(e); Ne = length(e); No = length(o);
           Io = eye(No);
           U = updateprimalmoments(xo,Io,xe,Mo,Me);
           ye = ye + U*yo;
           Mj(k+skip*eo:2*skip:N,1:n) = Me;
        else,
           U = 0;
        end
        % KEEP INFO FROM CURRENT SCALE AND SHIFT %
        if addoutputinfo == true,
           finalupdatesj{k+skip*eo} = U;
           % finalupdatesj{2*k+eo+1} = U;
        end
        % FILLING IN %
        y(k+skip*eo:2*skip:N,:) = ye;
        wj(k+skip*(1-eo):2*skip:N,:) = yo;
        % PREPARE SECOND PART OF NONDECIMATED TRANSFORM (cycle spin)
        % switch roles even-odd
        ecopy = e; e = o; o = ecopy;
     end % for eo=1:2
  end
  % KEEP INFO FROM THIS SCALE %
  if addoutputinfo == true,
     finalupdates{j} = finalupdatesj;
  end
  % FILLING IN %
  w{j} = wj;
end
w{M+1} = y;

if size(y,2) == 1,
   wmatrix = zeros(N,M+1);
   for j = 1:M+1, wmatrix(1:N,j) = w{j}; end
   if vectoroutput, wmatrix = reshape(wmatrix,N*(M+1),1); end
   if transposey, w = wmatrix'; else, w = wmatrix; end
elseif vectoroutput,
   K = size(y,2);
   wmatrix = zeros(N*(M+1),K); k = 0;
   for j = 1:M+1, wmatrix(k+1:k+N,1:K) = w{j}; k = k+N; end
   if transposey, w = wmatrix'; else, w = wmatrix; end
end
if addoutputinfo == true,
   outputinfo = cell(1);
   outputinfo{1} = finalupdates;
   switch nargout,
   case 1
      w = {outputinfo{1},w};
   end
end
