
function P = averinterpolynomialprediction(xe,ye,xo,nt)

% averinterpolynomialprediction - ThreshLab2/Irregular/1D/ --- 
%                                 average interpolating prediction
%   Usage
%      yo = averinterpolynomialprediction(xe,ye,xo,nt);
%      P  = averinterpolynomialprediction(xe,I,xo,nt);
%   Inputs
%      xe  x-coordinates of even indexed locations
%      ye  y-values in even indexed locations
%      I   matrix (e.g., identity matrix of size = length(xe) )
%      xo  x-coordinates of odd indexed locations
%      nt  number of dual vanishing moments (= order of polynomial
%          prediction, i.e.: 2 = linear; 4 = cubic, etc.)
%   Outputs
%      yo  predicted values in odd indexed locations
%      P   for I = eye(length(xe)), P is the prediction operation matrix
%   Description
%      If ye is a matrix (e.g., ye=I), then polynomial prediction is
%      performed on all ROWS of ye if xe is a row vector
%                       COLUMNS of ye if xe is a column vector
%   See Also
%      help interpolationnevillederivative
%      help interpolynomialprediction

nxe = length(xe);
nxo = length(xo);
[xo oo] = sort(xo);
% we assume xe and xo are row vectors. If xo turns out to be column
% vector, then a final tranpose is applied to P
Ptranspose = false;
if size(xe,1) > 1, xe = xe'; Ptranspose = true; end
if size(xo,1) > 1, xo = xo'; Ptranspose = true; end
[ry nye] = size(ye);
if ~ismember(nye,[nxe nxe-1]), 
   ye = ye';
   [ry nye] = size(ye);
   if ~ismember(nye,[nxe nxe-1]), 
      error('dimension mismatch inputs xe & ye'),
   end
end
% if nye==nxe, xe = [xe xo(nxo)]; end
if nye==nxe,
   % add artificial even knot beyond last knot
   xee = max(2*xo(nxo)-xe(nxe),2*xe(nxe)-xo(nxo));
   xe = [xe xee];
end

nt = nt(1);

if nt==0, P = zeros(nxo,nxe);
else,

   Deltaxe = ones(ry,1)*(xe(2:nye+1)-xe(1:nye));
   yye = [zeros(ry,1) cumsum(Deltaxe.*ye,2)];
   % Explanation: why is there zeros(ry,1) in front?
   %    We look for the polynomial p(x) so that 
   %    int_xe(k)^xe(k+1) p(x) dx = ye(k)*Deltaxe(k)
   %    Let pp(x) be an antiderivative of p(x), then we need
   %    pp(xe(k+1)) - pp(xe(k)) = ye(k)*Deltaxe(k)
   %    Let pp(xe(k)) = 0 (integration constant), then the condition becomes
   %    pp(xe(k+1)) = ye(k)*Deltaxe(k)
   %    pp is denoted by yye in this routine. 

   lefteven = closestleftinvector(xo,xe,'strict');
   Deltaxo = xe(lefteven+1) - xo;

   % flip coordinates. This is a matter of convention: For odd values of nt+1,
   % interpolynomialprediction places ceil((nt+1)/2) evens on the left and
   % floor((nt+1)/2) on the right of x(odd). For averinterpolynomialprediction,
   % we prefer one even point less on the left, and one more on the right (this
   % is because average interpolation looks at averages over [x(k),x(k+1)],
   % which means that this approach wants to look more to the right than an
   % interpolating procedure around x(k) only)

   xe1 = -xe(nye+1:-1:1);
   xo1 = -xo(nxo:-1:1);
   yye1 = yye(1:ry,nye+1:-1:1);
   P = interpolynomialprediction(xe1,yye1,xo1,nt+1);
   P = P(1:ry,nxo:-1:1);

   P = yye(1:ry,lefteven+1)-P;

   Deltaxo(abs(Deltaxo)<eps) = 1;
   P = P/diag(Deltaxo);

   P(:,oo) = P;
end
if Ptranspose==true, P = P'; end
