
% The experiments in this file illustrate the function finescaleBsplinecoefs.m
% The results are explained by the Section 
% ``The experimental approximation error of a spline wavelet decomposition''
% in the paper on ``Non-equispaced B-spline wavelets''
n = 1000;
knots = makegrid(n,1)';
ptilde = 4;

xJ = knots;
y = 1+xJ.^3;

% extend set of knots:
xx = [0; xJ; 0];
xx(1) = xJ(1)-(xJ(n)-xJ(1))/n;
xx(n+2) = xJ(n)+(xJ(n)-xJ(1))/n;
% values of y in extension set are arbitrary
yy = [0; y; 0];

PhiJ = Bsplinebasis(xJ,xx,ptilde,false);

% (1) take observations as fine scaling coefficients
sJ1 = yy;
% plot the errors:
plot(xJ,(PhiJ*sJ1-y)./y)
axis([0 1 -0.001 0.001])
% The errors are of the order 10^(-3)

% (2) find the solution for PhiJ*sJ = y, that minimizes norm(sJ(2:n+1)-y)
% (default setting of finescaleBsplinecoefs.m)
sJ2 = finescaleBsplinecoefs(xx,yy,ptilde);
plot(xJ,PhiJ*sJ2-y)
% The solution is exact

% (3) alternative: find the solution so that the derivative of PhiJ(x)*sJ has 
% the minimum norm
sJ3 = finescaleBsplinecoefs(xx,yy,ptilde,'derivative');
plot(xJ,PhiJ*sJ3-y)
% again an exact solution

% (4) Now consider data with jumps
f = makesignal(xJ,'heavisine'); yy = [0; f; 0];
sJ1 = yy;
plot(xJ,PhiJ*sJ1-f)
sJ2 = finescaleBsplinecoefs(xx,yy,ptilde);
plot(xJ,PhiJ*sJ2-f)
% The solution is still exact, but...
plot(sJ2)
% ... near the jumps, we see that the coefficients fluctuate more than the
% function itself. These fluctuations create additional, large wavelet
% coefficients
% Choose the following parameters for the wavelet transform
nu = 3; primalmoments = 1; n_levels = 5;
w1 = FWT2G1D(xx,sJ1,'Bspline',ptilde,[primalmoments nu-primalmoments],n_levels);
w2 = FWT2G1D(xx,sJ2,'Bspline',ptilde,[primalmoments nu-primalmoments],n_levels);
% One can check:
plot(w1,'r')
hold on
plot(w2)
hold off
% that w2 has more large coefficients than w1, but the coefficients in w1 are
% never 0...
%
% The approximation error may propagate through a threshold scheme
% The following effect occurs for small thresholds. The threshold should not
% remove any information about features in f. The effect may get hidden in a
% threshold scheme for noise reduction
smallthr = 0.05;
w1thr = HT(w1,smallthr);
w2thr = HT(w2,smallthr);
sJ1thr = IWT2G1D(xx,w1thr,'Bspline',ptilde,[primalmoments nu-primalmoments],n_levels);
sJ2thr = IWT2G1D(xx,w2thr,'Bspline',ptilde,[primalmoments nu-primalmoments],n_levels);
plot(xJ,PhiJ*sJ1-f)
hold on
plot(xJ,PhiJ*sJ1thr-f,'r','linewidth',2)
plot(xJ,PhiJ*sJ2thr-f,'k','linewidth',2)
hold off
legend('error \Phi_J*c-f for c=f','error after HT(wav.coef.)',...
       'same for c with \Phi_J*c=f')
aadetail = [0.1 0.25 -0.05 0.05];
axis(aadetail)


% (5) The case with errors/noise
f = makesignal(xJ,'heavisine');
signal2noise = 10;
y = makenoisy(f,signal2noise); yy = [0; y; 0];
sJ1 = yy;
sJ2 = finescaleBsplinecoefs(xx,yy,ptilde);
plot(sJ2)
% while y was homoscedastic, sJ2 is nothing of that kind. It is impossible to
% use sJ2 as a starting point for a wavelet noise reduction scheme. 
% Even the exact solution that minimizes the variance:
sJ2opt = finescaleBsplinecoefs(xx,yy,ptilde,'pinv');
% is not suitable for use in wavelet denoising
plot(sJ2opt)
% Therefore, we use an approximative regularization:
% choose between:
regmethod = {'Landweber'};
% and
alfa = 0.2; regmethod = {'Tichonov',alfa};
sJ3 = finescaleBsplinecoefs(xx,yy,ptilde,regmethod{:});
plot(sJ3)
% The result is better now (although not perfect for use in wavelet denoising),
% but Landweber iterations do not guarantee an exact or close solution:
plot(xJ,PhiJ*sJ3-y)
% Moreover, going back to a polynomial 
y = xJ.^3; yy = [0; y; 0];
sJ2 = finescaleBsplinecoefs(xx,yy,ptilde);
sJ3 = finescaleBsplinecoefs(xx,yy,ptilde,regmethod{:});
plot(xJ,PhiJ*sJ3-y)
% the result is NOT an exact solution, where as sJ2 was an exact solution; and
% moreover:
w2 = FWT2G1D(xx,sJ2,'Bspline',ptilde,[primalmoments nu-primalmoments],n_levels);
w3 = FWT2G1D(xx,sJ3,'Bspline',ptilde,[primalmoments nu-primalmoments],n_levels);
plot(w3)
hold on
plot(w2,'r')
hold off
A = 1.e-2;
axis([0 n -A A])
% the wavelet transform of a polynomial ends up having no real zeros: the dual
% vanishing moments do not have the effect they have been made for.
% So, we need a good combination of sJ2 (exact solution -> polynomials with
% exact zero wavelet coefficients) and sJ3 (robust for noise)

% Going back to the case with noise:
f = makesignal(xJ,'heavisine');
signal2noise = 10;
y = makenoisy(f,signal2noise); yy = [0; y; 0];
sJ1 = yy;
sJ2 = finescaleBsplinecoefs(xx,yy,ptilde);
sJ3 = finescaleBsplinecoefs(xx,yy,ptilde,regmethod{:});
sJ4 = finescaleBsplinecoefs(xx,yy,ptilde,'powerLS');
SJ = finescaleBsplinecoefs(xx,eye(n),ptilde,'powerLS');


textpar = {'fontsize',14,'FontWeight','b'};
figure(1)
plot(xJ,y)
hold on
plot(xJ,PhiJ*SJ*y,'color',[0 1 1])
plot(xJ,f,'k','linewidth',4)
plot(xJ,PhiJ*SJ*f,'r','linewidth',2)
hold off
aa = axis; aa(2) = 1.1; axis(aa), axis off
yticks = get(gca,'ytick');
drawaxes('xtick',[0.5 1],'ytick',yticks)
set(findobj(gcf,'type','text'),textpar{:})
title('reconstruction',textpar{:})
legend('Y = f+noise','\Phi_J*S_J*y','f','\Phi_J*S_J*f')

bias = PhiJ*SJ*f-f;
figure(2)
plot(xJ,bias)
aa = axis; aa(2) = 1.1; axis(aa), axis off
yticks = min(abs(aa(3:4)))*[-1 1]*0.5;
drawaxes('xtick',[0 1],'ytick',yticks)
set(findobj(gcf,'type','text'),textpar{:})
title('The bias',textpar{:})

figure(3)
plot(SJ*(y-f))
axis off
yticks = get(gca,'ytick');
xticklabel = {0,'n+2'};
drawaxes('xtick',[0 n+2],'ytick',yticks,'xticklabel',xticklabel)
set(findobj(gcf,'type','text'),textpar{:})
title('The noise on the coefficients',textpar{:})
% met QQ-plot
q = (1:n+2)/(n+3);
QZ = invcumgauss(q);
sortcoefnoise = sort(SJ*(y-f));
stdevcoefnoise = MAD1(sortcoefnoise)/invcumgauss(0.75);
q75 = ceil(0.75*n);
q25 = floor(0.25*n);
plot(QZ,sortcoefnoise,'.')
hold on
plot(QZ,stdevcoefnoise*QZ,'r')
plot(QZ(q75),sortcoefnoise(q75),'r*')
plot(QZ(q25),sortcoefnoise(q25),'r*')
hold off
axis([-4 4 -6*stdevcoefnoise 6*stdevcoefnoise])
aa = axis; K = floor(aa(4)); k = max(1,floor(K/4)); K = floor(K/k)*k;
axis off
drawaxes('xtick',(-4:4),'ytick',(-K:k:K))
set(findobj(gcf,'type','text'),textpar{:})
title('The noise on the coefficients (QQ plot)',textpar{:})
