
function UP = liftingstepsfromrefinement(H);

% liftingstepsfromrefinement - ThreshLab2/Irregular/1D/ --- 
%                  factoring of refinement matrix into lifting steps
% Usage
%   UP = liftingstepsfromrefinement(H);
%   UP = liftingstepsfromrefinement(H,e);
% Input
%   H     matrix in refinement (2-scale) equation: PhiJ*H = PhiL
%         H must be of size n x ne, where ne = ceil(n/2);
%   e     index into even subset
% Output
%   UP    cell array of matrices, with lifting steps ordered for application
%         from right to left (matrix product) in the refinement
%   G     matrix in wavelet equation:              PhiJ*G = PsiL
% Description
%   PhiJ   is the fine scaling basis
%   PhiL   is the coarse scaling basis
% Notes
%   (1) This routine asumes a partitioning into alternating evens and odds. 
%       This means: critical downsampling, no overlap, 
%                   no nondecimated transforms, no cycle-spinning, no
%                   redundancy
%   (2) As the factoring into lifting steps is not unique, the sequence
%           H = refinementfromliftingsteps(UP1);
%           UP2 = liftingstepsfromrefinement(H);
%       may lead to a result UP2 =/= UP1
%   (3) It should hold that H1==H2 in the following scheme
%           UP = liftingstepsfromrefinement(H1);
%           H2 = refinementfromliftingsteps(UP);
%       As yet, this routine is not always able to find a correct factoring.
%       Problems occur when elements within the bandwidth of the matrix H are
%       unexpectedly zero. (This is work in progress)
% See also
%   help refinementfromliftingsteps

[n, ne] = size(H);
if ne>n, H=H'; nee = ne; ne = n; n = nee; end
no = n-ne;
if ceil(n/2)~=ne,
  error('refinement matrix has incorrect dimension for factoring into lifting')
end
e = (1:2:n); o = (1:n); o(e) = 0; o = find(o);

Ho = H(o,1:ne); He = H(e,1:ne);
% find bandwidths; beu = bandwidth in He, upper side
%                  bol = bandwidth in Ho, lower side and so on
beu = 0; bel = 0; bou = 0; bol = 0;
for d = 0:ne, if any(diag(He,d)), beu = d; end, end
for d = 1:ne, if any(diag(He,-d)), bel = d; end, end
be = beu+bel+1;
for d = 0:ne, if any(diag(Ho,d)), bou = d; end, end
for d = 1:no, if any(diag(Ho,-d)), bol = d; end, end
bo = bou+bol+1;
if be==bo, 
   if rem(be,2)==0, beu = beu+1; be = be+1; 
   else, bou = bou+1; bo = bo+1;
   end
end
b = max(be,bo);

UP = {};
while b>2,
   if be==b,
      U = zeros(ne,no);
      Ud = -diag(He(1:no,1:ne),beu)./diag(Ho,bou);
      Ud(isnan(Ud))=0; % entries in U that are NaN correspond to 0/0
                       % => corresponding entry in He is already zero
                       % No need for prediction coefficient there.
      d = length(Ud);
      U(1:d,1:d) = U(1:d,1:d)+diag(Ud);
      % consider the submatrices He(2:ne,1:ne) and Ho(1:ne-1,1:ne)
      % diagonal -bel in He becomes -bel+1 in He(2:ne,1:ne)
      Ul = -diag(He(2:ne,1:ne),-bel+1)./diag(Ho(1:ne-1,1:ne),-bol);
      Ul(isnan(Ul))=0; % entries in U that are NaN correspond to 0/0
                       % => corresponding entry in He is already zero
                       % No need for prediction coefficient there.
      l = length(Ul);
      ll = (ne-l:ne-1); U(ll+1,ll) = U(ll+1,ll)+diag(Ul);
      UP = {UP{:},U};
      He = He+U*Ho;
      beu=beu-1; bel=bel-1; be=be-2;
   else,
      P = zeros(no,ne);
      Pd = diag(Ho,-bol)./diag(He(1:no,1:ne),-bel);
      Pd(isnan(Pd))=0; % entries in P that are NaN correspond to 0/0
                       % => corresponding entry in Ho is already zero
                       % No need for prediction coefficient there.
      d = no-length(Pd)+1;
      P(d:no,d:no) = diag(Pd);
      % diagonal beu in He corresponds to diagonal beu+1 in He(2:ne,1:ne)
      Pu = diag(Ho(1:ne-1,1:ne),bou)./diag(He(2:ne,1:ne),beu+1);
      Pu(isnan(Pu))=0; % entries in P that are NaN correspond to 0/0
                       % => corresponding entry in Ho is already zero
                       % No need for prediction coefficient there.
      u = length(Pu);
      uu = (1:u); P(uu,uu+1) = P(uu,uu+1)+diag(Pu);
      UP = {UP{:},P};
      Ho = Ho-P*He;
      bou=bou-1; bol=bol-1; bo=bo-2;
   end
   b = max(be,bo);
end
P = Ho;
UP = {UP{:},P};
D = He;
UP = {UP{:},D};
