
function [Omega B] = aggregatesmoothset(OmegaL,OmegaR);

% aggregatesmoothset - ThreshLab2/Irregular/BrokenbasisWT/ --- 
%                   assembling sets of locally defined smooth functions
%                   into one set of globally defined smooth functions
% Usage
%    [Omega B] = aggregatesmoothset(OmegaL,OmegaR);
%    [Omega B] = aggregatesmoothset(Omega);
%    [Omega B] = aggregatesmoothset({OmegaL,OmegaR});
%  Input
%    OmegaL real matrix of size nutilde x nutilde or [(n-1)*nutilde] x nutilde
%    OmegaR real matrix of size [(n-1)*nutilde] x nutilde
%  Output
%    Omega real matrix, of size (n*nutilde) x nutilde
%    B     cell array of length n-1, containing nutilde x nutilde matrices
%  Description
%    Let knots be an ordered n-tuple of knot values (the actual values are not
%    needed for the calculations in this function)
%    Then OmegaR(k*nutilde+1:(k+1)*nutilde,q) contains the value and the
%    derivatives (1st to (nutilde-1)st) in knots(k+1) of the q-th (q=1:nutilde)
%    smooth function defined on the segment
%    [knots(k) knots(k+1)].
%        Denote ORk = OmegaR(k*nutilde+1:(k+1)*nutilde,1:nutilde)
%    The value and derivates of that same function in knots(k) appear in
%       OmegaL(1:nutilde,q) (if size(OmegaL) is nutilde x nutilde) or in
%       OmegaL(k*nutilde+1:(k+1)*nutilde,q).
%    Then, for these two possibilities
%       Denote OLk = OmegaL
%           or OLk = OmegaL(k*nutilde+1:(k+1)*nutilde,1:nutilde)
%    Let Omega_k(x) = [... Omega_k,q(x)... ] denote the row vector of the
%    nutilde local functions satisfying the boundary conditions above.
%    Then define the functions Omega2_k(x) by
%    Omega2_1(x) = Omega_1(x)
%    Omega2_k(x) = Omega2_{k-1}(x)*inv(OLk)*ORk,
%    then taking Omega2 to be Omega2_k(x) on [knots(k) knots(k+1)] defines a
%    set of nutilde functions with continuous derivatives 0,1,...,nutilde
%    The output Omega(l:nutilde:end,q) then contains the l-th derivative of the 
%    q-th function in the knots 

if nargin<2,
   if iscell(OmegaL),
      OmegaR=OmegaL{2}; OmegaL=OmegaL{1}; 
   else
      OmegaR = OmegaL; nutilde = size(OmegaR,2);
      OmegaL = diag(factorial(0:nutilde-1)); % default values (corresponding to
                                             % power functions)
   end
end
nutilde = size(OmegaR,2);
nnutilde = size(OmegaR,1)+nutilde;
n = floor(nnutilde/nutilde);
if n*nutilde ~= nnutilde,
   error('incompatible sizes input variable OmegaR; #rows must be k*#columns')
end
if size(OmegaL,1)==nutilde, oneL=true;
elseif size(OmegaL,1)==nnutilde-nutilde, oneL=false;
else, error('incompatible sizes input variable OmegaL')
end
r0 = (1:nutilde); 
l0 = (1:nutilde);l1=l0;
Omega = zeros(nnutilde,nutilde);
Omega(1:nutilde,1:nutilde)=OmegaL(l0,1:nutilde);
Omega(nutilde+1:nnutilde,1:nutilde)=OmegaR;
C = cell(1,n); C{1} = eye(nutilde); C{n} = eye(nutilde);
for k=2:n-1,
   r1=r0+nutilde; if oneL==false, l1=r1; end 
   C{k}=OmegaL(l1,1:nutilde)\OmegaR(r0,1:nutilde);
   r0=r1;
end
% C is correction in each knot for jumping from OmegaR(left interval) onto
% OmegaL(right interval) (No correction is needed in the end points)
% B is the accumulated correction in each segment's right end
B=cell(1,n-1); B{1}=C{1}; for s=2:n-1, B{s}=B{s-1}*C{s}; end
r0 = (1:nutilde);
for k=2:n,
   r1=r0+nutilde; Omega(r1,1:nutilde) = OmegaR(r0,1:nutilde)*B{k-1}; r0=r1; 
end
