function [PhiBB aa Dknots] = brokenbasis(x,knots,Omega,varargin);

% brokenbasis - ThreshLab2/Irregular/BrokenbasisWT/ --- construct basis of
%                           piecewise elements from given smooth functions
% Usage
%    [PhiBB aa Dknots] = brokenbasis(x,knots,Omega);
%    [PhiBB aa Dknots] = brokenbasis(x,knots,Omega,nutilde);
%    [PhiBB aa Dknots] = brokenbasis(x,knots,nutilde);
%    aa = brokenbasis(Omega);
%    
% Inputs
%    x       points in which splines are evaluated, vector of length m
%    knots   knots of the spline basis, vector of length n
%    Omega   cell array or matrix or integer defining the set of smooth
%            functions from which segments will be used
%            Any form that can be understood by smoothsetmatrix for
%            preprocessing will work.
%    nutilde integer; number of smooth functions used in the construction of
%            the broken basis; (nutilde-1) is number of continuity conditions
%            in the knots
%            in case of B-splines constructed from power functions: nutilde-1 
%            is degree of polynomials on each segment
% Outputs
%     PhiBB  matrix of size m x (n+nutilde-2), each column having the
%            evaluations of a basis function phi_k(x)
%     aa     cell of length n+nutilde-2, each of its elements being a matrix of
%            size (nutilde x nutilde) so that
%            phi_k(x) = sum_l aa{k}_(l,s) * Omega_l(x)
%            if x is in [x_s, x_{s+1}]
%     Dknots matrix of size (2n x n+nutilde-2)
%            Dknots(2l-1,k) has the left limit of the (nutilde-1)th derivative
%                           of phi_k in x_l
%            Dknots(2l,k) has the right limit
%            These values can be used in the construction of a refinement
%            scheme (see help refinementbrokenbasis)
% Description
%     constructs basis of piecewise linear combination of given smooth 
%     basis with support no wider than nutilde intraknots intervals.
%     Starting from a power function basis, this is a B-spline basis
% See Also
%     help Bsplinebasis
%     help refinementbrokenbasis
%     help smoothsetmatrix
%     help aggregatesmoothset

nvarargin=length(varargin);
omegaoptions = {};
if nargin==1,
   Omega = 1; if iscell(x), Omega = x; elseif max(size(x))>1, Omega = {x}; end
   if iscell(Omega),
      Omega0 = Omega{1}; n = max(size(Omega0));
      knots = (0:n-1)/(n-1); x = knots;
   else
      knots = x;
   end
elseif nargin==2,
   Omega = knots;
   knots = x;
end
n = length(x); OmegaL=NaN; 
% agg = false;
% %%% agg seems to be defined later on - to be tested
boundaryonly = false;
% The variable agg will activate the call of the routine aggregatesmoothset.m
% for the assembly (aggregation) of the smooth functions from locally defined
% segments. (Hence, if agg==true, this means that the values in Omega are
% interpreted as locally defined segments, to be joint into a set of global
% smooth functions). Aggregation of locally defined segments has a better
% numerical condition than working with a globally defined smooth functions.
k = 0;
while k < nvarargin, k=k+1; vark = varargin{k}; 
   if ischar(vark), switch(vark)
   % case {'agg','local'},
   %    agg = true;
   %%%%% agg seems to be defined later on - to be tested
   %%%%% in case {'agg','local'}, it seems better to include 'agg' into Omega
   %%%%% for proper use in smoothsetmatrix
   case {'boundary'},
      boundaryonly = true;
   otherwise
      omegaoptions = {omegaoptions{:},vark};
   end
   else
      omegaoptions = {omegaoptions{:},vark};
end, end
% convert Omega into a matrix containing all values and derivatives
[OmegaL Omega] = smoothsetmatrix(knots,Omega);
% make sure knot values and x values are properly defined
[Omega Omegax] = knotxvalues(knots,x,Omega,OmegaL);
nnutilde = size(Omega,1); nutilde = size(Omega,2);
n = length(knots); m = length(x);
agg = ~isnan(OmegaL(1,1));

% Find the coefficients aa
aa = cell(1,n+nutilde-2);
ll = [ones(1,nutilde-1) (1:n-1)]; rr = [(2:n) n*ones(1,nutilde-1)];
bb = [ones(1,nutilde-1) zeros(1,n-nutilde) n*ones(1,nutilde-1)];
sumPhik = zeros(nutilde-1,n); % The i-th row, k-th column of sumPhik contains 
                              % the sum of the i-th derivatives of Phi in knot
                              % k
Phik = cell(1,n+nutilde-2); % The cell entry Phik{k} is a matrix whose i-th
                            % rows has the i-th derivatives of the k-th basis
                            % function in all the knots within its support
orderk = [(1:n-1) (n+nutilde-2:-1:n)];
if boundaryonly, orderk = [(1:nutilde-1) (n+nutilde-2:-1:n)]; end
for k=orderk, % loop over basis functions
   l = ll(k); r = rr(k); kk = (l:r); nk = (r-l)+1;
   kkk = (nutilde*(l-1)+1:nutilde*(r-agg));
   O = Omega(kkk,1:nutilde);
   if agg, OL = OmegaL(kkk,1:nutilde); [O B] = aggregatesmoothset(OL,O); end
   b = 1; if nk>2 & nk<nutilde+1, b = -sumPhik(nk-1,bb(k)); end
   % b is for temporary normalistion
   [a Phik{k}] = solvecoef(O,b,l+nutilde>n);
   sumPhik(1:nutilde-1,kk) = sumPhik(1:nutilde-1,kk)+Phik{k};
   if agg, for i=2:nk-1, a(1:nutilde,i) = B{i}*a(1:nutilde,i); end, end
   aa{k} = a;
end

% Final normalisation, imposing sum(Phi)=1 and sum(Phi derivatives)=0
% (the derivatives have to be controled in order to reduce numerical error
%  propagation)
if ~boundaryonly, for k=2:n-nutilde+1, % loop over knots
   b = [1; zeros(nutilde-2,1)]-sumPhik(1:nutilde-1,k);
   % b contains the residual to be corrected (i.e., the desired sumPhik(:k)
   % minus the actual value of sumPhik(:k))
   A = zeros(nutilde-1,nutilde-1);
   for i=1:nutilde-1,
      ki = k+i-1;
      Phiki = Phik{ki};
      A(1:nutilde-1,i) = Phiki(1:nutilde-1,k-ll(ki)+1);
   end
   c = A\(b+A*ones(nutilde-1,1));
   for i=1:nutilde-1,
      ki = k+i-1; ci = c(i); l = ll(ki); r = rr(ki); 
      sumPhik(1:nutilde-1,l:r) = sumPhik(1:nutilde-1,l:r)+Phik{ki}*(ci-1);
      Phik{ki} = Phik{ki}*ci;
      aa{ki} = aa{ki}*ci;
   end
end, end

% Find the broken basis functions
PhiBB = zeros(m,n+nutilde-2);
s0 = max(0,(1:n+nutilde-2)-nutilde);
if ~boundaryonly, for s = 1:n-1, % loop over interknot segments/intervals
   % segment s, between knots s and s+1, with nonzero values for basis
   % functions s,s+1,...,s+nutilde-1
   ii = find(x>=knots(s)&x<knots(s+1)); mii = length(ii);
   kk = (s:s+nutilde-1);
   As = zeros(nutilde,nutilde);
   for j=1:nutilde,
      k=kk(j); As(1:nutilde,j) = aa{k}(1:nutilde,s-s0(k));
   end
   PhiBB(ii,kk) = Omegax(ii,1:nutilde)*As;
end
else, for k=orderk, % loop over boundary basis functions only
   l = ll(k); r = rr(k); a = aa{k};
   for s=l:r-1,
      ii = find(x>=knots(s)&x<knots(s+1)); mii = length(ii);
      PhiBB(ii,k) = Omegax(ii,1:nutilde)*a(1:nutilde,s-l+1);
   end
end, end
% provide left continuity in right boundary
i = find(x==knots(n)); PhiBB(i,n+nutilde-2)=1;

% find the (nutilde-1)st derivatives
if ~boundaryonly,
if ~agg, OmegaL = Omega; Omega = Omega(nutilde+1:nnutilde,1:nutilde); end
Dknots = zeros(2*n,n+nutilde-2);
for k = 1:n+nutilde-2,
   l = ll(k); r = rr(k); a = aa{k}; kk = l*nutilde:nutilde:(r-1)*nutilde;
   Dknots(2*l:2:2*r-2,k) = sum(OmegaL(kk,1:nutilde).*a',2);
        % right limits at left end points
   Dknots(2*l+1:2:2*r-1,k) = sum(Omega(kk,1:nutilde).*a',2);
        % left limits at right end points
end
end % if ~boundaryonly,
% dx = initmom(x);
% I = PhiBB'*dx;
% PhiBB = PhiBB*diag(sign(I));
if nargin<3 & nargout==1,
   PhiBB = aa;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function A = knotvalues2continuityconditions(O)

% sets up the matrix for solving the continuity conditions in the knots
% By comparing the number of knots to the cardinality of the smooth function
% set, this functions knows whether it is in the middle or on the boundary.
% If a boundary is detected, it is supposed to be at the 1st knot in O

nutilde = size(O,2);
nk = floor(size(O,1)/nutilde);
A = zeros(nutilde*(nk-1),nutilde*(nk-1));
if nk==nutilde+1,
   r = floor(nk/2); rr = nutilde*r+(1:nutilde);
   if nutilde==1,rr=1; end
   A(nutilde^2,rr) = O(r*nutilde+1,1:nutilde);
end
r0 = min(nk,nutilde)-1; % number of continuity conditions in leftmost knot
rr = cumsum([0 r0 (nutilde-1)*ones(1,nk-1)]);
cc = (0:nk)*nutilde;
% rr and cc define the blocks; now fill in the blocks
for i=1:nk-1,
   r = rr(i)+1:rr(i+1); c = cc(i)+1:cc(i+1); nr = rr(i+1)-rr(i);
   A(r,c) = O(c(1:nr),1:nutilde);
   r = rr(i+1)+1:rr(i+2); c = cc(i)+1:cc(i+1); nr = rr(i+2)-rr(i+1);
   A(r,c) = -O(c(1:nr)+nutilde,1:nutilde);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function OO = inverseorderknots(O)

nutilde = size(O,2); nsnutilde = size(O,1); ns = floor(nsnutilde/nutilde);
ii = repmat((1:nutilde)',ns,1)+nutilde*floor((nsnutilde-1:-1:0)'/nutilde);
OO = O(ii,1:nutilde);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [a Phik] = solvecoef(O,b,right)

% compute coefficients a and knot values and derivatives Phik of the k-th
% broken basis function, using the knot values and derivatives O of the smooth
% basis functions. The scalar b (default 1) is for the normalisation. 

if nargin<3, right=false; end
if nargin<2, b = 1; end
nutilde = size(O,2);
if right, OO = inverseorderknots(O); else, OO = O; end
A = knotvalues2continuityconditions(OO);
nc = size(A,2); % nc = size(O,1)-nutilde;
nk = nc/nutilde+1;
e = zeros(nc,1); if nk<nutilde+1, d=nk-1; else, d=nutilde^2; end, e(d)=b; 
a = A\e; a = reshape(a,nutilde,nk-1); ii = (1:nk-1);
if right, a = a(1:nutilde,nk-1:-1:1); end % O = O(nutilde+1:nc,1:nutilde); end
Phik = zeros(nutilde-1,nk);
% last column will remain zero (first column if right is true)
for i=1:nk-1,
   j=i+right;
   r = nutilde*(j-1)+(1:nutilde-1);
   Phik(1:nutilde-1,j) = O(r,1:nutilde)*a(1:nutilde,i);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Omega Omegax] = knotxvalues(knots,x,Omega,OmegaL)

% Input: Omega(x) or Omega(knots)
% Output Omega(x) and Omega(knots)
m = length(x);
n = length(knots);
[nnutilde nutilde] = size(Omega);
% Following lines handle the case where OmegaL is supposed to be a constant,
% specified only at one knot. This specification is no longer in use.
% if isnan(OmegaL(1,1)), OmegaL = diag(factorial(0:nutilde-1)); end
% if size(OmegaL,1)==nutilde,
%    nO = nnutilde/nutilde; OmegaL = repmat(OmegaL,nO,1);
% end
if nnutilde==(n-1)*nutilde,
   Omegax = zeros(m,nutilde); 
   for s=1:n-1,
      ii = find(x<=knots(s+1) & x>=knots(s));
      kk = (s-1)*nutilde+(1:nutilde);
      for q=1:nutilde,
         Dfknotss = [OmegaL(kk,q) Omega(kk,q)];
         Omegax(ii,q) = hermitesplinesegment(x(ii),knots(s:s+1),Dfknotss);
      end
   end
elseif nnutilde==n*nutilde,
   Omegax = zeros(m,nutilde); 
   for s=1:n-1,
      ii = find(x<=knots(s+1) & x>=knots(s));
      kk = (s-1)*nutilde+(1:nutilde);
      for q=1:nutilde,
         Dfknotss = [Omega(kk,q) Omega(kk+nutilde,q)];
         Omegax(ii,q) = hermitesplinesegment(x(ii),knots(s:s+1),Dfknotss);
      end
   end
elseif nnutilde==m*nutilde,
   Omegax = Omega;
   Omega = zeros(n*nutilde,nutilde);
   for q=1:nutilde,
      Omegaqx = Omegax(1:nnutilde,q);
      Omega(1:n*nutilde,q) = hermitespline(knots,x,Omegaqx,0:nutilde-1);
   end
   Omegax = Omegax(1:nutilde:nnutilde,1:nutilde);
end
% if isnan(OmegaL(1,1)),

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
