function M = brokenbasismoments(Mom,aa,qq);

% brokenbasismoments - ThreshLab2/Irregular/BrokenbasisWT/ --- finds moments of
%                                     broken basis functions
% Usage
%    M = brokenbasismoments(Mom,aa);
%    M = brokenbasismoments(knots,Omega,qq);
%    M = brokenbasismoments(knots,nutilde,qq);
% Inputs
%    Mom     cell array of moments (length p) Mom{q} has the q-th order moment
%            (output of smoothsetmoments)
%    aa      cell array of coefficients per segment (length nn), output of
%                              [Phi aa LRknots] = brokenbasis(...)
%    knots   vector of length n; knots of the broken basis
%    Omega   cell array defining smooth basis from which broken basis is
%            constructed (for more info: help brokenbasis)
%    nutilde integer; # segments used in piecewise smooth basis functions
%    qq      integer vector of length pp (default is (1:p), i.e.,pp=p)
% Outputs
%    M       nn x pp matrix with qq-th moments of broken basis functions
% Note
%    Mom is supposed to contain
%    Mom{q} = int_knot0^knot1 Omega(x)*x^qq(q) dx
%    these values can be computed either numerically or analytically (if
%    possible)
%    the x^q can be replaced by any other function g_q(x) of interest. The
%    routine brokenbasismoments does not rely on g_q(x) being a polynomial or
%    anything else
% See also
%    help brokenbasis
%    help smoothsetmoments

if iscell(Mom),
   p = length(Mom); 
elseif min(size(Mom))>1,
   p = 1; Mom = {Mom}; 
else % Mom is actually knots; aa is assumed to specify Omega (no further check
     % is carried out here)
   % Use piecewise polynomial approximation
   knots = Mom;
   Omega = aa;
   aa = brokenbasis(knots,Omega);
   if nargin<3, qq = 0; end
   nn = length(aa); n = length(knots); nutilde = nn-n+2;
   Mom = smoothsetmoments(knots,Omega,qq);
   p = length(Mom);
end
nn = length(aa);
a = aa{1}; nutilde = size(a,1); n = nn-nutilde+2;
M = zeros(n,p);
for q=1:p, 
   Momq = Mom{q}; if size(Momq,1) ~= n-1, Momq=Momq'; end
   if any(sort(size(Momq))~=sort([nutilde n])),
      error('sizes inputs Mom and aa do not match')
   end
   if size(Momq,2)~=nutilde, Momq = Momq'; end
   Momq = Momq(2:n,1:nutilde);
   for k = 1:nn,
      l = max(1,k+1-nutilde); r = min(n-1,k);
      a = aa{k};
      M(k,q) = sum(sum(Momq(l:r,1:nutilde).*a'));
   end
end
