
function [phi aa] = hermitespline(x,knots,Dfknots,deriv)

% hermitespline - ThreshLab2/Irregular/BrokenbasisWT/ Hermite spline interpol.
% Usage
%   [phi aa] = hermitespline(x,knots,Dfknots);
%   phi = hermitespline(x,knots,Dfknots);
%   aa = hermitespline(knots,Dfknots);
% Input
%   x        vector of abscis values in which function is evaluated
%   knots    vector of n knots in which the interpolation is set up
%   Dfknots  (p+1) x n or n x (p+1) matrix or n*(p+1) vector containing
%            function values and derivatives (up to the p-th derivative)
%            in the knots
%   deriv    integer vector (default 0), adds derivatives to output
% Output
%   phi     vector of length(x), containing phi(x), so that
%           phi^(q)(x_i) = Dfknots(q+1,i)
%           (in case adderivative has more than [0], phi is matrix, containing
%            derivates and higher order derivatives as well)
%   aa      2*(p+1) x (n-1) or (n-1) x 2*(p+1) matrix 
%           containing power coefficients for each segment: 
%           phi(x) = a(s,1)+a(s,2)*(x-xs1)+a(s,3)*(x-xs1)^2+... 
%                  +a(s,p+1)*(x-xs1)^p*(xs2-x)+a(s,p+2)*(x-xs1)^p*(xs2-x)^2+...
%                on [xs1 xs2] = [knots(s) knots(s+1)]
% See also
%   help hermitesplinemoments

if nargin<4, deriv=0; end
nox = false; if nargin<3, Dfknots = knots; knots = x; nox = true; end
n = length(knots);
shapeasvector = false;
if min(size(Dfknots))==1,
   p = floor(length(Dfknots)/n)-1; Dfknots = Dfknots(1:(p+1)*n);
   Dfknots = reshape(Dfknots,p+1,n);
   shapeasvector = true;
end
transposea = false;
if size(Dfknots,2) ~= n, Dfknots=Dfknots'; transposea=true; end
if size(Dfknots,2) ~= n,
   error('incompatible dimensions in variables knots and Dfknots')
end
p = size(Dfknots,1)-1;
aa = zeros(2*(p+1),n-1);
if nox==false, 
   [x transposephi] = column(x);
   nd = length(deriv);
   m = length(x);
   phi = zeros(m,nd);
end
for s=1:n-1,
   a = hermitesplinesegment(knots(s:s+1),Dfknots(1:p+1,s:s+1));
   aa(1:2*(p+1),s) = a;
   if nox==false,
      x1 = knots(s); x2 = knots(s+1);
      ii = find(x<=x2 & x>=x1);
      X = ones(m,2*p+2);
      for q=1:p, X(1:m,q+1) = (x-x1).^q; end
      for q=0:p, X(1:m,p+2+q) = (x-x1).^(p+1).*(x-x2).^q; end
      D = 1;
      if deriv(1)>0|nd>1, 
         % define the derivative operator on the function set in X
         D = zeros(2*p+2,2*p+2); D(1:p+1,2:p+2) = diag(1:p+1);
         B = eye(p+1); B(1:p,2:p+1) = B(1:p,2:p+1)+(x2-x1)*eye(p); C = inv(B);
         D(p+1:2*p+1,p+3:2*p+2) = (p+1)*C(1:p+1,2:p+1);
         D(p+2:2*p+1,p+3:2*p+2) = D(p+2:2*p+1,p+3:2*p+2)+diag(1:p);
      end
      for i=1:length(deriv),
         q = deriv(i);
         phi(ii,i) = X(ii,1:2*p+2)*D^q*a;
      end
   end
end
if transposea, aa = aa'; end
if nox, phi=aa;
elseif shapeasvector,
   phi = reshape(phi',m*nd,1);
elseif transposephi,
   phi = phi';  
end
