
% This procedure contains a few examples illustrating the construction and use
% of broken bases in wavelet transforms. The procedure may be use to get
% started with FWT2G1D.m with option 'BB' and brokenbasis.m

% settings
fontsize = 14;
fontopt  = {'fontsize',fontsize,'fontweight','bold'};
m = 1000; x = makegrid(m,1); f = makesignal(x,'skyline');
chooseknotsasinbook
knotsf = knots7plus6; % fine scale knots
nf = length(knotsf);
knotsc = knotsf(1:2:nf); % coarse scale knots
nutilde = 4; % cardinality of set of smooth functions used in the construction
             % of the broken basis (parameter of the refinable basis)
nu = 2; % primal wavelet function parameter
M = 5; % number of resolution levels used in wavelet transforms

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% start by constructing a broken basis
[PhiBspline aa Dknots] = brokenbasis(x,knotsc,nutilde);
% using nutilde as input, the default set of smooth functions is used. This is
% the set of power functions {1,x,x^2,...,x^(nutilde-1)}, leading to the broken
% basis being a B-spline basis.
% The outputs aa and Dknots are optional and may be used in further
% construction. (see help brokenbasis for more details)
figure(1)
plot(x,PhiBspline,'linewidth',3)
axy = axis; axy(2) = 1.1; axis(axy); axis off;
drawaxes('xticks',[0 1],fontopt{:})
title('B-spline basis',fontopt{:})
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now define another set of smooth functions
Omega = combinebasesdiff(x,nutilde,[3 4],[2 3]);
% This creates a set of smooth functions and (nutilde-1) derivatives
% containing: Omega(x) = [1 x sin(2*pi*x) cos(2*pi*x)]
figure(2)
plot(x,Omega{1},'linewidth',3)
axy = axis; axy(2) = 1.1; axis(axy); axis off;
drawaxes('xticks',[0 1],fontopt{:})
% For use in brokenbasis, we have to define Omega in the knots
Omega = combinebasesdiff(knotsc,nutilde,[3 4],[2 3]);
% Omega is a cell array of size nutilde
% Omega{q+1} contains the q-th derivative
% The cell array can be converted into a matrix representation 
Omega = smoothsetmatrix(Omega);
% which is the internal working form for all routines (but they accept other
% forms as well, trying to convert them using smoothsetmatrix first before
% further processing)
% We now compare with B-splines
[PhiBB aa Dknots] = brokenbasis(x,knotsc,Omega);
figure(3)
plot(x,PhiBspline,'linewidth',3,'color',0.5*[1 1 1])
hold on
plot(x,PhiBB,'linewidth',3)
hold off
axy = axis; axy(2) = 1.1; axis(axy); axis off;
drawaxes('xticks',[0 1],fontopt{:})
title('Broken basis with sine and cosine functions, compared to B-splines',...
       fontopt{:})
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now, reconstruct smooth set as linear combinations of broken basis
A = broken2smooth(aa); 
% This uses the coefficients aa on each interval
% Equivalently, we may reconstruct Omega(x) = Phi(x)*A by specifying knots and
% Omega(knots):
A = broken2smooth(knotsc,Omega);
figure(4)
plot(x,PhiBB*A,'linewidth',3)
axy = axis; axy(2) = 1.1; axis(axy); axis off;
drawaxes('xticks',[0 1],fontopt{:})

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% We now go back to power functions as smooth set (hence B-splines as broken
% basis), this time on two nested sets of knots
Omegaf = powerdiff(knotsf,nutilde);
Omegac = powerdiff(knotsc,nutilde);
% We construct the corresponding broken basis at two resolution levels:
PhiBc = brokenbasis(x,knotsc,Omegac);
[PhiBf aaf LRknotsf] = brokenbasis(x,knotsf,Omegaf);
% The coarse scale basis can be found by subsampling the fine scale basis,
% using a two-scale or refinement equation PhiBf(x)*H = PhiBc(x), where H is
% found by:
[H o] = refinementbrokenbasis(LRknotsf,nutilde);
% refinementbrokenbasis uses optional output of brokenbasis.
% Just as in broken2smooth, this optional output can be replaced by the input
% specifications, so that refinementbrokenbasis performs the calculations of
% brokenbasis itself:
H = refinementbrokenbasis(knotsf,Omegaf);
checkPhifH = max(max(abs(PhiBc-PhiBf*H)))
% the value of checkPhifH confirms that H is indeed the correct refinement
% matrix. This can also be verified graphically:
%   plot(x,PhiBc,'linewidth',3,'color',0.5*[1 1 1])
%   hold on 
%   plot(x,PhiBf*H,'linewidth',3)
%   hold off
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now, we want to construct the wavelet transform from the set Omega.
% We have to define Omega on the finest level J, i.e., on x.
% This may lead to ill conditioning in the construction of PhiJ at finest level
Omega = powerdiff(x,nutilde); Omega = smoothsetmatrix(Omega);
PhiJ = brokenbasis(x,x,Omega);
% We may prefer to define Omega locally, as follows
dx = x(2:m)-x(1:m-1);
dOmega = powerdiff(dx,nutilde); dOmega = smoothsetmatrix(dOmega);
% Then Omega can be reconstructed using:
[Omegabis B] = aggregatesmoothset(dOmega);
% It can be verified that the reconstruction Omegabis is given by Omega(x-x(1))
Omega1 = powerdiff(x-x(1),nutilde); Omega1 = smoothsetmatrix(Omega1);
checkOmegabis = max(max(abs(Omegabis-Omega1)))
% Now find the wavelet coefficients
v = FWT2G1D(x,f,'BB',Omega,nu,M);
vbis = FWT2G1D(x,f,'BB',dOmega,nu,M);
fbis = IWT2G1D(x,vbis,'BB',dOmega,nu,M);
% The wavelet transform has extended the data with nutilde-2 boundary
% coefficients, so in comparing input with output, we have to take off these
% working boundare coefficients
figure(5)
npt = max(0,ceil(nutilde/2)-1);
plot(x,f-fbis(npt+(1:m)))
title('Check perfect reconstruction f-IWT(IWT(f))',fontopt{:})
