
function [Phi, HH] = Bsplinebasisbyrefinement(x,knots,ptilde,ext);

% Bsplinebasisbyrefinement - ThreshLab2/Irregular/1D/ --- 
%              construct Bspline basis using a two-scale equation
% Usage
%    [Phi, HH] = Bsplinebasisbyrefinement(x,knots,ptilde,ext);
% Inputs
%    x      points in which splines are evaluated, vector of length n
%    knots   knots of the spline basis, vector of length nk
%    ptilde  number of vanishing moments (ptilde-1 is degree of polynomials)
%    ext     (optional) binary value (default true) 
%            By setting ext=false one generates the interior basis functions
%            only. The number of basis functions then equals the number of
%            knots (= nk)
%            ext=true means that the basis will be extended so that the basis
%            functions sum up to one. The number of basis functions then equals
%            nk+ptilde-2
% Outputs
%    Phi     matrix of size n x (nk+ptilde-2) (if ext==true)
%                           n x nk            (if ext==false)
%    HH      (optional) cell array comtaining the (sparse) refinement matrices
% Description
%    Constructs a Bspline basis using refinement/subdivision. This means that
%    the grid of knots is refined up to a fine scale grid that covers all
%    points in x, so that all these points can be attributed function values.
% Notes
%    (1) Unlike Bsplinebasis.m, it makes no direct use of a recursion on the
%    order of the spline.
%    But:
%    The initial values are found by recursion and the refinement equation is
%    found through Bsplinerefinement, which does rely on a recursion.
%    Also:
%    The refinement is found analytically using a lifting scheme with
%    bidiagonal lifting matrices for the nk interior B-spline basis functions.
%    The refinement for the ptilde-2 basis functions on the boundary is found
%    numerically from function values obtained by recursion.
%    (2) In order to maintain the order ptilde at all steps of the refinement, 
%    this refinement must insert a refinement point between each pair of coarse
%    scale points (NB.:order = vanishing moments)
%    Therefore the refinement must be dyadic. Part of the code is devoted to
%    construct a dyadic scheme that refines the grid knots into a grid xx that
%    contains the grid x
% See Also
%    help Bsplinebasis
%    help Bsplinerefinement
%    help liftingstepsBspline
%    help refinementfromliftingsteps

if nargin<4, ext = true; end
if ptilde<3, ext = false; end
tol = 1.e-6;

% First find, eliminate and keep track of doubles in vector x
[x duplicates] = uniquewithduplicates(x);   

nk = length(knots); n = length(x);
[x xorder] = sort(x); x = column(x); knots = column(sort(knots));
% eliminate points from x that are also in knots (up to tolerance tol)
% These points will be moved in front.
knotsroundtol = round(knots/tol); xroundtol = round(x/tol);
knotinx = find(ismember(knotsroundtol,xroundtol));
% We want knotinx to contain one entry for coinciding knots that are also in x
% (reason: we will use knotinx to reconstruct x, where we assume that x
%  contains no coinciding values; so we do not want doubles in the
%  reconstruction, as this would cause conflicts of sizes)
  [dummy uniqueknotinx] = unique(knotsroundtol(knotinx));
  knotinx = knotinx(uniqueknotinx);
noknot = find(~ismember(xroundtol,knotsroundtol));
% make x contain non-knots only, so that these points can be used for
% refinement purproses
x = x(noknot); n = length(x);
[xx e] = sort([knots;x]);
e = find(e<nk+1);
% e now contains the position of the knots in xx
% Add points to make a dyadic refinement possible
de = e(2:nk)-e(1:nk-1); nr = max(de); J = ceil(log2(nr)); nr = 2^J;
for k=1:nk-1,
   % propose intermediate points xk between knots(k) and knots(k+1)
   xk = knots(k) + (1:nr-1)'/nr*(knots(k+1)-knots(k));
   % for each point already in xx, take out one point from xk (the closest)
   for l=e(k)+1:e(k+1)-1,
      [dm m] = min(abs(xk-xx(l)));
      xk = setdiff(xk,xk(m));
   end
   % add the remaining points of xk to x
   x = [x;xk];
end
% sort the grid with the newly added values
[xx sorted] = sort([knots;x]); nn = length(xx);
e = zeros(nn,1); e(sorted) = (1:nn); 
% e contains the position of the unsorted [knots;x] in the sorted xx
% The first nk elements of e are *in principle* the knots. Exceptions occur
% if knots coincide, therefore e will be redefined for that puprose. But first
% it is used for in the definition of ii
% ii contains indices into the extended vector xx, containing the original
% points x
ii = [knotinx;(nk+1:nk+n)']; ii = e(ii); ii = sort(ii);
% Now define e as the position of the knots in xx
% e = e(1:nk); --> fails if knots coincide
e1 = e(1);
e = (e1:nr:nk*nr);

e2 = e(nk);
nPhi = nk; if ext==true, nPhi = nk+ptilde-2; end
Phi = eye(nPhi);
HH = cell(1,J);
for j=1:J,
   ne = length(e);
   o = (e(2:ne)+e(1:ne-1))/2;
   ne = length(e); no = length(o); nf = ne+no;
   f = zeros(nf,1); f(1:2:nf) = e; f(2:2:nf) = o;
   xf = xx(f); e = (1:2:nf);
   Hj = Bsplinerefinement(xf,ptilde,ext,e);
   HH{j} = Hj;
   Phi = Hj*Phi;
   e = f;
end
% next line (as yet in comment) corrects for the slow convergence of
% subdivision
% (to be developed in future versions)
% PhiJ = Bsplinebasis(xf,xf,ptilde); Phi = PhiJ*Phi;
Phi = [zeros(e1-1,nPhi); Phi; zeros(nn-e2,nPhi)];
Phi = Phi(ii,1:nPhi);
Phi(xorder,1:nPhi) = Phi;
Phi = Phi(duplicates,1:nPhi);
