
function w = RWT_2Gspline(x,y,nt,n,M,varargin);

%  RWT_2Gspline -- 1D nondecimated spline wavelet transform on noneq. points
%   Usage
%     w       = RWT_2Gspline(x,y,nt,n,M,<other arguments>);
%   Inputs
%     x      grid-locations of observations (time, space)
%     y      fine scaling coefficients
%     nt     dual vanishing moments -- order of B-spline scaling function
%     n      primal vanishing moments (0-moments of primal wavelet function)
%     M      number of resulution levels
%   Optional inputs
%     'outputvector'  reshapes the output as a single vector or a single matrix
%   Outputs
%     w      by default:
%             N x (M+1) or (M+1) x N matrix, where N=length(y) 
%             containing non-decimated wavelet coefficients
%             if y is a row-vector, then w has M+1 rows of length N
%             if y is a column vector, then w has M+1 columns of length N
%             if y is a matrix, then w is a cell array of M+1 matrices with
%                                    size equal to that of y
%            if 'outputvector'
%             row or column vector of length N*(M+1)
%             matrix of size N*(M+1) x K, where [N K] is size of input
%   Description
%     Non decimated spline wavelet transform, type Cohen-Daubechies-Feauveau
%     (CDF) on irregular point sets
%   Note
%     (1)
%     The input values y are supposed to be fine scale spline coefficients. As
%     spline basis functions are not interpolating (for degrees higher than 2),
%     these scaling coefficients CANNOT be approximated by the function values
%     f(x_i) of the function represented by the coefficients. Prefiltering
%     required!!!
%     (2)
%     In this routine, dual vanishing moments are understood as the
%     approximation order of the scaling basis, i.e., the order/degree of the
%     B-spline scaling function. Primal vanshing moments are integrals of
%     x^n*psi(x), where psi is the wavelet basis function.
%     In the equidistant sample version of CDFspline.m, the roles of primal
%     and dual are switched. 
%   See also
%     help FWT2G1D, option 'Bspline' (decimated = fast CDF wavelet transform)
%     help IRT_2GPspline (inverse nondecimated transform)
%     help CDFspline 

% Set default values

% Parsing varargin
nvarargin = length(varargin);
addoutputinfo = false;
dualtransform = false;
vectoroutput = false;
for k = 1:nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'dualtransform','dual','adjoint'},
     dualtransform = true;
   case {'addoutputinfo'},
     addoutputinfo = true;
   case {'vector','vectorout','vectoroutput','outvector','outputvector'},
     vectoroutput = true;
   case {'matrix','matrixout','matrixoutput','outmatrix','outputmatrix'},
     vectoroutput = true;
   case {'todo'}
     warning(['no implementation for ' vark])
   otherwise
     warning(['no implementation for ' vark])
end, end, end

transposex = false;
if size(x,2) > size(x,1),
   x = x'; transposex = true;
end
transposey = false;
if size(y,1) ~= size(x,1),
   y = y'; transposey = true;
end

if n > 0,
   % Mj = initmom(x,(0:n-1));
   Mj = Bsplinemoments(x,nt,(0:n-1),false);
end

if addoutputinfo == true,
   finalupdates = cell(1,M);
   liftedrefinements = cell(1,M);
end

N = length(x);
w = cell(1,M+1);
for j = 1:M
%%%%%%%  scale = j
  skip = 2^(j-1);
  wj = zeros(size(y));
  if addoutputinfo == true,
     finalupdatesj = cell(1,2*skip);
     liftedrefinementsj = cell(1,2*skip);
  end
  for k=1:skip,
     xk = x(k:skip:N);
     yk = y(k:skip:N,:);
     if n>0, Mf = Mj(k:skip:N,1:n); end
     % LIFTING STEPS FOR PRIMAL SCALING FUNCTION (= B-spline) %
     [e o] = splitevenodd(xk);
     for eo=0:1,
        UP = nondecimatedliftingsteps('liftingstepsBspline',xk,nt,'shift',eo);
        if dualtransform == true, UP = duallifting(UP); end
        % e in the next line are the evens. Note that e and o (evens and odds)
        % change roles at the end of the iteration for eo=0:1.
        ykeo = frwliftingsteps(yk,UP,e);
        % MOMENTS %
        if n > 0,
           [H G] = refinementfromliftingsteps(UP,e);
           Me = H'*Mf; 
           Mo = G'*Mf;
           % This is the transpose of the integrated 2-scale eq.: 
           %           Phi_fine*H = Phi_coarse
           % Mfine and Mcoarse are transposed integrals of Phi*x^k
        end
        % UPDATE for primal vanishing moments (wavelet functions) %
        yo  = ykeo(o,:); ye = ykeo(e,:); U = 0;
        if n > 0,
           xo = xk(o); xe = xk(e); Ne = length(e); No = length(o);
           Io = eye(No);
           U = updateprimalmoments(Mo,Me);
           ye = ye + U*yo;
           Mj(k+skip*eo:2*skip:N,1:n) = Me;
        end
        % KEEP INFO FROM CURRENT SCALE AND SHIFT %
        if addoutputinfo == true,
           finalupdatesj{k+skip*eo} = U;
           liftedrefinementsj{k+skip*eo} = UP;
           % finalupdatesj{2*k+eo+1} = U;
           % liftedrefinementsj{2*k+eo-1} = UP;
        end
        % FILLING IN %
        y(k+skip*eo:2*skip:N,:) = ye;
        wj(k+skip*(1-eo):2*skip:N,:) = yo;
        % PREPARE SECOND PART OF NONDECIMATED TRANSFORM (cycle spin)
        % switch roles even-odd
        ecopy = e; e = o; o = ecopy;
     end % for eo=1:2
  end
  % KEEP INFO FROM THIS SCALE %
  if addoutputinfo == true,
     finalupdates{j} = finalupdatesj;
     liftedrefinements{j} = liftedrefinementsj;
  end
  % FILLING IN %
  w{j} = wj;
end
w{M+1} = y;

if size(y,2) == 1,
   wmatrix = zeros(N,M+1);
   for j = 1:M+1, wmatrix(1:N,j) = w{j}; end
   if vectoroutput, wmatrix = reshape(wmatrix,N*(M+1),1); end
   if transposey, w = wmatrix'; else, w = wmatrix; end
elseif vectoroutput,
   K = size(y,2);
   wmatrix = zeros(N*(M+1),K); k = 0;
   for j = 1:M+1, wmatrix(k+1:k+N,1:K) = w{j}; k = k+N; end
   if transposey, w = wmatrix'; else, w = wmatrix; end
end
if addoutputinfo == true,
   outputinfo = cell(2);
   outputinfo{1} = finalupdates;
   outputinfo{2} = liftedrefinements;
   switch nargout,
   case 1
      w = {outputinfo{1:2},w};
   end
end

