function xtilde = allpowercoefsBsplines(knots,ptilde,varargin)

% allpowercoefsBsplines - ThreshLab2/Irregular/1D/ --- 
%                              coefs of x^q in B-spline basis of order ptilde
% Usage
%   xtilde = allpowercoefsBsplines(knots,ptilde,'N','all');
%   xtilde = allpowercoefsBsplines(knots,ptilde,'all');
%   Xtilde = allpowercoefsBsplines(knots,ptilde,'N');
%   Xtilde = allpowercoefsBsplines(knots,ptilde);
% Input
%   knots   vector of length n; knots of spline basis
%   ptilde  order of splines; ptilde=1: constant (Haar-scaling functions)
%                             ptilde=2: linear splines
%                             ptilde=4: cubic splines
%   'N'     use standard B-spline indexing instead of scaling function indexing
%           (see below)
%   'all'   return cell matrix with all coefficients in lower order bases
% Output
%   xtilde  cell matrix, where xtilde{pt,q+1} contains the coefficients of x^q
%                        in B-spline basis of order pt (degree pt-1)
%   Xtilde  matrix of size n x ptilde
% Description
%   Columns of Xtilde satisfy: 
%          Phi^{[ptilde]}(u)*Xtilde(q) = u^q for q = 0,...,ptilde-1
%          or (if input contains 'N')
%          N^{[ptilde]}(u)*Xtilde(q) = u^q for q = 0,...,ptilde-1
%   xtilde{pt,q} satisfies 
%                Phi^{[pt]}(u)*xtilde{pt,q} = u^q
%                or (if input contains 'N')
%                N^{[pt]}(u)*xtilde{pt,q} = u^q
%                for pt = 1,...,ptilde and q = 0,...,pt-1
%   The option 'N' makesi the B-splines to be indexed as in the standard 
%   notation N_k^{[pt]}. This notation is a shifted version of the indexing
%   for scaling functions Phi_{J,k}^{[ptilde]}in a multiresolution analysis.
% Note on end points
%   allpowercoefsBsplines has to rely on knots outside the given set of knots 
%   for the calculation of coefficients in the knots near the end points.
%   Therefore, the routine copies the end knots, thus creating coinciding knots
%   on the boundary. These coinciding knots are used only in the calculation of
%   power coefficients, i.e., only one output coefficient is associated to
%   the end knot. In order to construct a B-spline basis on a bounded
%   interval that perfectly generates the power functions near the boundaries,
%   we also need to introduce coinciding knots on the boundaries. These extra
%   knots for basis functions are not provided by allpowercoefsBsplines. In
%   particular, consider the following setup
%     nk = 7; a = -1; b = 2; ptilde = 7; n = 1000;
%     knots = sort(rand(nk,1)); 
%     knots = a+(b-a)*(knots-knots(1))/(knots(nk)-knots(1));
%     x = makegrid(n,0)*(b-a)+a;
%     Phi = Bsplinebasis(x,knots,ptilde);
%   Then the power coefficients are found by
%     l = ceil(ptilde/2)-1; r = floor(ptilde/2)-1;
%     knotss = [ones(l,1)*knots(1); knots; ones(r,1)*knots(nk)];
%     Xtilde = allpowercoefsBsplines(knotss,ptilde);
%   Check with 
%     plot(x,Phi*Xtilde,'b','linewidth',3)
%     hold on
%     for q=0:ptilde-1, plot(x,x.^q,'r'), end
%     hold off
% See also
%     help Bsplinebasis
%     help Bsplinebasisbyrefinement


switch2scalingbasis = true;
returnallorders = false;
nvarargin = length(varargin);
for k = 1:nvarargin, vark = varargin{k}; if ischar(vark), switch(vark),
   case 'N', switch2scalingbasis = false;
   case 'all', returnallorders = true; 
end, end, end

if size(knots,2) == 1, knots = knots'; end

n = length(knots);
% We add ceil(ptilde/2)-1 knots to the right and floor(ptilde/2)-1 to the left,
% Indeed the recursion formula for the rightmost knot includes ceil(ptilde/2)-1
% knots further to the right. For the leftmost knot we find floor(ptilde/2)-1
% knots to the left.
r = ceil(ptilde/2)-1; l = floor(ptilde/2)-1;
r = max(1,r); l = max(0,l); % if ptilde==1 then the vector of constant coeff.
                            % needs to be extended, in order to select a
                            % shifted version (kk starts with 2)
                            % if ptilde==0, then l should not be -1, as this
                            % leads to kk starting with 0
x = [zeros(1,ptilde-1) ones(1,l)*knots(1) knots ones(1,r)*knots(n)];
n = length(x);
% zeros in front are needed, because coinciding zeros are the only
% configuartion for which all coefficients xtilde(pt,q) are known without
% recursion. We need these coefficients as starting values in the recursion
% formulae. We need ptilde-1 zeros, as the starting coefficients (like all
% other coefficients) depend upon pt-1 knots, with pt going from 1 to ptilde

xtilde = cell(ptilde,ptilde);
xtilde{1,1} = ones(1,n-1);    % pt = 1; q = 0;
if ptilde>1,
   xtilde{2,1} = ones(1,n-1); % pt = 2; q = 0;
   xtilde{2,2} = x(2:n);      % pt = 2, q = 1;
end
for pt = 3:ptilde,
   xtilde{pt,1} = ones(1,n-1);
   for q = 1:pt-1,
      xtildeptmin1qmin1 = xtilde{pt-1,q};
      xtildeptq = zeros(1,n-1);
      % xtildeptq(1) = q/(pt-1)*xtildeptmin1qmin1(1)*(x(pt)-x(1));
      % for k = 2:n-pt+1,
      %    xtildeptq(k) = xtildeptq(k-1) + q/(pt-1)*xtildeptmin1qmin1(k)*...
      %                       (x(k+pt-1)-x(k));
      % end
      %% the loop above is equivalent to the vector operation below:
      xtildeptq(1:n+1-pt) = q/(pt-1)*...
          cumsum(xtildeptmin1qmin1(1:n+1-pt).*(x(pt:n)-x(1:n+1-pt)));
      xtilde{pt,q+1} = xtildeptq;
   end
end

n = length(knots); % restore original value
if switch2scalingbasis == true,
   for pt = 1:ptilde, for q = 0:pt-1,
      kk = (ptilde-1+l)-floor(pt/2)+(1:n);
      xtildeptq = xtilde{pt,q+1};
      xtildeptq = xtildeptq(kk);
      xtilde{pt,q+1} = xtildeptq;
   end, end
else
   kk = (ptilde-1+l)+(1:n);
   for pt = 1:ptilde, for q = 0:pt-1,
      xtildeptq = xtilde{pt,q+1};
      xtildeptq = xtildeptq(kk);
      xtilde{pt,q+1} = xtildeptq;
   end, end
end


if returnallorders == false,
   n = length(knots); % restore original value
   Xtilde = ones(n,ptilde);
   for q = 1:ptilde-1, Xtilde(1:n,q+1) = column(xtilde{ptilde,q+1}(1:n)); end
   xtilde = Xtilde;
end
