
function [w, Ne, moments, odds, hulls, bandwidths] = FMLPT1D(x,y,M,varargin);

% FMLPT1D -- ThreshLab2/Irregular/MultiscaleKernel -- ThreshLab's routine 
%            for Forward Multiscale Local Polynomial Transforms, 1D, which is
%            a Burt-Adelson's pyramid on irregular point sets
%            with a prefilter Ft, prediction P, and an update U
%  Usage
%    [w, Ne, moments] = FMLPT1D(x,y,M,varargin);
%    [w, Ne, moments, odds, hulls, bandwidths] = FMLPT1D(x,y,M,varargin);
%    [w, outputinfo] = FMLPT1D(x,y,M,varargin,'addoutputinfo');
%    outputinfo = FMLPT1D(x,zeros(size(x)),M,varargin,'addoutputinfo');
%  Inputs
%    x        grid-locations of observations (time, space)
%    y        observed values
%    M        number of resulution levels
%    varargin several options (see below)
%  Outputs
%    w        vector with wavelet coefficients
%    Ne       (optional) number of scaling coefficients (dependent on M)
%    moments  (optional) moments of intermediate scaling functions (before
%                       update in each step)
%    odds     (optional) array or cell array of indices into x; each entry in
%             the (cell) array corresponds to one scale: x(odds{j}) are the
%             points that are taken out at scale j, thereby defining the
%             coarser grid at scale j+1
%             (this information is used in constructMLPTtree)
%    hulls    (optional) cell array carrying for each resolution level j a
%             vector with values in [0,1], corresponding to each predictable
%             point (evens and odds) at that scale. The value is 0 if the
%             prediction is not used in the reonstruction. In that case, the
%             detail offset is not computed, nor is it stored. In case of a
%             positive value (which includes all odds), the detail is computed
%             and stored for use in the reconstruction.
%             (this information is used in constructMLPTtree)
%    bandwidths  real vector with bandwidths used at each resolution level,
%                ordered from coarse to fine:
%                bandwidths(1) is the scale at the coarsest level
%                bandwidths(end-1) is the scale at the finest level
%                bandwidths(end) is a proposed scale for superresolution
%                                refinement
%    outputinfo (optional) cell array, containing detail information about the
%                                      lifting steps, 
%                         in particular information for use in reconstruction
%                         (IMLPT1D.m)
%                         outputinfo = {predictions,finalupdates,hulls,Ne,odds}
%                         This information is generated and added to the
%                         output only if the input contains the string
%                         'addoutputinfo'
%  Description
%    Applies repeated smoothed lifting filter bank with prefiltering,
%    one prediction step and one update step. 
%    If y is a matrix, then the transform is performed on its COLUMNS
%  Examples
%    Set up
%       N = 1000;
%       x = makegrid(N,1);
%       f = makesignal(x,'skyline');
%       y = makenoisy(f,10);
%       n = 2; M = 7;
%       predkernel = 'cos';
%       finescalebandwidth = (max(x) - min(x)) * log(N)/(2*N);
%       hJ = finescalebandwidth;
%       parFMLPT = {'P','kernel','kerneltype',predkernel,'n',n,'hJ',hJ};
%    Analysis
%       [w , Ne, moments] = FMLPT1D(x,y,M,parFMLPT{:});
%  Additional (optional) parameters (varargin; see above)
%    {'P',Pmethod}
%          examples of Pmethod: 'kernel','locpol','intpol'
%    {'kerneltype',kernel} 
%          for list of possible kernels: help evalkernel
%    {'bandwidth',hfine} or {'bandwidths',hfine} with hfine a real number
%                 specifies bandwidth at finest resolution level
%    {'h',bandwidths} or {'hJ',bandwidths} or 
%    {'bandwidths',bandwidths} with bandwidths a real vector 
%                bandwidths fixes scales at each resolution level,
%                bandwidths is ordered from fine to coarse 
%                (bandwdiths(1) is the scale at the finest level)
%                !!! This is in contrast to vector bandwidths at the output
%                if bandwidths has less than M entries (in particular, if it
%                has only one entry), bandwidths at coarser levels are computed
%                from further specifications (or by multiplication by 2, by
%                default)
%    {'nt',nt}   number of dual vanishing moments
%    {'convex',true}
%    {'nU',n}
%    {'hull', kernel}
%    {'split','hull'}
%    {'minnn', minnn}  integer or vector of integer: minimum number of even
%                      neighbours used in prediction of odds: this parameter
%                      has effect in combination with {'split','hull'}
%  Note
%     n(1) is the number of primal vanishing moments (unless specified
%     otherwise by optional input - no such option has been implemented so far)
%     nu = sum(n) is number of free parameters in final update step
%     In the current implementation, n(2) = nu-n(1) free parameters are used to
%     make the transform as close as possible to being orthogonal
%  See also
%    help FMLPT1D1step
%    help IMLPT1D
%    type illustrateFMLPT1D

x = column(x);
N = length(x);
wtranspose = false;
if size(y,1) ~= N, y = y'; wtranspose = true; end
if size(y,1) ~= N, error('dimension mismatch x and y'), end
ncols = size(y,2);

n = 0; % default value; may be overwritten by subsequent statement
nvarargin = length(varargin);
onestepvarargin = varargin;
addoutputinfo = false;
Pmethod = 0;
Ftmethod = 1;
sout = false;
hull = 1;
nt = 1;
minnn = NaN; % minimum number of neighbours used for prediction. Default will
             % be set to nt, as soon as nt has been fixed
bandwidths = (max(x) - min(x)) * log(N)/(2*N);
kerneltype = [];
prefilteroptions = {};
partitioning = 'evenodd';
logfactorinnextbandwidth = false;
bandwidthbysubsamplerate = false;
prefiltcovariates = false;
usegridforMcoarse = false;
reorder = false;
Poptions = {};
subsampleP = false;

k = 1;
while k <= nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'n','nU','primom','primalmoments','nprimal','nprimalmoments','p',...
         'primal'}
      n = varargin{k+1};
      k = k+2;
   case {'nt','ntilde','ntP','dualmom','dualmoments','ndual',...
         'ndualmoments','pt','ptilde','dual'}
      nt = varargin{k+1};
      k = k+2;
   case {'Ft','Ftfilter','Ht','Htfilter','prefilter','prefilt'}
      Ftmethod = varargin{k+1}; 
      k = k+2;
   case {'P','Pmethod','prediction','Prediction','Predict','predict'}
      Pmethod = varargin{k+1};
      k = k+2;
   case {'bandwidth','bandwidths','h','hj','hJ'}
      bandwidths = varargin{k+1};
      k = k+2;
   case {'kerneltype','type'}
      kerneltype = varargin{k+1};
      k = k+2;
   case {'prefilteroptions','prefiltoptions','Htopt','prefiltoption',...
         'prefiltopt','Ftopt'}
      Ftoption = varargin{k+1};
      k = k+2;
      if iscell(Ftoption),
         prefilteroptions = {prefilteroptions{:},Ftoption{:}};
      else
         prefilteroptions = {prefilteroptions{:},Ftoption};
      end
   case {'split1off','take1off','takeoneoff','1coefatatime','1coef-at-a-time'}
      partitioning = 'onecoef';
      bandwidthbysubsamplerate = true;
      k = k+1;
   case {'split','splitscheme','partitioning','takeoff','takeaway'};
      partitioning = varargin{k+1};
      k = k+2;
   case {'hull'}
      hull = varargin{k+1};
      k = k+2;
   case {'minnn'}
      minnn = varargin{k+1};
      k = k+2;
   case {'logfactorbandwidth'},
      % On nonequispaced data, it may be interesting to have bandwidths a bit
      % larger than the inverse of the subsampling rate
      logfactorinnextbandwidth = true;
      k = k+1;
   case {'bandwidthbysubsamplerate'}
      bandwidthbysubsamplerate = true;
      k = k+1;
   case {'prefiltx','prefiltgrid'},
      % Applies the prefilter to the fine scale covariate values.
      % Usefull if Ft*x =/= x
      prefiltcovariates = true;
      k = k+1;
   case {'addoutputinfo'},
      addoutputinfo = true;
      k = k+1;
   case {'useinIMLPT'},
      addoutputinfo = true; Ft = 1;
      k = k+1;
   case {'scalingcoefs','apprx','scaling'},
   % replace detail offsets in output by scaling coefficients
   % (=approximations) at successive scales. Can be used in MLP-Fisz
   % transforms
      sout = true;
      k = k+1;
   case {'reorderU','reorderupdate','reorder','relocate','relocation'}
      reorder = 1; B = vararginsentinel{k+1}; 
      if islogical(B)|isnumeric(B), if ~isnan(B),
         reorder = B; k = k+1;
      end, end
      k = k+1;
   case {'subsampleP'},
      % for experimental use only: this option can be used to simulate a
      % transform or a refinement where even indexed knots are left out from
      % prediction. This will create fractal-like basis functions.
      subsampleP = true;
      k = k+1;
   case {'adaptive','adaptive2','knearest','knearest2'},
      if isempty(kerneltype), kerneltype = 'unif'; end
      bandwidths = NaN;
      Poptions = {Poptions{:},vark};
      k = k+1;
   case {'todo'}
      warning(['no implementation for ' vark])
      k = k+1;
   otherwise % the argument cannot be interpreted by this routine and it is
             % passed to the next level
      Poptions = {Poptions{:},vark};
      k = k+1;
   end
   else % if ischar(vark)
      Poptions = {Poptions{:},vark};
      k = k+1;
end, end

if isempty(kerneltype), kerneltype = 'cos'; end
N = length(x);
% partitioning = findoddsinfinestgrid(partitioning,N,true);
nu = sum(n);
m = nu-n(1);
n = n(1);
% from now on, n is the number of primal vanishing moments
if m > 0,
   Wt = sparse(eye(N));
   W = Wt;
end
if n > 0,
   Mfine = initmom(x,(0:n-1));
   nM = n;
elseif ischar(Ftmethod),
   Mfine = initmom(x);
   nM = 1;
else
   Mfine = [];
   nM = 0;
end
moments = Mfine;

if addoutputinfo == true,
   predictions = cell(1,M+1);
   finalupdates = cell(1,M+1);
end
odds = cell(1,M);
hulls = cell(1,M+1);

w = [];
usedbandwidths = zeros(1,M+1);
usedbandwidths(M+1) = bandwidths(1)/2;
j = M+1;
I = sparse(eye(N));
yeven = y; Ne = N;
while j>1,
   j = j-1;
%%%%%%%  Design of transform, scale = j

   %%%%%%%%%%%%%%%%%%%%%%%%% SPLIT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   if isnumeric(partitioning), % if partitioning is a vector, then this should
                               % be interpreted as 1-coef-at-a-time
      np = length(partitioning);
      odd = partitioning(1);
      partitioning = partitioning(2:np);
   elseif iscell(partitioning), % cell array of split strategies or 
                                % partitionings for each scale; finest scales
                                % come first in cell aray (<-> output odds!!)
      np = length(partitioning);
      if np>0,
         odd = partitioning{1};
         partitioning = {partitioning{2:np}};
      else % partitioning is empty cell array; not specified
         odd = 'evenodd';
      end
   else
      odd = partitioning;
   end
   if ischar(odd),
      switch odd
      case {'onecoef','1coef','1coef-at-a-time'},
         [even odd] = split1off(x,bandwidths(1));
      case {'withbandwidth','bandwidth','bandwidthbased','h','hull'},
         if isnan(minnn), minnn = nt; end
         [even odd] = splitfrombandwidth(x,bandwidths(1),minnn(1));
      otherwise % default partitioning = even/odd
         [even, odd] = splitevenodd(x);
      end
   else
      even = setdiff((1:N),odd);
   end
   yeven = y(even,:);
   xeven = x(even);
   Ne = length(even); 
   if nM>0,
      Meven = Mfine(even,1:nM); Modd = Mfine(odd,1:nM); 
   end
   odds{j} = odd;
   %%%%%%%%%%%%%%%%%%%%%%%%% PREFILTER DESIGN (1)  %%%%%%%%%%%%%%%%%%%%%%%%%
   Ft = 1;
   if prefiltcovariates == true & ischar(Ftmethod),
      % Normally, the prefilter is designed for a given prediction.
      % Some prefilters, however, (in particular the Haar averaging operator)
      % are set up independently from P (prediction). In those cases, it may be
      % interesting to apply the prefilter on the covariates as well,
      % especially if Ft*x =/= x, and let the prediction depend on the
      % prefilter's effect on the covariate data.
      Ft = constructprefilter(Ftmethod,x,prefilteroptions{:});
      Ftx = Ft*x;
      if length(Ftx) == length(xeven),
         xeven = Ftx;
      elseif length(Ftx) == length(x),
         xeven = Ftx(even);
      end
   end
   %%%%%%%%%%%%%%%%%%%%%%%%% PREDICTION MATRIX %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   P = 0;
   Ie = sparse(eye(Ne));
   h = NaN; % necessary for use in findhullMLPT below
   if ischar(Pmethod), Pmethod2 = [Pmethod '     '];
      switch(Pmethod2(1:6))
      case {'kernel'}
         h = bandwidths(1);
         Pvars = {x,xeven,Ie,h,kerneltype};
         Pmethod = 'kernelestimation';
      case {'interp','polyno','intpol'}
         Pvars = {row(xeven),Ie,row(x),nt};
         Pmethod = 'interpolynomialprediction';
      case {'locpol','localp'}
         h = bandwidths(1);
         Pvars = {x,xeven,Ie,h,kerneltype,nt-1};
         Pmethod = 'locpolestimation00';
      case {'deriva','differ'}
         h = bandwidths(1);
         Pvars = {x,xeven,Ie,h,kerneltype,nt-1};
         Pmethod = 'derivlocpolestimation00';
      otherwise
         h = bandwidths(1);
         Pvars = {'x',x,'X',xeven,'Y',Ie,'bandwidths',h,'degree',nt-1};
      end
      if exist(Pmethod,'file') ~= 0,
         Pvars = {Pvars{:},Poptions{:}};
         P = sparse(feval(Pmethod,Pvars{:})');
      end
   end
   if isscalar(Pmethod), 
      P = sparse(diag(Pmethod*ones(1,N))); P = P(1:N,even);
      h = bandwidths(1);
   end
   % if subsampleP==true, P(even,1:Ne) = eye(Ne); end
   % subsampleP amounts to no prediction in the evens. Since the 
   %%%%%%%%%%%%%%%%%%%%%%%%% PREDICTION HULL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % The balance Q is mainly used in the reconstruction (IMLPT1D1step), where
   % it controls the combination of two independent reconstructions from the
   % transformation (which provides an overcomplete decomposition) Here we need
   % Q in the computations of the moments. These moments are integrals of
   % synthesis basis functions, i.e., basis functions used upon reconstruction.
   % For this reason, the balance Q used in the reconstruction, has to be
   % anticipated in the decomposition. We also need to know where Q is nonzero,
   % because we only have to keep offsets where Q is nonzero. Third, we can
   % only use prefilters where Q equals 1.
   if ~isnan(h), Q = findhullMLPT(x,x(odd),h/2,hull);
   else, Q = ones(size(x));
   end
   if subsampleP==true, Q(even) = 0; end
   Qnot1e = find(Q(even)<1);
   Qnot1 = find(Q<1);
   Qnon0 = find(Q>eps); % Q is assumed to be non-negative
   Qnon0e = find(Q(even)>eps);
   Q0 = find(Q<eps);
   Q = sparse(diag(Q));
   %%%%%%%%%%%%%%%%%%%%%%%%% MOMENTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % subsampleMcoarse = false;
   Mcoarse = Mfine; 
   if nM > 0,
      if usegridforMcoarse == true,
         Mcoarse = Mfine + [Mfine(2:nM); 0];
         % subsampleMcoarse = true;
         % Mcoarse will be subsampled later on, keeping the evens only
         % Moments corresponding to scaling coefficients that are reconstructed
         % from copying the corresponding coarse scaling coefficient, should
         % not be modified:
         Mcoarse(Q0) = Mfine(Q0);
         Mcoarse = Mcoars(even,1:nM);
      else
         % Mcoarse = zeros(Ne,n); 
         % Mcoarse = Mfine(even,:); Mcoarse0 = Mcoarse;
         % Mcoarse(1:Ne,:) = P(1:N,1:Ne)'*Mfine(1:N,:);
         Mcoarse0 = (I-Q')*Mfine; Mcoarse0 = Mcoarse0(even,1:nM);
         Mcoarse1 = P'*Q'*Mfine(1:N,1:nM);
         Mcoarse = Mcoarse0+Mcoarse1;
      end
   end
   %%%%%%%%%%%%%%%%%%%%%%%%% PREFILTER MATRIX %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % Note: some functions create squared filter matrices, while others include
   % a subsampling step. Those matrices are rectangular. This has to be checked
   % upon application of the prefilter (see below)
   if ischar(Ftmethod), Ft2 = [Ftmethod '                      '];
      if Ft2(1:3) == 'Unb' | Ft2(1:3) == 'unb', % Unbalanced Haar
         prefilteroptionsj = {Mfine,Mcoarse,prefilteroptions{:}};
         Ft = constructprefilter(Ftmethod,x,prefilteroptionsj{:});
      elseif ~isempty(findstr('iorth',Ftmethod)),
         Ft = findbiorthfilter(P,Mfine,even);
      % first check biorth, then orth (do not change order of checking)
      elseif ~isempty(findstr('rth',Ftmethod)) & ...
             ~isempty(findstr('notsparse',Ftmethod)),
         Ft = prefilterorthnotsparseSTCO(x,nt,even,prefilteroptions{:});
      else switch Ft2(2:5)
      case {'dem ','demp','anis','nvari'},
         %idempotent, vanishing moments, invariance
         h =  bandwidths(1);
         Ft = prefiltbyvanmom(P,Mfine,x,even,'prefilter',Pmethod,'nt',nt,...
                            'h',h,'kerneltype',kerneltype,prefilteroptions{:});
      otherwise
         Ft = constructprefilter(Ftmethod,x,prefilteroptions{:});
      end, end
   elseif isscalar(Ftmethod),
      Ft = I*Ftmethod;
   elseif isscalar(Ft),
      Ft = I*Ft;
   end
   %%%%%%%%%%%%%%%%%%%%%%%%% UPDATE MATRIX %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   U = 0;
   if n>0,
      Mf = Q'*Mfine(1:N,1:n); Mc = Mcoarse(1:Ne,1:n);
   end
   if reorder == 1 && n>1,
      xo = Mf(1:N,2)./Mf(1:N,1);
      xe = Mc(1:Ne,2)./Mc(1:Ne,1);
   else
      xe = xeven; xo = x; % use variable xo as in FWT2G1D
   end
   No = N; % use variable No as in FWT2G1D
   Me = NaN; Mo = NaN; ee = (1:Ne); oo = (1:No);
   if n > 0,
      [xo oo] = sort(xo); [xe ee] = sort(xe);
      Mo = Mf(oo,1:n); Me = Mc(ee,1:n);
      Io = sparse(eye(No));
      if m == 0, % all free parameters for VANISHING MOMENTS
         Ue = updateprimalmoments(xo,Io,xe,Mo,Me);
      end
   end
   if m > 0,
      % make transform as close as possible to ortho.
      J = I(even,1:N); 
      Ht = J'; Gt = I-J'*P'; H = Q*P+(I-Q)*J'; G = Q;
      PD = {'filterbank',H,G,Ht,Gt};
      dd = bandliftingstep(xe,xo,nu);
      [Ue, W, Wt] = updateLSapprxprimmomQ(PD,nu,Me,Mo,W,Wt,dd,Qnon0e,Qnon0);
      % Although filters Ht, Gt are computed explicitly for optimisation of
      % the update, Wt is NOT used for calculation of the actual transform, as
      % Wt*input requires more computations that the lifting scheme
   end
   if nu > 0,
      % yeven(ee,:) = yeven(ee,:) + Ue*yodd(oo,:);
      U = zeros(size(Ue)); U(ee,oo) = Ue;
   end
   if isscalar(U), U = sparse(diag(U*ones(1,N))); U = U(even,1:N); end
%%%%%%%%%%%%%% Execution of transform %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %%%%%% PREFILT %%%%%%
   % SUBSAMPLE %
   % some prefilter constructors include the subsampling
   % (e.g. prefilterorthnotsparseSTCO)
   % others (e.g. constructprefilter, idle prefilter) do not
   if size(Ft,1)==N, Ft = Ft(even,1:N); end
   yeven = Ft*y;
   % no prefilter allowed in even points where coarse scaling coefficient will
   % be used in reconstruction (Qnot1)
   if ~isempty(Qnot1), yeven(Qnot1e) = y(Qnot1); end
   %%%%%% PREDICT %%%%%%
   offsets = y - P*yeven;
   %%%%%% UPDATE %%%%%%
   yeven = yeven + U*offsets;
%%%%%%%%%%%%%% write output %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   usedbandwidths(j) = h;
   odds{j} = odd;
   hulls{j} = diag(full(Q));
   if nM > 0, moments = [Mcoarse;moments]; else, moments = []; end
   if addoutputinfo == true,
      predictions{j} = P;
      finalupdates{j} = U;
   end
   if max(size(w)) == 0,
      w = offsets(Qnon0,:);
   else
      w = [offsets(Qnon0,:); w];
   end
%%%%%%%%%%%%%% Prepare next step %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   nh = length(bandwidths);
   if nh == 1,
      if logfactorinnextbandwidth == true,
         bandwidths = N/Ne*log(Ne)/log(N)*bandwidths;
      elseif bandwidthbysubsamplerate == true,
         % bandwidthbysubsamplerate has been set true either explicitly or by
         % calling "split1off"
         bandwidths = N/Ne*bandwidths;
      else % typically Ne == ceil(N/2), so N/Ne is close to 2; 
           % In this case, we take 2 for reasons of backwards compatibility; 
           % may be replaced by N/Ne in future versions.
           % This option should also be used for bandwidth based split on
           % heterogeneous grids.
         bandwidths = bandwidths*2;
      end
   else bandwidths = bandwidths(2:nh);
   end
   x = xeven; y = yeven;
   N = Ne; I = Ie;
   Mfine = Mcoarse;
   if length(minnn) > 1, minnn = minnn(2:end); end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
w = [yeven; w];
bandwidths = usedbandwidths;

if wtranspose, w = w'; end
if addoutputinfo == true,
   outputinfo = cell(1,5);
   outputinfo{1} = predictions;
   outputinfo{2} = finalupdates;
   outputinfo{3} = hulls;
   outputinfo{4} = Ne;
   outputinfo{5} = odds;
   switch nargout,
   case 1
      w = {outputinfo{1:5},w};
   case 2
      Ne = outputinfo;
   case 3
      moments = outputinfo;
   end
end
