function [w, Ne, moments, adaptinfo] = FMLPT2D(x,y,z,M,varargin);

% FMLPT2D -- ThreshLab2/Irregular/MultiscaleKernel -- ThreshLab's routine 
%            for Forward Multiscale Local Polynomial Transforms, 2D, which is
%            a Burt-Adelson's pyramid on irregular point sets
%            with a prefilter Ht, prediction P, and an update U
%            applied on scattered data (2D-functions; including images)
%  Usage
%    [w, Ne, moments] = FMLPT2D(x,y,z,M,varargin);
%  Inputs
%    x        x-coordinate of scattered data
%    y        y-coordinate of scattered data
%    z        observation of f(x,y)+error
%    M        number of resulution levels
%    varargin several options (see below)
%  Outputs
%    w        vector with wavelet coefficients
%    Ne       (optional) number of scaling coefficients (dependent on M)
%    moments  (optional) moments of intermediate scaling functions (before
%                       update in each step)
%    adaptinfo (optional) information (in appropriate format) on adaptive
%                         choices (if applicable)
%  Description
%    Applies repeated smoothed lifting filter bank with prefiltering,
%    one prediction step and one update step. 
%    If y is a matrix, then the transform is performed on its COLUMNS
%  Examples
%  Note
%  See also
%    help FMLPT2D1step (1 step in forward transform)
%    help IMLPT2D (inverse transform)
%    help FMLPT1D


x = column(x); y = column(y);
N = length(x);
if size(y,1) ~= N, error('dimension mismatch x and y'), end
wtranspose = false;
if size(z,1) ~= N, z = z'; wtranspose = true; end
if size(z,1) ~= N, error('dimension mismatch (x,y) and z'), end
ncols = size(z,2);

n = [0 0]; % default value; may be overwritten by subsequent statement
nvarargin = length(varargin);
addzeroupdate = false;
addoutputinfo = false;
for k = 1:nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
    case {'n','nU','primom','primalmoments','nprimal','nprimalmoments'}
       if varargin{k+1} == 0, addzeroupdate = true; end % see below for use
       n1 = varargin{k+1}; 
       if length(n1) == 1, n1 = n1*[1 1];
       else n1 = row(n1); n1 = n1(1:2); end
       n = max(n,n1);
    case {'Ht','Htfilter','prefilter','prefilt'}
    % we need scaling moments for the construction of prefilters. In order to
    % activate the creation of a vector of moments, we set n to at least 1.
       n = max(n,[1 1]);
    case {'addoutputinfo'},
       addoutputinfo = true;
end, end, end

if any(n)>0,
   nx = max(1,n(1)); ny = max(1,n(2)); p = nx*ny;
   nn = zeros(p,2); k = 1;
   for kx = 0:nx-1, for ky = 0:ny-1, nn(k,1:2) = [kx ky]; k=k+1; end, end
   Mcoarse = initmom2D(x,y,nn);
else
   Mcoarse = [];
end
moments = Mcoarse;

xcoarse = x; ycoarse = y;
swcoarse = z;
w = swcoarse;
Ne = length(xcoarse);
Nw = size(w,1);
onestepvarargin = varargin;
if addoutputinfo == true,
   grids = cell(1,M+1);
   bandwidths = cell(1,M);
end

%%% compare to FMLPT1D.m for possible extensions at this point

if addoutputinfo == true, grids{M+1} = (1:N); end
for j = M:-1:1,
   xfine = xcoarse; yfine = ycoarse;
   sfine = swcoarse(1:Ne,1:ncols);
   Mfine = Mcoarse;
   previousdetails = w(Ne+1:Nw,1:ncols);
   [xcoarse ycoarse swcoarse Mcoarse listofoptions usedbandwidths e] = ...
            FMLPT2D1step(xfine,yfine,sfine,Mfine,onestepvarargin{:});
   w = [swcoarse; previousdetails];
   onestepvarargin = listofoptions;
   Nw = size(w,1);
   Ne = length(xcoarse);
   moments = [Mcoarse;moments];
   if addoutputinfo == true,
      f = grids{j+1};
      grids{j} = f(e);
      bandwidths{j} = usedbandwidths;
   end
end

%%% compare to FMLPT1D.m for possible extensions at this point

if wtranspose == true, w = w'; end

adaptinfo = 'none';

if addoutputinfo == true,
   outputinfo = cell(3);
   outputinfo{1} = moments;
   outputinfo{2} = grids;
   outputinfo{3} = bandwidths;
   if nargout == 2,
      Ne = outputinfo;
   elseif nargout == 1,
      w = outputinfo;
   end
end
