function [xcoarse ycoarse wcoarse Mcoarse outputoptions usedbandwidths even] = ...
            FMLPT2D1step(xfine,yfine,sfine,Mfine,varargin);

% FMLPT2D1step -- ThreshLab2/Irregular/MultiscaleKernel -- One step in
%                ThreshLab's routine FMLPT2D.m
%  Usage
%    [xcoarse ycoarse wcoarse Mcoarse outputoptions usedbandwidths even] = ...
%    FMLPT2D1stepxfine,yfine,sfine,Mfine,varargin);
%  Inputs
%    xfine  fine scale x coordinates of grid-locations of observations 
%    yfine  fine scale y coordinates of grid-locations of observations
%    sfine  fine scale scaling coefficients
%    Mfine  fine scale moments of scaling basis functions
%    ...    several optional parameters, for use in constructprefilter
%
%  Outputs
%    xcoarse coarse scale x coordinates of grid-locations of observations 
%    ycoarse coarse scale y coordinates of grid-locations of observations 
%    wcoarse coarse scale scaling and wavelet coefficients 
%    Mcoarse coarse scale moments of scaling basis functions
%    usedbandwidths (optional) bandwidths used in this scale
%  Description
%    Applies one step lifting filter bank with an optional prefilter, at most 
%    one prediction step and at most one update step. 
%    If sfine is a matrix, then the transform is performed on its COLUMNS
%  Note
%    the first length(xcoarse) elements in wcoarse are coarse scale scaling
%    coefficients. Remaining elements in wcoarse are coarse scale wavelet
%    coefficients
%  See also
%    help FMLPT2D
%    help FMLPT1D1step


transposex = 0;
if size(xfine,2) > size(xfine,1),
   xfine = xfine'; transposex = 1;
end
transposey = 0;
if size(yfine,2) > size(yfine,1),
   yfine = yfine'; transposey = 1;
end
transposew = 0;
if size(sfine,1) ~= size(xfine,1),
   sfine = sfine'; transposew = 1;
end
x = xfine;
y = yfine;
z = sfine;
N = length(x);
if length(y) ~= N, error('sizes xfine and yfine dismatch'), end
n = size(Mfine,2);

% following values are the default values; they may be overwritten while
% parsing through varargin
Ht = 1;
P = 1;
U = 0; if n>0, U = 'primal'; end
nt = [1 1];
bandwidthsx = (max(x) - min(x)) * 3/sqrt(N);
bandwidthsy = (max(y) - min(y)) * 3/sqrt(N);
bandwidths = [bandwidthsx; bandwidthsy];
% if all x (or all y) are equal, then take bandwidth 1:
constantcoordinate = abs(bandwidths)<eps;
bandwidths(find(constantcoordinate)) = 1;
kerneltype = 'cos';
prefilteroptions = {};
adaptive = 0;
adaptinfo = NaN;
reorder = 0;
splitrate = 2;
splitscheme = 'splitbypeel';
% Parsing varargin
nvarargin = length(varargin);
outputoptions = {};
otheroptions = {};
k = 1;
while k <= nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'split','splitscheme','partitioning','takeoff','takeaway'}
      splitscheme = varargin{k+1};
      k = k+2;
   case {'splitrate'}
      splitrate = varargin{k+1};
      k = k+2;
   case {'nt','ntilde','ntP','dualmom','dualmoments','ndual','ndualmoments'}
      nt = varargin{k+1};
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   case {'n','nU','primom','primalmoments','nprimal','nprimalmoments'}
      % n is passed implicitly by the size of Mfine, no need to pass on n
      % explicitly
      k = k+2;
   case {'Ht','Htfilter','prefilter','prefilt'}
      Ht = varargin{k+1};
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   case {'P','Pmethod','prediction','Prediction','Predict','predict'}
      P = varargin{k+1};
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   case {'U','update'}
      U = varargin{k+1};
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   case {'bandwidth','bandwidths','h','hj','hJ'}
      bandwidths = varargin{k+1};
      % bandwidth in outputoptions will be different (see below)
      k = k+2;
   case {'kerneltype','type'}
      kerneltype = varargin{k+1};
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   case {'prefilteroptions','prefiltoptions','Htopt','prefiltoption',...
         'prefiltopt'}
      Htoption = varargin{k+1};
      if iscell(Htoption),
         prefilteroptions = {prefilteroptions{:},Htoption{:}};
      else
         prefilteroptions = {prefilteroptions{:},Htoption};
      end
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   case {'addoutputinfo'},
      % do nothing/ignore this argument and do not pass to subsequent functions
      % (this argument has been processed in FMLPT2D.m)
      k = k+1;
   otherwise % the argument cannot be interpreted by this routine and it is
             % passed to the next level
      outputoptions = {outputoptions{:},vark};
      otheroptions = {otheroptions{:},vark};
      k = k+1;
   end
   else % if ischar(vark)
      outputoptions = {outputoptions{:},vark};
      otheroptions = {otheroptions{:},vark};
      k = k+1;
end, end

usedbandwidths = bandwidths;

% Design of the transform
  % SPLIT %
  switch splitscheme,
  case {'rowbyrow'},
     [even, odd] = splitevenoddrowbyrow(x,splitrate);
  otherwise
     % splitbypeelscattered does not work properly when one of the coordinates
     % is a constant
        cc = constantcoordinate; 
        xx = x; xx(1) = xx(1)+cc(1); yy = y; yy(1) = yy(1)+cc(2);
     [even, odd] = splitbypeelscattered(xx,yy,'splitrate',splitrate);
  end
  xodd = x(odd);
  xeven = x(even);
  yodd = y(odd);
  yeven = y(even);
  zeven = z(even);
disp('split done')
  % PREDICTION MATRIX %
  Ne = length(even);
  Ie = sparse(eye(Ne));
  if ischar(P), Pmethod = [P '     '];
     switch(Pmethod(1:6))
     case {'locpol','localp'}
        h = bandwidths(:,1); usedbandwidths = h;
        Pvars = {x,y,xeven,yeven,Ie,h,kerneltype,nt-1};
        P = 'locpolestimation2D';
     otherwise
        Pvars = {'x',x,'y',y,'X',xeven,'Y',yeven,'Z',Ie,...
                 'bandwidths',bandwidths,'degree',nt-1};
     end
     if exist(P,'file') ~= 0,
        Pvars = {Pvars{:},otheroptions{:}};
        P = sparse(feval(P,Pvars{:})');
     else
        P = 1;
     end
  end
  if isscalar(P), P = sparse(diag(P*ones(1,N))); P = P(1:N,even); end
disp('P done')
  % MOMENTS %
  % Mcoarse = Mfine(even,:) + P'*Mfine;
  if ~isempty(Mfine),
     Mcoarse = P'*Mfine;
  else
     Mcoarse = Mfine;
  end
  % PREFILTER %
  % not yet implemented
  Ht = sparse(eye(N));
  % UPDATE %
  if ischar(U), U = [U '     '];
  switch U(2:6),
  case {'oment','rimmo','rimal'} % minimum 
     if n>0,
        n = min(n,N);
        Mf = Mfine(1:N,1:n); Mc = Mcoarse(1:Ne,1:n);
        Io = sparse(eye(N));
        % At this moment no reordering has been implemented (cfr.
        % FMLPT1D1step.m). Such a reordering would take the form of a
        % computation of a center of mass for all basis functions, in order to
        % identify or redefine the closest neighbours
        xo = x; yo = y; xe = xeven; ye = yeven;
        U = updateprimalmoments2D(xo,yo,Io,xe,ye,Mf,Mc);
     else
        U = 0;
     end
  case {'innor'} % minimum norm solution within 
     disp('not yet implemented')
  otherwise
     U = 0;
  end, end
  if isscalar(U), U = sparse(diag(U*ones(1,N))); U = U(even,1:N); end
disp('U done')
% Execution of transform
  % PREFILT %
  zeven = Ht*z;
  % SUBSAMPLE %
  if size(Ht,1)==size(Ht,2), zeven = zeven(even,:); end
  % PREDICT %
  offsets = z - P*zeven;
  % UPDATE %
  zeven = zeven + U*offsets;

% Filling in
% Prepare next step
  % bandwidths are assumed to be ordered fine to coarse (this is in contrast to
  % detail coefficients and moments, which are assumed to be ordered coarse to
  % fine)
  nh = size(bandwidths,2);
  if nh == 1, bandwidths = bandwidths*sqrt(splitrate);
  else bandwidths = bandwidths(:,2:nh);
  end
  outputoptions = {outputoptions{:},'bandwidths',bandwidths};

  No = length(xodd); Ne = length(xeven);
  wcoarse = zeros(2*Ne+No,size(sfine,2));
  wcoarse(Ne+1:2*Ne+No,:) = offsets;
  wcoarse(1:Ne,:)         = zeven;

  xcoarse = xeven; ycoarse = yeven;

if transposex, xcoarse = xcoarse'; end
if transposey, ycoarse = ycoarse'; end
if transposew, wcoarse = wcoarse'; end

 
