function [yk sides newsides] = LCRadaptiveestimation1off(estmethod,varargin);

% LCRadaptiveestimation - ThreshLab2/Irregular/1D/ --- adaptive estimation
%   Usage
%      [yk sides newsides] = LCRadaptiveestimation1off(estmethod,arguments)
%   Inputs
%      estmethod          string with valid name of estimation routine
%                         including:
%                               'interpolynomialprediction'
%                               'kernelestimation','locpolestimation00',
%                               'extlocpolestimation00',
%                               'derivlocpolestimation00'
%      list of parameters for estmethod has to be included in the input
%      paremeters specific for LCRadaptiveestimation:
%      'adapt2input' followed by a vector (yin, say)
%                           the input values used in the definition of adaptive
%                           choice (see Notes for formal elaboration)
%      'adapt2output' followed by a vector (yout, say)
%                           the output values used in the definition of
%                           adaptive choice (see Notes for formal elaboration)
%      'critical','criticalvalue','sidethreshold'
%                           threshold/critical value used in the adaptive
%                           decision (see Notes for formal elaboration)
%                           select a fixed number of breakpoints
%      'sides','LCR','adaptinfo' 
%                           followed by a vector with entries in {-1,0,1}
%                           are adaptive choices previously made
%      'fixed', 'fixedsides','predefined'
%                           makes the current routine interprete the
%                           vector in adaptinfo, sides or LCR as previous
%                           choices to be used again in this estimation
%                           Otherwise, new adaptive choices are made and those
%                           are added to the earlier ones.
%                           The presence of one of these strings deactivates
%                           any value of 'adapt2input' and/or 'adapt2output'
%      'forward','back'
%                           will cause the current routine to pick the last
%                           elements in the vector adaptinfo, sides or LCR.
%                           (Within the queue of the vector, the order of
%                           elements is NOT reversed)
%                           Has no effect when 'fixed(sides)' or 'predefined'
%                           is not among the input parameters
%      'backward','inverse','front' 
%                           will cause the current routine to pick the first
%                           elements in the vector adaptinfo, sides or LCR for
%                           interpretation
%                           Has no effect when 'fixed(sides)' or 'predefined'
%                           is not among the input parameters
%      'varinput',Sigma_in
%      'covarinput',Sigma_in   covariance matrix, corresponding to the
%                              observations in 'adapt2input'
%      'varoutput',Sigma_out
%      'covaroutput',Sigma_out covariance matrix, corresponding to the
%                              observations in 'adapt2output'
%      'covarIO',Sigma_io
%              E(Yinput*Youtput') - E(Yinput)*E(Youtput)'
%              default entries are zero unless xin(l) == xout(m) 
%   Outputs
%      yk        real vector of adaptive estimation
%      sides     vector with elements in {-1,0,1}, where newsides has been
%                added or removed (depending on fixedsides) from sides vector 
%                at input
%      newsides  vector of size(yk), with elements in {-1,0,1}, representing
%                the adaptive choices made in the computation of the
%                corresponding element in yk
%   Notes
%      LCRadaptiveestimation choses between a full estimation method and a
%      left- and right-truncated version of it. The choice is based on
%      statistical testing for the presence of a singularity. The presence of a
%      singularity is detected by comparing the output from left-truncated, a
%      right truncated and a two-sided version of the smoothing method.
%
%      Let Pleft be the left truncated version, Pright and Pfull the right and
%      full alternatives, then this routine compares the values
%      Dx = abs(yout-Px*yin)         (for x in {left,right,full})
%      H0 = {no singularity present} is rejected if
%      max(Dfull/Dright,Dfull/Dleft) > criticalvalue
%   See also
%      help interpolynomialprediction
%      help locpolestimation00
%      help kernelestimation
%      help extlocpolestimation00
%      help derivlocpolestimation00


% Default values
criticalvalue = 3;
sides = NaN;
fixedsides = 0;
forward = 0;
Sigma_in = NaN;
Sigma_out = NaN;
Sigma_io = NaN;
yin = NaN;
yout = NaN;
numberofbreakpoints = Inf;
breakpointsoninput = 0;

% Parsing varargin for parameters related to the adaptivity

nvarargin = length(varargin);
listofoptions = {};
k = 1;
while k <= nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'criticalvalue','critical','sidethreshold'}
      criticalvalue = varargin{k+1};
      k = k+2;
   case {'numberofbreakpoints'},
      numberofbreakpoints = varargin{k+1};
      k = k+2;
   case {'breakpointsoninput'},
      breakpointsoninput = varargin{k+1};
      k = k+2;
   case {'sides','LCR','adaptinfo'} % adaptinfo contains information about
                                    % previous adaptive steps, which is
                                    % primarily passed to the calling routine
                                    % Instructions about this and future steps
                                    % should be passed through other variables
      sides = varargin{k+1};
      k = k+2;
   case {'adapt2input'} % y-values used for adaptivity
      yin = column(varargin{k+1});
      k = k+2;
   case {'adapt2output'}
      yout = column(varargin{k+1});
      k = k+2;
   case {'fixed','fixedsides','predefined'},
      fixedsides = 1;
      k = k+1;
   case {'forward','back'}
      forward = 1;
      k = k+1;
   case {'backward','inverse','front'}
      forward = 0;
      k = k+1;
   case {'varinput','covarianceinput','varianceinput','covarinput'}
      Sigma_in = varargin{k+1};
      k = k+2;
   case {'varoutput','covarianceoutput','varianceoutput','covaroutput'}
      Sigma_out = varargin{k+1};
      k = k+2;
   case {'covarIO'}
      Sigma_io = varargin{k+1};
      k = k+2;
   otherwise % the argument cannot be interpreted by this routine and it is
             % passed to the next level
      listofoptions = {listofoptions{:},vark};
      k = k+1;
   end
   else % if ischar(vark
      listofoptions = {listofoptions{:},vark};
      k = k+1;
end, end

% Parsing listofoptions for position y0 and x

posy0 = NaN; posx = NaN; posx0 = NaN;
switch(estmethod)
case {'kernelestimation','locpolestimation00','derivlocpolestimation00'},
    posx = 1; posx0 = 2; posy0 = 3;
case 'interpolynomialprediction', posx0 = 1; posy0 = 2; posx = 3;
otherwise
   noptions = length(listofoptions);
   for k = 1:noptions-1, optionk = listofoptions{k};
   if ischar(optionk), switch(optionk)
   case {'y0','Y'}
      posy0 = k+1;
   case {'x'}
      posx = k+1;
   case {'x0','X'}
      posx0 = k+1;
   end, end, end
end

% y0 is assumed to be arranged as (one or more) rows (cfr.
% extlocpolestimation00.m)
y0 = listofoptions{posy0};
if size(y0,2) == 1, y0 = y0'; end
ny0 = size(y0,2);
Ie = eye(ny0);
listofoptions{posy0} = Ie;
% if no input vector is given for adaptivity, then take (first row of) y0
if isnan(yin),
   yin = y0(1,1:ny0);
end
x = listofoptions{posx};
nx = length(x);
x0 = listofoptions{posx0};
nx0 = length(x0);
if nx0 ~= ny0, error('conflicting lengths x0 and y0'), end

iter = 0;

if fixedsides == 0,

   yin = column(yin);
   yout = column(yout);
   if size(yin,1) ~= ny0,
      size(yin)
      ny0
      error('dimension of adapt2input mismatches')
   end
   switch(estmethod)
   case {'kernelestimation','locpolestimation00','derivlocpolestimation00'},
      listofoptionsL = {listofoptions{:},-1};
      listofoptionsC = {listofoptions{:},0};
      listofoptionsR = {listofoptions{:},1};
   case {'interpolynomialprediction'},
      error('adaptive version polynomial prediction not yet implemented')
   case {'extlocpolestimation00'},
      listofoptionsL = {listofoptions{:},'side',-1,'flexiblebandwidth',1};
      listofoptionsC = {listofoptions{:},'side',0};
      listofoptionsR = {listofoptions{:},'side',1,'flexiblebandwidth',1};
   otherwise
      listofoptionsL = {listofoptions{:},'side',-1};
      listofoptionsC = {listofoptions{:},'side',0};
      listofoptionsR = {listofoptions{:},'side',1};
   end

   Pcentral = truncatedestimation(estmethod,listofoptionsC{:})';
   [n m] = size(Pcentral);
   if n~=nx,
     error('mismatch dimensions at output')
   end
   if isnan(yout(1)) | isnan(yin(1)), 
      yout = zeros(n,1); yin = zeros(m,1);
   end
   searchforjumps = 1;
   if ~isinf(numberofbreakpoints),
      criticalvalue = 0;
   end
   newsides = zeros(n,1);
   while searchforjumps,
      if exist(estmethod,'file') ~= 0,
      Pleft    = ...
      truncatedestimation(estmethod,listofoptionsL{:},'breakpoints',newsides)';
      Pcentral = ...
      truncatedestimation(estmethod,listofoptionsC{:},'breakpoints',newsides)';
      Pright   = ...
      truncatedestimation(estmethod,listofoptionsR{:},'breakpoints',newsides)';
      else
         error('estimation method not found')
      end

      offsetL = yout-Pleft*yin;
      offsetC = yout-Pcentral*yin;
      offsetR = yout-Pright*yin;

      noSigma_io = isnan(Sigma_io(1,1));
      noSigma_in = isnan(Sigma_in(1,1));
      noSigma_out = isnan(Sigma_out(1,1));
      if noSigma_io | noSigma_in | noSigma_out,
         % find upsampling matrix (see function nofiltereffect included below)
         Ups = nofiltereffect(estmethod,listofoptions{:});
      end
      stdev = 1;
      if noSigma_out & noSigma_in,
         Sigma_in = eye(ny0);
         Sigma_out = eye(nx);
         stdev = NaN;
      elseif noSigma_out, % this case not tested
         Sigma_out = Ups*noSigma_in*Ups';
      elseif noSigma_in, % this case not tested
         noSigma_in = Ups'*noSigma_out*Ups;
      end
      if max(size(Sigma_in)) == 1,
         Sigma_in = Sigma_in*eye(ny0);
      end
      if min(size(Sigma_in)) == 1,
         Sigma_in = diag(Sigma_in);
      end
      if max(size(Sigma_out)) == 1,
         Sigma_out = Sigma_out*eye(nx);
      end
      if min(size(Sigma_out)) == 1,
         Sigma_out = diag(Sigma_out);
      end
      
      if noSigma_io,
         Sigma_io = Sigma_in*Ups';
      end

      % Next lines are old versions of covariance matrices; they are incorrect
      % Gleft = Ups-Pleft;
      % Gcentral = Ups-Pcentral;
      % Gright = Ups-Pright;
      % SigmaL = Gleft*Sigma_in*Gleft';
      % SigmaC = Gcentral*Sigma_in*Gcentral';
      % SigmaR = Gright*Sigma_in*Gright';

      SigmaL = Sigma_out + Pleft*Sigma_in*Pleft' - ...
               Pleft*Sigma_io - Sigma_io'*Pleft';
      SigmaC = Sigma_out + Pcentral*Sigma_in*Pcentral' - ...  
               Pcentral*Sigma_io - Sigma_io'*Pcentral';
      SigmaR = Sigma_out + Pright*Sigma_in*Pright' - ...  
               Pright*Sigma_io - Sigma_io'*Pright';

      sigmaL = sqrt(diag(SigmaL));
      sigmaC = sqrt(diag(SigmaC));
      sigmaR = sqrt(diag(SigmaR));
      OL = find(abs(sigmaL)<eps);
      OC = find(abs(sigmaC)<eps);
      OR = find(abs(sigmaR)<eps);

      sigmaL(OL) = 1; sigmaC(OC) = 1; sigmaR(OR) = 1;

      offsetL = offsetL./sigmaL;
      offsetC = offsetC./sigmaC;
      offsetR = offsetR./sigmaR;

      offsetL(OL) = 0; offsetC(OC) = 0; offsetR(OR) = 0;

      if isnan(stdev),
         stdevL = MAD1(offsetL)/invcumgauss(0.75);
         stdevC = MAD1(offsetC)/invcumgauss(0.75);
         stdevR = MAD1(offsetR)/invcumgauss(0.75);
      else
         stdevL = stdev; stdevC = stdev; stdevR = stdev;
      end

      if stdevL > eps, offsetL = offsetL/stdevL; end
      if stdevC > eps, offsetC = offsetC/stdevC; end
      if stdevR > eps, offsetR = offsetR/stdevR; end

      if criticalvalue > 0,
         jumpsides = (abs(offsetC)>criticalvalue);
         jumpsidesL = (abs(offsetL)<criticalvalue & abs(offsetL)<=abs(offsetR));
         jumpsidesR = (abs(offsetR)<criticalvalue & abs(offsetR)<=abs(offsetL));
         jumpsides = jumpsides.*(jumpsidesR-jumpsidesL);
      elseif numberofbreakpoints > 0 & any(abs(offsetC))> eps, 
         jumpsides = ones(size(offsetC));
      else
         jumpsides = zeros(size(offsetC));
      end

      searchforjumps = 0;
      % Stop searching for jumps unless new jumps are detected in this
      % iteration
      while any(jumpsides~=0) & numberofbreakpoints > 0,
         searchforjumps = 1;
         numberofbreakpoints = numberofbreakpoints-1;
         [maxcoef lr] = max(abs(offsetC.*jumpsides));
         s = sign(jumpsides(lr));
         if s<0,
            newsides(lr:lr+1) = [-1 1];
         else
            newsides(lr-1:lr) = [-1 1];
         end
         jumpsides(lr) = 0;
         rangelr = find(abs(Pcentral(lr,1:nx0))>eps);
         rangelr = x0(rangelr);
         xl = min(rangelr); xr = max(rangelr);
         rangelr = find((x<=xr)&(x>=xl));
         jumpsides(rangelr) = 0;
      end    
   end
   if isnan(sides),
      sides = newsides;
   else
      sides = [newsides; sides];
   end
else
   n = nx;
   if breakpointsoninput==1, n = length(x0); end
   ns = length(sides);
   if ns >= n, 
      if forward == 1,
         newsides = sides(ns-n+1:ns);
         sides = sides(1:ns-n);
      else
         newsides = sides(1:n);
         sides = sides(n+1:ns);
      end
   else
      newsides = zeros(1,n);
   end
   Pcentral = ...
      truncatedestimation(estmethod,listofoptions{:},'breakpoints',newsides)';
end

P = Pcentral;
yk = y0*P';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Ups = nofiltereffect(estmethod,varargin);

% this function mainly tries to find x0/xin/xeven and x/xout/xodd
% (this is upsampling from xeven into x)

posx = NaN; posx0 = NaN;
switch(estmethod)
case {'kernelestimation','locpolestimation00','derivlocpolestimation00'},
     posx = 1; posx0 = 2;
case 'interpolynomialprediction', posx0 = 1; posx = 3;
end
if ~isnan(posx),
   x = varargin{posx};
end
if ~isnan(posx0),
   x0 = varargin{posx0};
end

nvarargin = length(varargin);
for k = 1:nvarargin-1, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'x'}
      x = varargin{k+1};
   case {'x0','X'}
      x0 = varargin{k+1};
   end
end, end
Ne = length(x0); No = length(x);
Ups = zeros(No,Ne);
for i = 1:No,
   Ups(i,:) = double(abs(x0-x(i))<eps);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function fhat = truncatedestimation(estmethod,varargin);

% This function truncates the interval on which the estimation is constructed.

listofoptions = {varargin{:}};
breakpoints = NaN;

% Parsing listofoptions for positions and values of parameters x,x0,y0

posx = NaN; posx0 = NaN; posy0 = NaN; y0 = NaN;
switch(estmethod)
case {'kernelestimation','locpolestimation00','derivlocpolestimation00'},
     posx = 1; posx0 = 2; posy0 = 3;
case 'interpolynomialprediction', posx0 = 1; posy0 = 2; posx = 3;
end
if ~isnan(posx),
   x = listofoptions{posx};
end
if ~isnan(posx0),
   x0 = listofoptions{posx0};
end
if ~isnan(posy0),
   y0 = listofoptions{posy0};
end

noptions = length(listofoptions);
for k = 1:noptions-1,
   optionk = listofoptions{k};
   if ischar(optionk), switch(optionk)
   case {'x'}
      x = listofoptions{k+1};
      posx = k+1;
   case {'x0','X'}
      x0 = listofoptions{k+1};
      posx0 = k+1;
   case {'y0','Y'}
      y0 = listofoptions{k+1};
      posy0 = k+1;
   case {'breakpoints'},
      breakpoints = listofoptions{k+1};
   end, end
end

Ne = length(x0); No = length(x); Ie = eye(Ne);
if isnan(y0),
   y0 = Ie;
   listofoptions = {listofoptions{:},'y0',y0};
   posy0 = length(listofoptions);
end

% y0 is assumed to be arranged as (one or more) rows (cfr.
% extlocpolestimation00.m)
rowsy0 = size(y0,1);
fhat = zeros(rowsy0,No);

% 1's in breakpoints correspond to LEFT boundaries of subintervals (those 
% that have to be estimated with a right left-over truncated kernel)
% -1's in breakpoints correspond to RIGHT boundaries of subintervals (those
% that have to be estimated with a left left-over truncated kernel)

if isnan(breakpoints), breakpoints = [1 zeros(1,No-1) 1]; end
breakpoints = row(breakpoints);
breakpoints = find(breakpoints==1);
if isempty(breakpoints), breakpoints = [1 No+1]; end
% the vector breakpoints contains now the left boundaries of each subinterval
if breakpoints(1) ~= 1, breakpoints = [1 breakpoints]; end
nb = length(breakpoints);
if breakpoints(nb) ~= No+1, breakpoints = [breakpoints No+1]; end
nb = length(breakpoints);

for b = 1:nb-1,
   I = (breakpoints(b):breakpoints(b+1)-1);
   nx = length(I);
   xI = x(I); xleft = xI(1); xright = xI(nx);
   J = find(x0>=xleft & x0<=xright);
   nx0 = length(J);
   x0J = x0(J);
   y0J = y0(1:rowsy0,J);
   listofoptions{posx0} = x0J;
   listofoptions{posy0} = y0J;
   listofoptions{posx} = xI;
   fhat(1:rowsy0,I) = sparse(feval(estmethod,listofoptions{:}));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
