
function H = constructprefilter(H,x,varargin);

% constructprefilter -- prefilter in Laplacian pyramid lifting scheme for
%                       multiscale smoothing (FMLPT1D)
%  Usage
%    H = constructprefilter(H,x);
%    H = constructprefilter(H,n);
%    H = constructprefilter(H,x,optional parameters);
%    H = constructprefilter(H,n,optional parameters);
%  Inputs
%    H   character string with name of filter operation
%    x   grid/locations of (mostly nonequidistant) observations
%    n   number of (equidistant) observations
%    optional parameters, depending on choice of H
%  Outputs
%    H   sparse filter matrix
%  Description
%    Unless otherwise indicated by the dimensions of Mfine and Mcoarse, this
%    routine creates a SQUARED prefilter matrix 
%    In other words, it does NOT include subsampling
%    This is in contrast to other prefilter constructions (e.g.
%    prefiltbyvanmom, prefilterorthnotsparseSTCO)
%    The routine FMLPT1D1step can deal with both cases
%  Examples
%    H = constructprefilter('I',x);
%        yields identical matrix
%    H = constructprefilter('Haar',x);
%        yields Haar smoothing (1/2,1/2)
%    H = constructprefilter('Unbalanced Haar',x,Mfine,Mcoarse);
%    H = constructprefilter('UnbHaar',x,Mfine,Mcoarse);
%        for Unbalanced Haar smoothing with integrals of scaling functions in
%        Mfine. Mfine and Mcoarse are optional, but if provided, size(Mfine)
%        and size(Mcoarse) must be equal to size(x)
%    H = constructprefilter('kernel',x,kerneltype,bandwidth,resolutionlevel);
%    H = constructprefilter('kernel',x,...
%          'type',kerneltype,'bandwidth',hJ,'level',j);
%    H = constructprefilter('wavelet',x,ht,sht)
%  See also
%    help FMLPT1D
%    help prefiltbyvanmom
%    help prefilterorthnotsparseSTCO


if length(x) == 1, 
   n = x; x = (0:n-1)/(n-1);
else
   n = length(x);
end

nvarargin = length(varargin);

switch H,
case {'none','I','eye','ident','identical'},
   % H = sparse(eye(n));
   H = 1;
case {'Haar','haar'},
   H = sparse(eye(n)+diag(ones(n-1,1),1))/2; H(n,n) = 1;
case {'Unbalanced Haar','UnbHaar'},
   % The unbalanced Haar transform relies on the moments of two adjacent fine
   % scaling functions and one moment of the coarse scaling function at the
   % same location. The coarse scale moments can be just the sum of the fine
   % scale moment (default), or it can be found by running the refinement or
   % subdivision scheme using the prediction.
   % In principle, this prefilter does not need the covariate values x
   Mfine = NaN; Mcoarse = NaN; even = NaN;
   kk = [];
   for k = 1:nvarargin, if isnumeric(varargin{k}), kk = [kk k]; end, end
   nrealvarargin = length(kk);
   if nrealvarargin > 0, Mfine = varargin{kk(1)}; end
   if nrealvarargin > 1, Mcoarse = varargin{kk(2)}; end
   if nrealvarargin > 2, even = varargin{kk(3)}; end

   if ~issorted(x), Mfine = x; Mcoarse = Mfine; even = Mcoarse; end
   if isnan(Mfine),
      Mfine = initmom(x); 
   elseif min(size(Mfine))> 1,
      Mfine = column(Mfine); Nf = size(Mfine,1); Mfine = Mfine(1:Nf,1);
   end
   Nf = length(Mfine);
   if isnan(Mcoarse), 
      Mcoarse = Mfine + [Mfine(2:Nf); 0];
      % averages with left evens and right odds. For averages with right evens
      % and left odds, use the following line:
      % Mcoarse = Mfine + [0; Mfine(1:n-1)];
   elseif min(size(Mcoarse))> 1,
      Mcoarse = column(Mcoarse); Nc = size(Mcoarse,1);
      Mcoarse = Mcoarse(1:Nc,1);
   end
   Nc = length(Mcoarse);
   if isnan(even), 
      if Nc==Nf, even = (1:Nf); else, even = (1:2:Nf); end
   end
   if length(even) ~= Nc, error('dimension mismatch even and Mcoarse'), end
   Mcc = ones(size(Mfine)); Mcc(even) = Mcoarse;
   H = sparse(diag(Mfine./Mcc)+diag(Mfine(2:Nf)./Mcc(1:Nf-1),1));
   H = H(even,1:Nf);
   % the following line (in comment) for computing averages with right evens
   % and left odds:
   % H = sparse(diag(Mfine./Mcoarse)+diag(Mfine(1:n-1)./Mcoarse(2:n),-1));
   %
   % In the simple Unbalanced Haar transform, 
   % Mcoarse(k) = sum(Mfine(2*k-1:2*k)), but this 
   % need not be the case with general dual lifting steps.
   % In the simple Unbalanced Haar transform, Mfine can be computed from the
   % locations x at this scale, without knowing the finer scales. In general
   % lifting Mfine needs to be updated and transfered from scale to scale.
case {'kernel','Kernel'}
   k = 1;
   hJ = NaN;
   side = NaN;
   j = 0;
   while k<nvarargin,
      varname = varargin{k};
      if ischar(varname),
         switch varname,
         case {'bandwidth','width','scale'},
            hJ = varargin{k+1}; k = k+1;
         case {'type','kerneltype'},
            kerneltype = varargin{k+1}; k = k+1;
         case {'level','resolution'},
            j = varargin{k+1}; k = k+1;
         case {'side','Side'},
            side = varargin{k+1}; k = k+1;
         case {'left','right','L','R','both','twoside','two','center','Center'},
            side = varname;
         otherwise
            if exist('kerneltype') ~= 1,
               kerneltype = varname;
            end
         end
      elseif isnumeric(varargin{k}),
         if isnan(hJ), hJ = varargin{k};
         elseif j == 0, j = varargin{k};
         elseif isnan(side), side = varargin{k};
         end
      end
      k = k+1;
   end
   if k == nvarargin & isnumeric(varargin{k}),
      if isnan(hJ), hJ = varargin{k};
      elseif j == 0, j = varargin{k};
      elseif isnan(side), side = varargin{k};
      end
   end
   if isnan(side), side = 0; end
   N = length(x);
   hj = hJ*2^j;
   H = kernelestimation(row(x),row(x),sparse(eye(N)),hj,kerneltype,side)';
   H = sparse(H);
case {'wavelet','wavfilt','2scale'}
   ht = [1 1]/2; sht = 0;
   if nvarargin > 0, ht = varargin{1}; end
   if nvarargin > 1, sht = varargin{2}; end
   % two lines below are correct but call of toeplitz is inefficient w.r.t.
   % memory use:
   % nht = length(ht); hht = zeros(1,n); hht(1:nht) = ht; hht = frwshift(hht,-sht);
   % H = toeplitz(hht,frwshift(hht(n:-1:1),-1));
   H = sparse(zeros(n,n));
   for k = 1:length(ht),
      o = 1-k-sht;
      H = H + sparse(diag(ones(1,n-abs(o))*ht(k),o));
      % next line for periodic extension only
      if o ~= 0,
         H = H + sparse(diag(ones(1,abs(o))*ht(k),-sign(o)*(n-abs(o))));
      end
   end
otherwise % try simple evaluation
   H = feval(H,x,varargin{:});
end

