
function X = findsparsenullspace(A,midpointsrow,midpointscol,selection,stoptol)

% findsparsenullspace -- finds sparse solutions X for AX=0
%  Usage
%    X = findsparsenullspace(A,midpointsrow,midpointscol,selection);
%    X = findsparsenullspace(A,s,0,selection);
%  Inputs
%    A             n x m sparse matrix
%    midpointsrow  central point on each row of A (optional)
%    midpointscol  central point in each column of A (optional)
%    selection     string, NaN, Inf, or integer vector with p components in 
%                                    {1,..,m}
%    stoptol    stopping criterion on residual (default:1.e-8);
%    s          scalar, indicating strategy (see below)
%  Outputs
%    X          m x p sparse matrix
%  Description
%    finds a sparse solution to the system AX=0. A must have less rows than
%    columns, i.e, n must be equal or smaller than m.
%
%    X has at most m columns of m rows (p=m); selection determines which of
%    these columns are returned; by default selection = NaN; If selection is
%    NaN, then the routine selects a maximal set of independent columns.
%    The search for independent columns is computationally complex.
%    If selection is Inf, then the routine selects all m columns.
%    Some string input ("all", "All") is also accepted.
%
%    The sparsity is constructed along central points in each column of X.
%    In principle, the central points in a column of X are computed from the
%    central points (midpoints) in the sparsity structure of A. The midpoints
%    of A can either be provided explicitly by the user (both for each column
%    and for each row of A), or they can be estimated by the routine. In case A
%    has no (clear) sparsity structure, it is better to impose sparsity on X
%    independent from the structure of A: the i-th column of X (before
%    selection of independent columns) is then centered around i. This is done
%    by 
%    X = findsparsenullspace(A,s,0,selection);
%    scalar s indicates the strategy to be followed as in sparsesolveAXisB
%  Note
%  See also:
%    help findmidpointsinsparsematrix
%    help sparsesolveAXisB
%    help findbiorthfilter

if nargin < 5, stoptol = 1.e-8; end
if nargin < 4, selection = NaN; end
if nargin < 3, midpointscol = NaN; end
if nargin < 2, midpointsrow = NaN; end

if isempty(midpointscol), midpointscol = NaN; end
if isempty(midpointsrow), midpointsrow = NaN; end

if isnan(midpointscol),
   midpointscol = findmidpointsinsparsematrix(A');
end

if isnan(midpointsrow),
   midpointsrow = findmidpointsinsparsematrix(A);
end

[n m] = size(A);
X = zeros(m,1);
if all(size(midpointscol)==[1 1]) & all(size(midpointsrow)==[1 1]),
   midpointscol = (1:m);
end
midpointscol = min(midpointscol,m-1);

% to do: explain the following empirical finding:
% if m is odd, then the last column should not be found using
% sparsesolveAXisB, because that solution will have too large norm
for c = 1:2*floor(m/2),
   minc = [(1:c-1) (c+1:m)];
   Aminc = A(1:n,minc);
   Ac = A(1:n,c);
   mpAminc = midpointsrow;
   mpAminc(mpAminc>c) = mpAminc(mpAminc>c)-1;
   mpAminc(mpAminc>m-1) = m-1;
   Xcminc = sparsesolveAXisB(Aminc,-Ac,mpAminc,midpointscol(c),stoptol);
   Xc = ones(m,1); Xc(minc) = Xcminc;
   X = [X Xc]; 
end
if c<m, % add one more column
   c = m; minc = (1:m-1);
   Aminc = A(1:n,minc); Ac = A(1:n,c);
   Xcminc = pinv(full(Aminc))*(-Ac);
   Xc = ones(m,1); Xc(minc) = Xcminc; X = [X Xc];
end
   
if ischar(selection),
   selection = [selection '     '];
   if selection(1:3) == 'All' | selection(1:3) == 'all',
      selection = (1:m);
   else
      selection = NaN;
   end
end
if isinf(selection(1)),
   selection = (1:m);
end
if isnan(selection(1)),
   selection = 1;
   r = 1; 
   Xs = X(1:m,selection);
   Qs = gramschmidt(Xs);
   % X(1:m,1) was set to be zeros(m,1)
   for c = 2:size(X,2),
% TO DO:
% The following lines code is VERY SLOW and not optimal in the application 
% check if next line should/could be made more efficient: 
      Xc = X(1:m,c);
      Qsc = gramschmidt(Xc,Qs);
      alfa = norm(Qsc(1:n,r+1));
      eps1 = 1.e-10;
      if alfa > eps1,
         r = r+1;
         selection = [selection c];
         Qs = Qsc;
         Xs = X(1:m,selection);
      end
   end
   p = length(selection);
   selection = selection(2:p)-1;
end
   

p = length(selection);
if p>0, X = X(1:m,selection+1); end

% Copyright (c) Maarten Jansen
%
% This software is part of ThreshLab and is copyrighted material.
