
function H = prefiltbyvanmom(P,M,x,even,varargin);

%  prefiltbyvanmom -- finds biorthogonal filter, given vanishing moments 
%  Usage
%    H = prefiltbyvanmom(P,M,x,even,varargin);
%  Inputs
%    P          n x m matrix, with n>m or n=m
%    M          p x n matrix
%    x          vector of length n
%    even       integer vector of length m with elements in {1,...,n}
%  Optional Inputs (varargin)
%  Outputs
%    H          m x n matrix
%  Description
%    finds a sparse solution to the system H*x^p = S2*x^p, where p = 0,...,nt-1
%    and where S2 reresents the downsampling operation S2*u = u(even,:)
%    all this under the side condition that M'*(I-P*H) = 0. 
%    This proceeds in 3 steps:
%    1) find a sparse solution H1 for H*x^p = x^p, by taking H1 = S2*Pp, where
%       Pp is the same operation as the prediction, but based on the full grid
%       (instead of the even subset predicting the full set)
%    2) find a sparse homogeneous solution H0 for H*P = 0
%       It is assumed that x^p belongs to the space generated by P, such that
%       H*P=0 implies H*x^p=0
%    3) find a sparse solution H = H1+X*H0 satisfying M'*(I-P*H) = 0
%       This is a system of equations in unknown X: M'*(I-P*H1) = M'*P*X*H0,
%       which is of the general form A*X*H0 = B,
%       where B = M'*(I-P*H1) and A = M'*P
%       The solution of this system proceeds in two steps
%       3a. U*H0 = B, or H0'*U' = B', which is an overdetermined system
%       3b. A*X = U, which is underdetermined, so imposing sparsity on X is
%                    posible here
%  Note
%    The paper Jansen (2013), IEEE Trans. Sign.Proc.,61(3),pp 545-555
%    uses the notation Htilde_j to denote the prefilter *without subsequent
%    subsampling*. The filter H in this routine *includes* the subsampling!
%    Similarly, the paper uses Pj for the prediction without preceding
%    upsampling. In this routine, P includes the upsampling, making it a
%    rectangular matrix (Pj is a square matrix)
%  See also
%    help sparsesolveAXisB
%    help findsparsenullspace
%    help findmidpointsinsparsematrix
%    help findbiorthfilter (for a strict version)
%    help constructprefilter
%    help prefilterorthnotsparseSTCO


[n m] = size(P);
if length(x) ~= n, error('dimension mismatch P and x'), end
if length(even) ~= m, error('dimension mismatch P and even'), end
if any(abs(even-round(even))>eps),
   error('vector even must not contain non-integer values')
end
if max(even) > n | min(even) < 1,
   error('even must not contain indices outside length(x)')
end

H1 = 'intpol';
nt = 1;
h = (max(x) - min(x)) * 3/n;
kerneltype = 'cos';
% midpoint = NaN;
midpoint = even;
adaptive = 0;
adaptinfo = NaN;
globalvanmom = true;

nvarargin = length(varargin);
otheroptions = {};
k = 1;
while k <= nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'nt','ntilde','ntP','dualmom','dualmoments','ndual','ndualmoments'}
      nt = varargin{k+1};
      k = k+2;
   case {'H','Ht','Htfilter','prefilter','prefilt','filter','Htmethod','method'}
      H1 = varargin{k+1};
      k = k+2;
   case {'bandwidth','bandwidths','h','hj','hJ'}
      h = varargin{k+1};
      k = k+2;
   case {'kerneltype','type'}
      kerneltype = varargin{k+1};
      k = k+2;
   case {'midpoint','midpointP'}
      midpoint = varargin{k+1};
      k = k+2;
   case {'nomoments','nomoment'},
      M = NaN;
      k = k+1;
   case {'adaptive'},
      adaptive = 1;
      k = k+1;
   case {'noglobalmoment'},
      globalvanmom = false;
      k = k+1;
   case {'LCR','sides','adaptinfo'} % information from previous adaptive steps
      % We check explicitly for 'LCR','sides','adaptinfo' as its presence
      % should activate adaptive=1
      adaptive = 1;
      adaptinfo = varargin{k+1};
      k = k+2;
   otherwise % the argument cannot be interpreted by this routine and it is
             % passed to the next level
      otheroptions = {otheroptions{:},vark};
      k = k+1;
   end
   else % if ischar(vark)
      otheroptions = {otheroptions{:},vark};
      k = k+1;
end, end


% following lines are obsolete if we set midpoint equal to even by default
if isnan(midpoint),
   midpoint = findmidpointsinsparsematrix(P');
end

if nargin < 2,
   M = [];
end

xeven = x(even);
I = sparse(eye(n));
if ischar(H1), H1description = [H1 '     '];
   switch(H1description(1:6))
   case {'kernel'}
      H1vars = {xeven,x,I,h,kerneltype};
      H1method = 'kernelestimation';
   case {'inter','polyno','intpol'}
      H1vars = {row(x),I,row(xeven),nt};
      H1method = 'interpolynomialprediction';
   case {'locpol','localp'}
      H1vars = {xeven,x,I,h,kerneltype,nt-1};
      H1method = 'locpolestimation00';
   case {'deriva','differ'}
      H1vars = {xeven,x,I,h,kerneltype,nt-1};
      H1method = 'derivlocpolestimation00';
   otherwise
      H1vars = {'x',xeven,'X',x,'Y',I,'bandwidth',h,...
                 'degree',nt-1,'kerneltype',kerneltype};
      H1method = H1;
   end
   if adaptive == 1,
      H1vars = {H1method,H1vars{:},otheroptions{:},'adaptinfo',adaptinfo,...
               'breakpointsoninput',1};
      % current implementation does not allow any output adaptinfo (so,
      % basically adaptive transform must be predefined) It does not define any
      % adapt2input or adapt2output breakpointsoninput is set to 1, meaning
      % that the definition of breakpoints, coming from the calculation of the
      % prediction filter, does not match its use in the prefilter. Indeed, the
      % breakpoints are defined on the locations of the output of the
      % prediction filter, which are the input locations for the prefilter.
      H1 = feval('LCRadaptiveestimation',H1vars{:});
      H1 = sparse(H1'); % mind the transpose
   elseif exist(H1,'file') ~= 0,
      H1vars = {H1vars{:},otheroptions{:}};
      H1 = sparse(feval(H1,H1vars{:})');
   else
      H1 = 1;
   end
end
if isscalar(H1), H1 = sparse(diag(H1*ones(1,n))); H1 = H1(even,1:n); end
if size(H1) == [n n], H1 = H1(even,1:n); end
if size(H1) ~= [m n], error('incorrect size proposed prefilter'); end


Im = eye(m);

if isempty(M) | isnan(M) | globalvanmom == false,
   H = H1;
else
   xo = mean(x); x = column(x)-xo;
   Xp = ones(n,1); for p = 1:nt-1, Xp = [Xp x.^p]; end

   % H0 = findsparsenullspace(Xp',0,0,Inf)'
   H0 = findsparsenullspace(Xp',0,0,even)';
   % We restrict to a selection (even) of the null space. Taking the whole null
   % space would lead to a vector X with more columns than rows, for which
   % sparsesolveAXisB requires a vector midpointX other than the default (and
   % hence a specific treatmeant)

   In = eye(n);
   B = M'*(In-P*H1);
   % U = (pinv(full(H0'))*B')';
   U = ((H0*H0')\(H0*B'))';
   if any(any(isnan(U))), % i.e., if (H0*H0') was not full rank
      % U = (pinv(full(H0'))*B')';
      H0 = findsparsenullspace(Xp',0,0)';
      % This H0 will be full rank, it probably contains less rows than the 
      % previous one, but if it has more rows, we select m rows
      if size(H0,1) > m,
         H0 = H0(1:m,1:n);
      end
      r = min(rank(H0),size(B,1));
      B = B(1:r,1:n);
      M = M(1:n,1:r);
      U = ((H0*H0')\(H0*B'))';
   end

   A = M'*P;
   X = sparsesolveAXisB(A,U,0,(1:n));
   H = H1+X*H0;
end

% Copyright (c) Maarten Jansen
%
% This software is part of ThreshLab and is copyrighted material.
