
function [ff moments odds xxx] = refinescalingfunctionsMLPT(xx,x,varargin);

% refinescalingfunctionsMLPT -- ThreshLab2/MultiscaleKernel --
%   This function refines MLPT scaling functions
%   Usage
%     ff = refinescalingfunctionsMLPT(xx,x,f,M,varargin);
%     PhiL = refinescalingfunctionsMLPT(xJ,xL,M,varargin);
%   Inputs
%     xx        vector of grid at superresolution (finest level)
%     x         vector of current fine scale grid (coarsest level in refinement)
%     xJ        vector of grid at finest resolution level
%     xL        vector of knots in which scaling functions are centered
%     f         vector (or matrix) with length equal to length(x)
%               observations at fine scale grid or fine scaling coefficients in
%               a Multiscale Local Polynomial Transform (MLPT)
%               default value: eye(length(x))
%     M         number of refinement steps; default value: 1
%     vararging parameters for MLPT
%  Outputs
%     ff        vector (or matrix) with length equal to length(xx)
%     PhiL      MLPT scaling functions defined by knots in xL and refined on xJ
%  Description
%     This routine can be used to evaluate the reconstruction from a MLPT 
%     In particular, it can be used to refine scaling functions beyond the
%     current fine resolition up to superresolution.
%  Notes
%     (1) If the bandwidths in the refinement are specified by a single number,
%     this is the finest scale of the MLP refinement. 
%     As refinescalingfunctionsMLPT is typically used for refinement beyond the
%     finest scale of the MLP transform, the specified finest scale should be
%     dependent on the number of refinement steps: see the example below where
%     hh = h0/2^M;
%     (2) The refinement at the coarsest scale (i.e., the first step in the
%     refinement scheme) is non-dyadic. In order to have convergence of 
%     subdivision, it is necessary to keep the same number of refinement points
%     at the coarsest scale: if M grows, then so should length(xx). More
%     precisely, if M<-M+1, then length(xx)<-2*length(xx)-1
%     (3) Compare the refinement scheme in this routine with that in
%     Bsplinebasisbyrefinement or DDbasisbyrefinement:
%     In these two refinement functions, the refinement is dyadic at all
%     scales.
%  Examples
%     (1) Nondyadic refinement at the level of the observations (dyadic
%     refinement at finer, superresolution level)
%     n = 20; nn0 = 125; M = 3;
%     x = makegrid(n,1); xx = makegrid(2^M*nn0-n-1,0); xx = unique([x xx]);
%     kerneltype = 'cos'; h0 = 0.25; hh = h0/2^M; 
%     Pmethod = 'extlocpolestimation00';
%     decpar = {M,'P',Pmethod,'kerneltype',kerneltype,'nt',2,'bandwidth',hh,...
%               'convex',true};
%     Phi = refinescalingfunctionsMLPT(xx,x,decpar{:});
%     plot(xx,Phi)
%     (2) dyadic refinement all the way (this may lead to
%     heterogeneous/intermittent scales of refinement within a single
%     resolution level)
%     rand('state',2);
%     n = 7; M = 8; m = 2^M; nn = m*(n-1)+1;
%     x = makegrid(n,1,true); xx = polyline(1:nn,1:m:nn,x);
%     decparDD = {M,'P','intpol','nt',4};
%     PhiDD4mlpt = refinescalingfunctionsMLPT(xx,x,decparDD{:});
%     PhiDD4wt = DDbasisbyrefinement(xx,x,4);
%     plot(xx,PhiDD4mlpt,'k','linewidth',3)
%     hold on
%     plot(xx,PhiDD4wt,'r')
%     hold off
%  See also
%     help Bsplinebasisbyrefinement
%     help DDbasisbyrefinement


x = row(x);
xx = row(xx);
n = length(x);
nn = length(xx);
f = eye(n);
M = 1;
nvarargin = length(varargin);
k = 1; vark = varargin{k};
while isnumeric(vark), 
   if any(size(vark)==n), f = vark; end
   if size(vark) == [1 1], M = vark; end
   k=k+1; if k<=nvarargin, vark = varargin{k}; else, vark = 'the end'; end
end
decparplus = varargin(k:nvarargin);
odds = NaN;
while k <= nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'split','splitscheme','partitioning','takeoff','takeaway'};
      odds = varargin{k+1};
      k = k+2;
   otherwise % the argument need not be interpreted by this routine 
      k = k+1;
   end
   else % if ischar(vark)
      k = k+1;
end, end
 

if size(f,1) ~= n, f=f'; end
if size(f,1) ~= n, error('dimension mismatch x and f'), end
m = size(f,2);

if M>0,
   xx0 = setdiff(xx,x); % points of xx that are no knots
   x0 = setdiff(x,xx);  % knots not in xx
   x = setdiff(x,x0);   % knots that are in xx
   n0 = length(x0);     % number of knots outside xx
   [xxx orderx] = sort([x0 x xx0]); 
   nnn = length(xxx);
   invorderx = (1:nnn);
   invorderx(orderx) = (1:nnn);
   knots = invorderx(1:n);
   ff = (1:nnn); 
   % ff contains the indices to refinement points in xxx
   out = sort(invorderx(n0+1:nnn));
   % out has the indices to the points of xx in xxx, the evaluations in these
   % points will be in the output
   if isnumeric(odds), if isnan(odds),
      odds = cell(1,M);
      for j = 1:M-1,
         % ff are indices (see above)
         [e o] = splitevenodd(ff); % subdivision of refinement points
         oo = ff(o); ee = ff(e);
         % knots must be prevented from being odd
         ee = sort(unique([ee knots])); oo = setdiff(oo,knots);
         odds{j} = oo; % odds{j} now contains the indices into the full xxx
         ff = ee;
      end
      % at coarsest scale, all points that are not original knots are odds
      odds{M} = setdiff(ff,knots);
      % at the coarsest level, the number of odds (in odds{M}) may be much
      % larger than the length of evens
      odds = findoddsinfinestgrid(odds,nnn,true);
      % odds{j} now contains the indices into the knots at level j+1
      decparplus = {decparplus{:},'split',odds};
% todo: if odds are given, its values should be made consistent with the
% creation of merged grid xxx
   end, end % if isnan(odds),
   oo = FMLPT1D(xxx,zeros(size(xxx)),M,decparplus{:});
   noo = length(oo) - size(f,1);
   f = [f; zeros(noo,m)];
   fff = IMLPT1D(xxx,f,M,decparplus{:});
   ff = fff(out,1:m);
   if nargout>1,
      o = sparse(zeros(size(xxx)));
      [o oo moments] = FMLPT1D(xxx,o,M,decparplus{:},'U',0);
      p = size(moments,2);
      if p>0, moments = moments(1:n,1:p); end
   end
else
   xm = [x(1) (x(1:n-1)+x(2:n))/2 x(n)];
   ff = staircase(xx,xm,[f;zeros(1,m)]);
   moments = initmom(x);
end
