
function X = sparsesolveAXisB(A,B,midpointA,midpointB,stoptol)

% sparsesolveAXisB -- finds sparse solutions X for AX=B
%  Usage
%    X = sparsesolveAXisB(A,B,midpointA,midpointB,stoptol);
%    X = sparsesolveAXisB(A,B,midpointA);
%    X = sparsesolveAXisB(A,B,midpointX,s);
%    X = sparsesolveAXisB(A,B,s,midpointX);
%    X = sparsesolveAXisB(A,B);
%  Inputs
%    A          n x m sparse matrix
%    B          n x p sparse matrix
%    midpointA  central point on each row of A
%    midpointB  central point in each column of B
%    midpointX  central point in each column of X
%    s          scalar, indicates strategy 
%               (edit sparsesolveAXisB.m for details)
%    stoptol    stopping criterion on residual (default:1.e-8);
%  Outputs
%    X          m x p sparse matrix
%  Description
%    finds a sparse solution to the system AX=B. A must have less rows than
%    columns, i.e, n must be equal or smaller than m.
%    Sparsity is constructed around central elements (midpoints) These
%    midpoints are either given explicitly w.r.t. the matrix X to be computed,
%    or computed from midpoints in A and B. The latter choice is the default
%    option when no midpoints parameters are given.
%  Note
%  See also:
%    help findmidpointsinsparsematrix
%    help findsparsenullspace

if nargin < 5, stoptol = 1.e-8; end
[n m] = size(A);
if n > m, error('A should have at least as many columns as rows'), end
% rankA = rank(full(A));

if nargin < 2,
   B = eye(n);
end

[nB p] = size(B);
if nB ~= n, error('dimension mismatch A and B'), end

if nargin < 3,
   midpointA = findmidpointsinsparsematrix(A);
end

if nargin < 4,
   midpointB = findmidpointsinsparsematrix(B')';
end

if all(size(midpointA)==[1 1]),
   s = midpointA;
   midpointA = midpointB;
   midpointB = s;
end

if size(stoptol) == 2,
   stopnorm = stoptol(2);
   stoptol = stoptol(1);
else
   stopnorm = 2;
end

X = zeros(m,p);

if all(size(midpointB) == [1 1]),
   % preliminary solution
   midpointB = midpointB(1);
   if length(midpointA)>p, midpointA = midpointA(1:p); end
   if length(midpointA)<p, error('insufficient elements in midpointX'), end
   x = sum(B(1:n,1:p),1)./sum(A(1:n,midpointA),1);
      % sums over all rows avoiding 0/0
      % This mean solution could be changed into LS:
      % sum(B(1:n,1:p).*A(1:n,midpointA),1)./ ...
      %  sum(A(1:n,midpointA).*A(1:n,midpointA),1)
   X(midpointA,1:p) = diag(x);
else
   %subsequent code assumes ordered midpoints in A
   [midpointA oo] = sort(midpointA);
   A = A(oo,1:m); B = B(oo,1:p);

   diagA = diag(A(1:n,midpointA));
   diagA = diagA(midpointB);
   diagA0 = find(abs(diagA)<eps);
   diagB = diag(B(midpointB,1:p));
   x = row(diagB)./row(diagA); x(diagA0) = 0;
   X(midpointA(midpointB),1:p) = diag(x);

end

% find a sparse solution of AX=B, column by column
if all(size(midpointB)==[1 1]),
   % then, midpointA contains actually midpointX, so for every column of X, the
   % midpoint is passed directly (in the other case, midpointX is computed from
   % midpointA and midpointB)
   strategy = midpointB;
   switch strategy,
   case {0,1}, % find solution in least squares sense, column by column
      ii = (0:2*n);
      dd0 = (-1).^(ii+1).*ii; dd0 = cumsum(dd0);
      for i=1:p,
         dd = midpointA(i)+dd0; dd = dd(find(dd>0)); dd = dd(find(dd<=m)); 
         ndd = min(n,length(dd)); dd = dd(1:ndd);
         Xi = X(1:m,i); Bi = B(1:n,i);
         l = 1; s= 1;
         Ri = Bi - A*Xi;
         while max(abs(Ri)) > stoptol,
            if s<n,
               l = l+1; s = s+1; ll = dd(1:l);
               All = A(1:n,ll);
               AllTAll = All'*All;
               % rankAll = rank(full(All));
               rankAllTAll = rank(full(AllTAll));
               if rankAllTAll < l,
                  % if rank(All'*All) is less than l, 
                  %    chose another column of A to add.
                  dd = [dd(1:l-1) dd(l+1:ndd) dd(l)];
                  l = l-1;
               else
                  if n == l, % i.e., if size(All,1) == size(All,2),
                     Xll = All\Bi;
                  else
                     Xll = (AllTAll)\(All'*Bi);
                  end
                  Xi(ll) = Xll;
                  Ri = Bi-A*Xi;
               end
            else % no submatrix All with n rows and l (less or equal to n)
                 % columns has been found
               if midpointB ==1 & l<n, 
                       % try one last time: maximal submatrix (n x n)
                       % square system has better condition than system of
                       % normal equations.
                  l = n; % no next time
                  Add = A(1:n,dd);
                  rankAdd = rank(full(Add));
                  if rankAdd == n,
                     Xdd = Add\Bi; Xi(dd) = Xdd; Ri = Bi-A*Xi; 
                  end
               else
                  % emergency exit
                  Ri = 0;
                  Xi = pinv(full(A))*Bi;
               end
            end
         end
         X(1:m,i) = Xi;
      end
   case 2, % find a sparse solution, column by column, in minimum norm sense
      ii = (0:2*m);
      dd0 = (-1).^(ii+1).*ii; dd0 = cumsum(dd0);
      for i=1:p,
         dd = midpointA(i)+dd0; dd = dd(find(dd>0)); dd = dd(find(dd<=m));
         ndd = min(m,length(dd)); dd = dd(1:ndd);
         Bi = B(1:n,i);
         l = n; 
         while l<m,
            ll = dd(1:l);
            All = A(1:n,ll);
            AllAllT = All*All';
            rankAllAllT = rank(full(AllAllT));
            if rankAllAllT == n,
               Xll = (All'/AllAllT)*Bi;
               if norm(Xll,stopnorm)<stoptol, l = m; end
            end
            l = l+1;
         end
         X(ll,i) = Xll;
      end
   case 3, % find sparse solution, minimizing the Frobenius-norm of X-I
      ii = (0:2*m);
      dd0 = (-1).^(ii+1).*ii; dd0 = cumsum(dd0);
      for i=1:p,
         dd = midpointA(i)+dd0; dd = dd(find(dd>0)); dd = dd(find(dd<=m));
         ndd = min(m,length(dd)); dd = dd(1:ndd);
         Bi = B(1:n,i);
         Ai = A(1:n,midpointA(i));
         l = n+1;
         while l<m,
            ll = dd(1:l);
            All = A(1:n,ll);
            AllAllT = All*All';
            rankAllAllT = rank(full(AllAllT));
            if rankAllAllT == n, 
               Xll = [1; zeros(l-1,1)]+(All'/AllAllT)*(Bi-Ai);
               l = m;
            end
            l = l+1;
         end
         X(ll,i) = Xll;
      end
   end
else
   ii = (0:2*n);
   dd0 = (-1).^(ii+1).*ii; dd0 = cumsum(dd0);

   for c=1:p,
      i = midpointB(c);
      j = midpointA(i);
      dd = i+dd0;
      dd = dd(find(dd>0)); dd = dd(find(dd<=n));
      dd = dd(find(dd+j-i>0)); dd = dd(find(dd+j-i<=m));
      ndd = length(dd);
      Xc = X(1:m,c); Bc = B(1:n,c);
      l = 1; s = 1;
      Rc = Bc-A*Xc;
      while max(abs(Rc)) > stoptol,
         if s<n,
            l = l+1; s = s+1; ll = dd(1:l);
            All = A(ll,ll+j-i);
            rankAll = rank(full(All));
            if rankAll < l,
               dd = [dd(1:l-1) dd(l+1:ndd) dd(l)];
               l = l-1;
            else
               Bll = Bc(ll);
               Xll = All\Bll;
               Xc(ll+j-i) = Xll;
               Rc = Bc-A*Xc;
            end
         else
            Rc = 0;
            Xc = pinv(full(A))*Bc;
         end
      end
      X(1:m,c) = Xc;
   end
end

% Copyright (c) Maarten Jansen
%
% This software is part of ThreshLab and is copyrighted material.
