function [U, We, Wet] = updateLSapprxprimmomQ(UP,nu,Me,Mo,We,Wet,dd,qe,qo)

% updateLSapprxprimmomQ - ThreshLab2/Irregular/1D/ applies updateLSapprxprimmom
% to a subset of rows in filters
%   Usage
%      [U, We, Wet] = updateLSapprxprimmomQ(UP,nu,Me,Mo,We,Wet,dd,qe,qo);
%   Inputs
%      UP  cell array of filters H,G,Ht,Gt
%      nu  integer; number of free parameters; i.e., nonzero diagonals in U
%      Me  matrix; column p contains integral of x^(p-1)*phi_e(x), where 
%                  phi_e(x): basis function on even branch of lifting scheme
%                  *after* application of all steps in UP
%      Mo  matrix; column p contains integral of x^(p-1)*phi_o(x), where 
%                  phi_o(x): basis function on odd branch of lifting scheme
%                  *after* application of all steps in UP
%      We  matrix; submatrix of inverse wavelet transform corresponding to the
%                  current scaling coefficients for the reconstruction of the
%                  finest scaling coefficients
%      Wet matrix; submatrix of forward wavelet transform mapping fine scaling
%                  coefficients onto current scaling coefficients
%      dd      vector or matrix of indices, with two possible interpretations
%              (1) integer vector of length n, containing list of nonzero
%                  diagonals: dd(d) is the difference k-l for an element U(k,l)
%                  on the d-th diagonal
%                  (n being the number of vanishing moments)
%              (2) n x No matrix of integers, containing for each column of U
%                  the list of nonzero rows
%      qe,qo   arrays of integers, list of active evens and odds, i.e., 
%              U(i,j) = 0 if i is not qe or j is not in qo
%   Outputs
%      U   matrix: update step
%      We  matrix: coarser scale version of input We
%      Wet matrix: coarser scale version of input Wet
%   Description
%   Reference
%   Note
%   See also
%      help updateLSapprxprimmom

if length(UP) == 5, UP = UP(2:5); end
H = UP{1}; G0 = UP{2}; Ht0 = UP{3}; Gt = UP{4};
Ne = size(H,2);
No = size(G0,2);
N = Ne+No;
U = zeros(Ne,No);
Neq = length(qe);
% Noq = length(qo);
if Neq > 0,
   % select active parts of filters and moments
   Hq = H(1:No,qe); Ht0q = Ht0(1:No,qe); Meq = Me(qe,:);
   G0q = G0(1:No,qo); Gtq = Gt(1:No,qo); Moq = Mo(qo,:);
   % first make an update for the vanishing moments
     % we impose: dd(1:nu,qo) = qe(ddq); the inverse of this operation is:
     dd(find(dd<1))=Ne+1;
     qeinv = zeros(1,Ne+1); qeinv(qe) = (1:Neq); ddq = qeinv(dd(1:nu,qo));
     % indeed: qeinv(qe) = (1:Neq) => qeinv(dd(1:nu,qo)) = qeinv(qe(ddq)) = ddq
   U0q = updateprimalmoments(Moq,Meq,ddq);
   % Then extend the bandwidth in order to make transform as close as possible
   % to orthogonal
   % Do not include zero lines of Gtq
tol = 1.e-8;
   r = find(sum(abs(Gtq))>tol);
   Gtqr = Gtq(1:No,r); G0qr = G0q(1:No,r); Moqr = Moq(r,:);
   UPqr = {'filterbank',Hq,G0qr,Ht0q,Gtqr};
   U0qr = U0q(1:Neq,r); ddr = ddq(1:nu,r);
   % Uqr = updateLSapprxprimmom(UPqr,nu,Meq,U0qr,We,Wet,ddr,'Krylov');
   Uqr = updateLSapprxprimmom(UPqr,nu,Meq,U0qr,We,Wet,ddr);
   Uq = zeros(size(U0q));
   Uq(1:Neq,r) = Uqr;
   U(qe,qo) = Uq;
end
We = We*H;
Wet = Wet*Ht0 + Wet*Gt*U';
