
function [y Dky] = weightedLSpol(x,x0,y0,w0,p,warninglowerdegree);

% weightedLSpol - ThreshLab2/Irregular/MultiscaleKernel/ --- 
%                                          Weighted Least Squares Polynomial
%   Usage
%      y = weightedLSpol(x,x0,y0,w0,p);
%      [y Dky] = weightedLSpol(x,x0,y0,w0,p);
%   Inputs
%      x      x-coordinates in which polynomial is evaluated
%      x0     x-coordinates of locations where polynonial is constructed
%      y0     y-coordinates of locations where polynonial is constructed
%      w0     weights corresponding to y0 (default = 1)
%      p      degree of polynomial
%      warninglowerdegree (optional) binary value; if true, routine produces
%                         warning when y0 is contains not enough values to
%                         define a p-degree polynomial (default false)
%   Outputs
%      y      polynomial evaluated in x
%      Dky    derivatives of y
%   Notes
%   See also

narginfilled = nargin;

n = length(x);
n0 = length(x0);
if narginfilled < 4, w0 = ones(n0,1); narginfilled = narginfilled +1; end
if n0 > 1 & length(w0) == 1,
   if nargin > 4, warninglowerdegree = p; end
   p = w0; w0 = ones(n0,1); narginfilled = narginfilled +1;
end
if narginfilled < 5, p = 0; narginfilled = narginfilled +1; end
if narginfilled < 6,
   warninglowerdegree = false; narginfilled = narginfilled +1;
end


if ischar(warninglowerdegree),
switch warninglowerdegree,
case {'on','ON','y','yes','true','TRUE'},
   warninglowerdegree = true;
otherwise
   warninglowerdegree = false;
end
end

if size(y0,1) ~= n0, y0 = y0'; end
if size(y0,1) ~= n0, error('dimension mismatch x0 and y0'); end
if size(w0,1) ~= n0, w0 = w0'; end
if size(w0,1) ~= n0, error('dimension mismatch y0 and w0'); end

tol = 1.e-10;
nonzeroweights = find(abs(w0)>tol);
w0 = w0(nonzeroweights);
x0 = x0(nonzeroweights);
y0 = y0(nonzeroweights,:);
n0 = length(nonzeroweights);

if n0-1<p && warninglowerdegree == true,
   wrn = ['# knots (=' num2str(n0) ') insufficient for LS pol. degree p='...
          num2str(p)];
   warning(wrn)
end
p = min(p,n0-1);
x0 = column(x0);
xo = mean(x0);
X0 = zeros(n0,p+1); X0(1:n0,1) = ones(n0,1); 
for k = 1:p, X0(1:n0,k+1) = (x0-xo).^k; end
% working with orthogonal polynomials would be more stable
W0 = diag(w0);

X0TW0X0 = X0'*W0*X0;
if cond(X0TW0X0) < 1.e8,
   coef = X0TW0X0\(X0'*W0*y0);
else
   coef = pinv(X0TW0X0)*(X0'*W0*y0);
end
nc = size(coef,2);

transposeresult = 0;
if size(x,2) > 1 & size(x,1) == 1, transposeresult = 1; x = x'; end
X = zeros(n,p+1); X(1:n,1) = ones(n,1);
for k = 1:p, X(1:n,k+1) = (x-xo).^k; end

y = X*coef;
if transposeresult, y = y'; end
if nargout > 1,
   Dky = zeros(n,p+1,nc);
   for k = 1:p+1,
      takederiv = repmat((1:p)',1,nc);
      coef = [takederiv.*coef(2:p+1,1:nc); zeros(1,nc)];
      Dky(1:n,k,1:nc) = X*coef;
   end
   if transposeresult, Dky = Dky'; end
end

if n0 == 0, y = zeros(n,nc); end
