
function LL = LLmargmix2Laplplusnormal(pp,aa0,aa1,stdev,y);

% LLmargmix2Laplplusnormal -- marginal log-likelihood for mixture of
% two Laplace RVs + normal noise
%   Usage
%      LL = LLmargmix2Laplplusnormal(pp,aa0,aa1,stdev,Y);
%      LL = LLmargmix2Laplplusnormal(pa,stdev,Y);
%   Inputs
%      Y     observations
%      pp    sparsity parameter value
%      aa0   parameter Laplace density
%      aa1   parameter Laplace density
%      pa    [pp aa0 aa1] or [pp; aa0; aa1]
%      stdev value for st.dev. of noise
%   Outputs
%      LL    marginal log likelihood of p given Y
%   Description
%      The model is Y = X + stdev*Z, with
%      p = P(V =/= 0)
%      V|V=/=0 ~ Laplace(a1)
%      V|V=0 ~ Laplace(a0)
%      Z ~ NID(0,1)
%   See also
%      help LLmargmixLaplnormplusnormal (mixture of double exp. & normal)
%      help LLmarg0inflLaplaceplusnormal (special case for ttau=0)

if nargin < 4,
   y = aa1;
   stdev = aa0;
   pa = pp;
   [s1 s2] = size(pa);
   if s2 == 3,
      pp = pa(:,1); aa0 = pa(:,2); aa1 = pa(:,3);
   elseif s1 == 3,
      pp = pa(1,:); aa0 = pa(2,:); aa1 = pa(3,:);
   else
      error('inconsistent input variables')
   end
end
na0 = length(aa0); na1 = length(aa1); np = length(pp);
nmax = max([na0 na1 np]);
if na0 < nmax,
   aa0 = aa0(1)*ones(1,nmax); aa1 = row(aa1); pp = row(pp);
end
if na1 < nmax,
   aa1 = aa1(1)*ones(1,nmax); aa0 = row(aa0); pp = row(pp);
end
if np < nmax,
   pp = pp(1)*ones(1,nmax); aa0 = row(aa0); aa1 = row(aa1);
end
np = length(pp); 
LL = zeros(size(pp));
for i = 1:np, 
   p = pp(i);
   a0 = aa0(i);
   a1 = aa1(i);
   LL(i) = sum(log(p*laplaceplusnormalpdf(y,a1,stdev) + ...
                   (1-p)*laplaceplusnormalpdf(y,a0,stdev)));
end
