
function F = laplaceplusnormalcdf(x,a,stdev);

% laplaceplusnormalcdf -- cumulative distribution function of Y = T+stdev*Z
%                         with T ~ Laplace(a) and Z ~ N(0,1)
%   Usage
%     F = laplaceplusnormalcdf(x,a,stdev);
%   Inputs
%     x     value in which cdf is evaluated
%     a     parameter of Laplace distribution (default 0.5)
%     stdev st.dev. of normal distribution (default 1)
%   Outputs
%     F     cdf of  Y = T+stdev*Z
%   Description
%     the definition of the Laplace distribution used here has pdf
%     f(t) = (a/2)*exp(-a*|t|)
%     i.e., T ~ Laplace(a), then E(T) = 0; var(T) = 2/a^2
%   See also
%     help laplaceplusnormalpdf
%     help JScauchyplusnormalcdf


if nargin < 3, stdev = 1; end
if nargin < 2, a = 1/2; end

[n1 n2] = size(x);
n = n1*n2;
reshapeF = false;
if n1>1 & n2>1,
   x = reshape(x,1,n);
   reshapeF = true;
end
[n1 n2] = size(a);
if n1*n2>1 & n>1 & n1*n2 ~= n,
   error('dimension dismatch a and x');
elseif n1>1,
   a = reshape(a,1,n);
end
[n1 n2] = size(stdev);
if n1*n2>1 & n>1 & n1*n2 ~= n,
   error('dimension dismatch stdev and x');
elseif n1>1,
   stdev = reshape(stdev,1,n);
end

nx = length(x);
na = length(a);
ns = length(stdev);
if nx > 1, 
   if na == 1, a = a*ones(size(x)); end
   if ns == 1, stdev = stdev*ones(size(x)); end
elseif na > 1,
   if nx == 1, x = x*ones(size(a)); end
   if ns == 1, stdev = stdev*ones(size(a)); end
elseif ns > 1,
   if nx == 1, x = x*ones(size(stdev)); end
   if na == 1, a = a*ones(size(stdev)); end
end
   


eps = 1.e-308;
% take eps such that 1/eps is still finite. It may happen that eps > 0, but
% 1/eps yields Inf.
% (1) General case
s = sign(x);
oo = find(gauss(abs(x)+stdev.^2.*a,stdev)>eps);
xmax = max([min(abs(x)) abs(x(oo))]);
     % xmax is the largest value in which the exact definition of F is
     % evaluated (larger values may cause troubles)
     % if oo is empty, xmax will be min(abs(x))
xx = s.*min(abs(x),xmax);
F = cumgauss(xx,stdev) + ...
    (1/2)*gauss(xx,stdev).*...
           (-cumgauss(xx-stdev.^2.*a,stdev)./gauss(xx-stdev.^2.*a,stdev)+...
            Rcumgauss(xx+stdev.^2.*a,stdev)./gauss(xx+stdev.^2.*a,stdev));
% (2) at least one of the arguments of gauss and cumgauss close to Inf
oo = find(gauss(abs(x)+stdev.^2.*a,stdev)<eps);
xabs = abs(x);
Fapprox = cumgauss(xx,stdev) + ...
          (-s/2).*exp(stdev.^2.*a.^2/2).*exp(-a.*xabs) + ...
          (s/2).*gauss(xabs,stdev).*stdev.^2./(xabs+stdev.^2.*a);
F(oo) = Fapprox(oo);
% (2a) avoid problems for stdev really close to 0 
oo = find(-xabs+stdev.^2.*a<eps);
Fapprox = -s.*exp(-a.*xabs)/2 + (s+1)/2;
F(oo) = Fapprox(oo);
% (3) a -> Inf, i.e., E|Laplace| -> 0
oo = find(a>1/eps);
F(oo) = cumgauss(x(oo),stdev(oo));
% (4) a -> 0, i.e., E|Laplace| -> Inf
oo = find(a<eps);
F(oo) = 0.5+isinf(x(oo))/2.*s(oo);
if reshapeF == true,
   F = reshape(F,n1,n2);
end
