
function f = laplaceplusnormalpdf(x,a,stdev);

% laplaceplusnormalpdf -- probablity density function of Y = T+stdev*Z
%                         with T ~ Laplace(a) and T ~ N(0,1)
%   Usage
%     f = laplaceplusnormalpdf(x,a,stdev);
%   Inputs
%     x     value in which pdf is evaluated
%     a     parameter of Laplace distribution (default 0.5)
%     stdev st.dev. of normal distribution (default 1)
%   Outputs
%     f     pdf of  Y = T+stdev*Z
%   Description
%     the definition of the Laplace distribution used here has pdf
%     f(t) = (a/2)*exp(-a*|t|)
%     i.e., T ~ Laplace(a), then E(T) = 0; var(T) = 2/a^2
%   See also
%     help laplaceplusnormalcdf
%     help JScauchyplusnormalpdf


if nargin < 3, stdev = 1; end
if nargin < 2, a = 0.5; end

[n1 n2] = size(x);
n = n1*n2;
reshapef = false;
if n1>1 & n2>1,
   x = reshape(x,1,n);
   reshapef = true;
   sizef = [n1 n2];
end
[n1 n2] = size(a);
if n1*n2>1 & n>1 & n1*n2 ~= n,
   error('dimension dismatch a and x');
elseif n1>1,
   a = reshape(a,1,n);
end
[n1 n2] = size(stdev);
if n1*n2>1 & n>1 & n1*n2 ~= n,
   error('dimension dismatch stdev and x');
elseif n1>1,
   stdev = reshape(stdev,1,n);
end

nx = length(x);
na = length(a);
ns = length(stdev);
if nx > 1,
   if na == 1, a = a*ones(size(x)); end
   if ns == 1, stdev = stdev*ones(size(x)); end
elseif na > 1,
   if nx == 1, x = x*ones(size(a)); end
   if ns == 1, stdev = stdev*ones(size(a)); end
elseif ns > 1,
   if nx == 1, x = x*ones(size(stdev)); end
   if na == 1, a = a*ones(size(stdev)); end
end

tol = 1.e-12;
eps = 1.e-308;
% take eps such that 1/eps is still finite. It may happen that eps > 0, but
% 1/eps yields Inf.
s = sign(x);
oo = find(~isinf(a) & stdev>tol & gauss(abs(x)+stdev.^2.*a,stdev)>eps);
if isempty(oo), 
   xmax = min(abs(x));
else
   xmax = max([min(abs(x)) row(abs(x(oo)))]); 
end
xx = s.*min(abs(x),xmax);
f = (a/2).*gauss(xx,stdev).*...
           (cumgauss(xx-stdev.^2.*a,stdev)./gauss(xx-stdev.^2.*a,stdev)+...
           Rcumgauss(xx+stdev.^2.*a,stdev)./gauss(xx+stdev.^2.*a,stdev));
% alternative expression (possible numerically more stable)
% f = a.*exp(stdev.^2.*a.^2/2)/2.* ...
%     (exp(-a.*xx).*cumgauss(xx-a.*stdev.^2,stdev) + ...
%      exp(a.*xx).*Rcumgauss(xx+a.*stdev.^2,stdev));

x = abs(x);
% (1) exception: gauss(x+stdev^2*a,stdev) ->  0 (x large compared to stdev*a)
E1 = (gauss(x+stdev.^2.*a,stdev)<eps);
oo = find(E1);
fapprox = (a/2).*exp(stdev.^2.*a.^2/2).*exp(-a.*x) + ...
          (a/2).*gauss(x,stdev).*stdev.^2./(x+stdev.^2.*a);
f(oo) = fapprox(oo);
% (2) avoid NaN or Inf for stdev close to 0; for abs(x) large, the general
% formula performs well 
% E2 = (stdev<eps & a>eps);
E2 = (x+stdev.^2.*a<3*tol);
oo = find(E2);
fapprox = a/2.*exp(-a.*abs(x));
f(oo) = fapprox(oo);
% (3) exception: a -> Inf (1/a -> 0)
%     problems occur when x/stdev-a*stdev is close to -Inf
%                      or x/stdev+a*stdev is close to +Inf
%     the case where stdev->0 and a->Inf is handled by (1) and (2)
E3 = (abs(x./stdev)+stdev.*a>40);
oo = find(E3&(1-E1)&(1-E2));
fapprox = gauss(x,stdev);
f(oo) = fapprox(oo);

if reshapef==true,
   f = reshape(f,sizef);
end
