
function [betahat thr] = iterativeMSEST(Y,K,beta,maxit,betahat0,KTK);

% iterativeMSEST -- Iterative soft thresholding with step dependent threshold
%  Usage
%    [betahat thr] = iterativeMSEST(Y,K,beta,maxit,betahat0);
%  Inputs
%    Y      observations in sparsity model Y = K*beta+noise
%    K      model of covariates (rectangular matrix)
%    beta      unobserved sparse vector beta
%    maxit  maximum number of iterations
%    betahat0  initial value for betahat
%    KTK    binary var.(default 0); if one, then thresholding on standardized
%           components
%  Outputs
%    betahat   estimate of sparse vector beta
%    thr    vector of threshold values used throughout iteration steps
%  Description
%    Iterative soft thresholding with step-dependent, minimum MSE threshold 
%    betahat_(i+1) = ST(betahat_(i) + K'*(Y-K*betahat_old,thr(i))
%    or (when KTK == 1)
%    betahat_(i+1) = D*ST(inv(D)*((betahat_(i) + K'*(Y-K*betahat_old,thr(i)))
%    with D = diag(sqrt(diag(K'*K)));
%  Note
%  See also:
%    help iterativeST
%    help iterativeGCVST


ysize = size(Y);
Y = column(Y);

if nargin < 2,
   K = eye(length(Y));
end
[n m] = size(K);

if nargin < 4,
   maxit = 1000;
end

if nargin < 5,
   betahat0 = zeros(m,1);
end

if nargin < 6,
   KTK = 0;
end

beta0 = ones(m,1);
beta1 = betahat0;

if norm(beta1-beta0) < eps, % i.e., if betahat0 happens to be ones(m,1)
   beta0 = zeros(m,1);
end

D = ones(size(beta0));
if KTK,
   D = sqrt(diag(K'*K));
end
Dinv = 1./D;
Dbeta = Dinv.*beta;

i = 1;
thr = [];
while norm(beta1-beta0) > eps,
   beta0 = beta1;
   y1 = beta0 + K'*(Y-K*beta0);
   y1 = Dinv.*y1;
   [beta1 t] = minmsesoft(Dbeta,y1);
   beta1 = D.*beta1;
   thr = [thr t];
   i = i+1; 
   if i>maxit, 
      beta0 = beta1; 
      warning('iterativeMSEST: maximum number of iterations reached')
   end
end
betahat = beta1;

% Copyright (c) Maarten Jansen
%
% This software is part of ThreshLab and is copyrighted material. 
