
function trees = allbinarysubtrees(n,nnodes);

% allbinarysubtrees - ThreshLab/Trees 
%   Usage
%     trees = allbinarysubtrees(n,nnodes);
%   Inputs
%     n       integer; size of full binary tree
%     nnodes  integer; size of subtree
%   Outputs
%     trees   matrix in which each row has nnodes-tuples of integers in
%             {1,2,...,n}
%   Description
%     Each row in the output represents a subtree of nnodes nodes, rooted at
%     the root of a full binary tree with n nodes.
%     The output is a set of indices referring to the positions of the knots in
%     the full tree that belong to the subtree. The index system of the full
%     tree is as follows:
%     The root (level 0) of the tree and the subtree has index 1. 
%     The knots of the full tree at the next level (j=1) have indices 2 and 3. 
%     In general, the k-th knot at level j has index i=2^j+k,
%     where j = 0,1,.... and k = 0,...,2^j-1. 
%     Its offsprings have indices 2*i and 2*i+1.
%   See also
%     help showasgreyleveltree
%     help randombinarysubtree
%     help maxmassubtree


if n<nnodes, n = 2^n; end
% create first tree, with the smallest indices first (trees will be in
% lexicographic order)
trees = zeros(1,nnodes);
trees(1,1) = 1;
adjacentleaves = [2 3]; % leaves that linked to a leaf in the actual subtree
nc = 2;
t = 1; k = 1; l = 1;
c = [1 2]; % indices into adjacentleaves that are candidates for next tree
while k>0,
   % (1) tree completion
   while ~isempty(c) & k<nnodes,
      candidates = sort(adjacentleaves(c));
      l = candidates(1);
      k = k+1;
      trees(t,k) = l;
      adjacentleaves = setdiff(adjacentleaves,l); nc = nc-1;
      if 2*l<n+1,
         adjacentleaves = [adjacentleaves 2*l]; nc=nc+1;
      end
      if 2*l<n,
         adjacentleaves = [adjacentleaves 2*l+1]; nc=nc+1;
      end
      c = find(adjacentleaves > l);
   end
   if k==nnodes, % we found a complete tree, prepare for next tree
      trees = [trees; trees(t,1:nnodes)]; t = t+1;
   end
   % (2) backtracking
   l = NaN;
   while isnan(l), % try to replace trees(t,k) by a leaf with a higher index
                   % if no such a candidate leaf is available, descend in the
                   % tree (i.e., k <- k-1)
      l = trees(t,k);
      k = k-1;
      adjacentleaves = [setdiff(adjacentleaves,[2*l 2*l+1]) l];
      nc = length(adjacentleaves);
      % adjacentleaves = sort(adjacentleaves);
      c = find(adjacentleaves > l);
      if isempty(c), l = NaN; end
      if k == 0, l = 1; end
   end
end
nt = size(trees,1)-1;
trees = trees(1:nt,1:nnodes);

% Copyright (c) Maarten Jansen
% 
% This software is part of ThreshLab and is copyrighted material. 
