
% This Monte-Carlo simulation illustrates the working of tree structured
% parameter selection (finetuned best kappa-subtree selection), applied to 
% regression trees (CART), considered as an instance of an
% Adaptive Unbalanced Haar Transform (AUHT)
% The pseudo-observations may be normally or Poisson distributed
% See also
%   help simulationstudychangepointpoissonAUHT
%       for a simulation study calling this routine
%   help FAUHT
%       for the CART + Adaptive Unbalanced Haar Transform
%   help montecarlomirrorAUHT and help montecarlomirrorAUHTpoisson
%       for the calculation of the degrees of freedom after selection


if exist('makeplots') ~= 1, makeplots = true; end
if makeplots==true,
   aligncenter = {'horizontalalignment','center'};
   alignbottom = {'verticalalignment','bottom'};
   alignhorver = {aligncenter{:},alignbottom{:}};
   fontsize = 16;
   fontopt = {'fontsize',fontsize,'fontweight','bold'};
   latex = {'interpreter','LaTeX'};
   forestgreen = [34 139 34]/255;
   muplot = {'color','r','linewidth',3};
   Yplot = {'color','b','linewidth',2};
   Ytreeplot = {'color','m','linewidth',3};
   Ythrplot = {'color','b','linewidth',3};
   wplot = {'color','b'};
   PEplot = {'color','r','linewidth',2};
   Cpnuplot = {'color',forestgreen,'linewidth',2};
   Cpkplot = {'color','k','linewidth',2};
   GCVplot = {'color','m','linewidth',2};
   CVplot = {'color','b','linewidth',2};
   mirrorplot = {'o-','linewidth',2,'markersize',6,'markerfacecolor','w'};
   mirrorplot = {mirrorplot{:},'color','k'};
   if exist('BW') ~= 1, BW = false; end % black-white plots
   if BW==true,
      muplot{2} = 'k'; Yplot{2} = [1 1 1]*0.7;
      Ythrplot{2} = [1 1 1]*0.5; Ytreeplot{2} = [1 1 1]*0.2;
      wplot{2} = 'k';
      Cpkplot = {':','color','k','linewidth',2};
      PEplot = {'color','k','linewidth',2};
      GCVplot = {'color',[1 1 1]/2,'linewidth',2};
      CVplot = {'-.','color',[1 1 1]/2,'linewidth',3};
      Cpnuplot = {'-.','color','k','linewidth',2};
   end
end
if exist('realrandom') ~= 1, realrandom = false; end
if realrandom==false, randn('state',0); rand('state',0); end
if exist('samplesize') ~= 1, samplesize = 3000; end
if exist('setupdata') ~= 1, setupdata = true; end
if exist('gam') ~= 1, gam = 2; end

if setupdata==true,
   x = makegrid(samplesize,0);
   % mu = makesignal(x,'skyline');
   nc = 17; c = sort(rand(nc,1)); 
   mu = staircase(x,c,(1:nc)); mu = mu-2*floor(mu/2);
   Z = randn(size(mu));
   stdev = 1;
   Y = mu+stdev*Z;
   SigmaY = eye(samplesize);
   pdfy = 'normal';
else
   samplesize = length(Y);
   if exist('pdfy') ~= 1, pdfy = 'normal'; end
   x = (x-min(x))/(max(x)-min(x));
   if exist('stdev') ~= 1, stdev = 1; end
   % i.e., if stdev is not given, it is assumed that SigmaY contains the
   % covariance matrix. Otherwise, the covariance matrix is given by
   % stdev^2*SigmaY
end
if exist('CPmu') ~= 1,
   % CPmu: true change points; may have been calculated by calling routine
   CPmu = find(mu(2:samplesize)~=mu(1:samplesize-1))+1;
end
nCPmu = length(CPmu);

n = samplesize; I = eye(n);
if exist('SigmaY') ~= 1, SigmaY = I; end
% SigmaY typically known or estimated by a pilot estimator (see
% simulationstudychangepointpoissonAUHT)
x = column(x); mu = column(mu); Y = column(Y);

if makeplots==true,
   yticks = bestticks(Y); 
   yrange = (yticks(end)-yticks(1))*1.1; ymid = (yticks(end)+yticks(1))/2;
   axyobs = [0 1.05 ymid+[-1 1]*yrange/2];
   figure(1)
   plot(x,Y,Yplot{:})
   hold on
   plot(x,mu,muplot{:})
   hold off
   axis(axyobs)
   figsize = [960 420]; papersize = figsize/96; figpaperpos = [0 0 papersize];
   figpos = get(gcf,'position'); figpos(3:4) = figsize;
   set(gcf,'position',figpos,'papersize',papersize,'paperposition',figpaperpos)
   axis off
   drawgrids((0:5)/5,yticks,'k')
   axis(axyobs)
   drawaxes('xtick',(0:5)/5,'ytick',yticks,fontopt{:})

   t = ones(1,nCPmu)*10; t(2:2:nCPmu) = t(2:2:nCPmu)+1;
   hold on
   for cp = 1:nCPmu,
      xcp = x(CPmu(cp)); mucp = mu(CPmu(cp)); tcp = t(cp);
      text(xcp,tcp,num2str(cp),fontopt{:},alignhorver{:})
      plot([xcp xcp],[mucp tcp],'k--','linewidth',2)
   end
   hold off

   title('Observations',fontopt{:})
end


%%%%%%%%%%%%%%%%%%%%%%%% Forward AUHT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[w cc parents] = FAUHT(x,Y,gam);
v = FAUHTfromrefinement(x,mu,cc);
Wt = FAUHTfromrefinement(x,I,cc);

%%%%%%%%%%%%%%%%%%%%%%%% Variance estimation %%%%%%%%%%%%%%%%%%%%%%%%
% Use the residuals of a pilot estimator (soft-thresholding/lasso+GCV) for the
% estimation of the variance 
% SigmaY is either a known covariance structure (up to a constant), or the
% outcome from a pilot estimator (see simulationstudychangepointpoissonAUHT)
Sigmaw = Wt*SigmaY*Wt';
stdevw = column(sqrt(diag(Sigmaw)));
% stdevw defines the weighting used in the definition of the weighted sum of
% squared residuals. 

stdevw(stdevw<eps) = 1;
w1 = w./stdevw;
v1 = v./stdevw;
% threshold estimation (threshold selection by GCV) - unstructured estimation
[w1thr thr] = mingcvsoft(w1);
wthr = w1thr.*stdevw;
stdevest = norm(w1-w1thr)/sqrt(n);
% stdevest = norm(w1.*(abs(w1)<thr))/sqrt(sum(abs(w1)<thr));
% stdevest = MAD1(w1)/invcumgauss(3/4);
Ythr = IAUHT(x,wthr,cc);

%%%%%%%%%%%%%%%%%%%%%%%% Display AUHT coefficients %%%%%%%%%%%%%

if makeplots==true,
   figure(2)
   plotscales(w,wplot{:})
   set(gca,fontopt{:})
   set(gcf,'position',figpos,'papersize',papersize,'paperposition',figpaperpos)
   title('Adaptive unbalanced Haar coeff.',fontopt{:})
end


%%%%%%%%%%%%%%%%%%%%%%%% Tree structured selection %%%%%%%%%%%%%%%%%%

if exist('kmax') ~= 1, kmax = n-1; end
dk = 1;
kk = (1:dk:kmax);
if kk(end)<kmax, kk = [kk kmax]; end
nk = length(kk);
w2 = w1.^2;
if exist('dummies') ~= 1, dummies = []; end
if length(dummies)>0, w2(dummies) = 2*max(abs(w2)); end
[Msubtree subtrees] = maxmassubtree(w2,parents,kmax);
%%% Oracular selection for mirror curve
v2 = v1.^2; if length(dummies)>0, v2(dummies) = 2*max(abs(v2)); end
[MsubtreeO subtreesO] = maxmassubtree(v2,parents,kmax);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SSE = samplePEonS(subtrees,w1,w1)*n;
samplePE = samplePEonS(subtrees,w1,v1);
samplePEoracle = samplePEonS(subtreesO,w1,v1);
% mirror correction term
switch pdfy
case {'Poisson','poisson','pois'}
   disp('Poisson mirror')
   mhat = montecarlomirrorAUHTpoisson(x,Ythr,gam); mhat = mhat(kk);
   % Note: if pdfy is Poisson, then SigmaY follows from a pilot estimator and
   % stdev can be taken to be one (see simulationstudychangepointpoissonAUHT)
otherwise
   mhat = montecarlomirrorAUHT(x,gam)*stdev^2; mhat = mhat(kk);
end
Cpk = SSE/n + 2*kk/n*stdev^2 - stdev^2;
nu = kk+mhat/stdev^2*n;
gcvnu = (SSE/n)./(1-nu/n).^2;
Cpnu = SSE/n + 2*nu/n*stdev^2 - stdev^2;
[minsamplePE istar] = min(samplePE);
[minCp iCp] = min(Cpnu);
treek = subtrees{iCp};
% treek = subtrees{istar};
kCp = kk(iCp);

% Identify the locations of the selected change points 
% CPYtree = find(Ytree(2:samplesize)~=Ytree(1:samplesize-1))+1;
% [CPYtree nodes] = sort(cc(treek(2:end))+1);
CPYtree = cc(treek(2:end))+1;
nCPYtree = length(CPYtree);
% Match true and selected change points
[match1 closestreal] = matchpoints(CPmu,CPYtree);
i1 = find(~isnan(match1));
match1(i1) = CPYtree(match1(i1));

% Find the selected nodes that do not correspond to a real change point
falsepositivenodes = treek(find(isnan(closestreal))+1);
falsepositiveleaves = setdiff(falsepositivenodes,parents(treek));



% perform tree based selection
wtree = zeros(size(w)); wtree(treek) = w(treek);

% Compare to Cross Validation
Wt1 = diag(1./stdevw)*Wt;
cvHhalf = CVsparseselection(Y,Wt1,subtrees,n/2);
cvHhalf = cvHhalf - cvHhalf(istar) + minsamplePE;

if makeplots==true,
   figure(3)
   o = max(1,floor(length(kk)/50));
   plot(kk(1:o:end),samplePEoracle(1:o:end),mirrorplot{:})
   hold on
   plot(kk,samplePE,PEplot{:})
   % plot(kk,Cpnu,Cpnuplot{:})
   plot(kk,Cpk+2*mhat,Cpnuplot{:})
   plot(kk,gcvnu-1^2,GCVplot{:})
   plot(kk,Cpk,Cpkplot{:})
   plot(kk,cvHhalf,CVplot{:})
   hold off
   axy = axis;
   axy4 = max(samplePE(istar:nk))*2;
   axy4 = max(axy4,max(Cpnu(iCp:nk))*2);
   if ~isnan(axy4), axy(4) = axy4; end
   axy(3) = min([0 Cpk cvHhalf]);
   axy = axy.*[1 1.07 1 1];
   axis(axy)
   axis off
   xtick = bestticks(kmax);
   set(gcf,'position',figpos,'papersize',papersize,'paperposition',figpaperpos)
   drawaxes('xtick',xtick,fontopt{:})
   alignment = {'HorizontalAlignment','right','VerticalAlignment','bottom'};
   text(axy(2)*0.95,0,'model size \kappa',alignment{:},fontopt{:})
   legend('$\mathrm{PE}(\widehat{\mathbf{v}}_{\mathrm{O}_\kappa})$',...
          '$\mathrm{PE}(\widehat{\mathbf{v}}_\kappa)$',...
          '$\Lambda(\widehat{\mathbf{v}}_\kappa)$',...
          '$\mathrm{GCV}(\widehat{\mathbf{v}}_\kappa)$',...
          'Naive $C_p$','CV',latex{:})
   set(gca,fontopt{:})
end

Ytree = IAUHT(x,wtree,cc);

%%%%%%%%%%%%%%%%%%%%%%%% Computation SNR %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

snrin = snr2(mu,Y-mu);
snrthr = snr2(mu,Ythr-mu);
snrtree = snr2(mu,Ytree-mu);
txt1 = 'SNR input: '; txt2 = '; SNR thr: '; txt3 = '; SNR tree: ';
disp([txt1 num2str(snrin) txt2 num2str(snrthr) txt3 num2str(snrtree)])

if makeplots==true,
   figure(4)
   plot(x,mu,muplot{:})
   hold on
   plot(x,Ythr,Ythrplot{:})
   hold off
   axis(axyobs)
   set(gcf,'position',figpos,'papersize',papersize,'paperposition',figpaperpos)
   axis off
   drawgrids((0:5)/5,yticks,'k')
   axis(axyobs)
   drawaxes('xtick',(0:5)/5,'ytick',yticks,fontopt{:})
   title('Unstructured selection (thr)',fontopt{:})

   figure(5)
   plot(x,mu,muplot{:})
   hold on
   plot(x,Ytree,Ytreeplot{:})
   hold off
   axis(axyobs)
   set(gcf,'position',figpos,'papersize',papersize,'paperposition',figpaperpos)
   axis off
   drawgrids((0:5)/5,yticks,'k')
   axis(axyobs)
   drawaxes('xtick',(0:5)/5,'ytick',yticks,fontopt{:})
   title('Tree structured selection',fontopt{:})

   figure(6)
   % Next four lines for screen filling figure
   % figpostree = get(0,'ScreenSize');
   % figpaperpostree = [0 0 figpostree(3:4)/96];
   % set(gcf,'position',figpostree,'paperposition',figpaperpostree)
   % set(gcf,'paperposition',figpaperpos)
   set(gcf,'position',figpos,'papersize',papersize,'paperposition',figpaperpos)
   maxlevels = 9;
   sizetree = 2^maxlevels;
   fulltreecolor = [1 1 1]*0.6;
   % fulltreeopt = {'fullcolor',fulltreecolor};
   fulltreeopt = {'onecolor',fulltreecolor};
   showasgreyleveltree(sizetree,parents,fulltreeopt{:},'maxlevels',maxlevels);
   hold on
   [pos level] = showasgreyleveltree(sizetree,parents,treek,'maxlevels',maxlevels);
   hold off
   axis off
   for cp=1:nCPYtree,
      cpreal = closestreal(cp);
      if ~isnan(cpreal),
         node = treek(cp+1);
         text(pos(node),level(node),num2str(cpreal),fontopt{:},alignhorver{:})
      end
   end

end
