
function [KappaXhat nit] = maxLsparseKappa(SigmaXhat,S,KappaXhat,maxit,tol);

% maxLsparseKappa - ThreshLab/Trees
%   Usage
%     [KappaXhat nit] = maxLsparseKappa(SigmaXhat,S,KappaXhat,maxit,tol);
%   Inputs
%     SigmaXhat   m x m sample covariance matrix (X*X/n)
%     S           binary m x m matrix, ones pointing to nonzeros in output
%     KappaXhat   m x m initial concentration matrix
%     maxit       integer; maximum number of iterations
%     tol         real number; tolerance for stopping criterion in iteration
%   Outputs
%     KappaXhat   estimated concentration matrix
%     nit         number of iterations used
%   Description
%     applies iterative gradient projection for the maximisation of the log
%     likelihood l(KappaX) = log(det(KappaX))-trace(KappaX*SigmaXhat)
%     constraint by the sparsity condition that KappaX(i,j) = 0 if S(i,j) = 0
%     The search takes place along the projected gradient
%     LambdahatS = inv(KappaXhat)-SigmaXhat
%   Remark
%     Non-symmetric inputs may lead to imaginary results
%   See also
%     help nodewisereggraph
%     help argmaxLKappaplusalphaLambda


if nargin<5, tol = 1.e-6; end
if nargin<4, maxit = 1; end

it = 0;
I = eye(size(SigmaXhat));
txt = 'maxLsparseKappa launching iterations. This may take a while. (maxit = ';
disp([txt num2str(maxit) ')...'])
while it<maxit,
   it = it+1;
disp(['   iteration step ' num2str(it)])
   KappaXhat0 = KappaXhat.*S;
   SigmaXhat0 = inv(KappaXhat);
   Lambdahat = SigmaXhat0-SigmaXhat;
   LambdahatS = Lambdahat.*S;
   [E Lambda1] = eig(SigmaXhat0^2*LambdahatS);
   Lambda2 = E*Lambdahat/E;
   alfa = argmaxLKappaplusalphaLambda(KappaXhat,LambdahatS,SigmaXhat);
   KappaXhat = KappaXhat0+alfa*LambdahatS;
   if norm(alfa*LambdahatS,'fro')<tol, maxit = it; end
end
nit = maxit;
