
function [maxmas trees] = maxmassubtree(x,parents,nnodes)

% maxmassubtree - ThreshLab/Trees - finds k size subtrees with max. weight/mass
%   Usage
%     [maxmas trees] = maxmassubtree(x,parents,nnodes);
%     [maxmas trees] = maxmassubtree(x,children,nnodes);
%   Inputs
%     x        real vector, weight/mass in every node of the tree (see below)
%     parents  integer vector of size(x), indices into x, for each element of x
%              containing the index of its ancestor (default = floor((1:n)/2),
%              i.e., maximal/full binary tree)
%     children cell or integer matrix, indices into x  (no default)
%     nnodes   integer larger than 2; maximum size of subtree (number of 
%              nodes; default = length(x))
%   Outputs
%     maxmas    vector with components sum(x(trees{i}))
%     trees     cell of nnodes arrays with integer elements
%   Description
%     This routine performs a backtracking search for optimal k-subtrees for
%     all values of k between 1 and nnodes.
%     The k-th cell of the output variable trees contains an integer vector of
%     length k, representing the subtree of size k with the maximum mass
%     A subtree is represented by the indices of its nodes into the full 
%     rooted tree.
%     The index system of the full tree is given by the vector parents
%   Notes
%     (1) The computational complexity is of order O(n*nnodes), where n is the
%     size of the full tree (i.e., size of x). 
%     This is quadratic when nnodes = O(n).
%     (2) Storage of the intermediate and final subtrees in output trees fails
%     when x and nnodes are very large. In that case, use maxmassubtreeXL
%     instead.
%   See also
%     help maxmassubtreeXL for large data sets

n = length(x);
% n0 = n; n = 2^(ceil(log2(n+1)))-1; x0 = x; x = zeros(1,n); x(1:n0) = row(x0);
if nargin < 3, nnodes = n; end
if nargin < 2, parents = floor((1:n)/2); end
if n>1,
   if max(size(nnodes)) == n,
      change = nnodes; nnodes = parents; parents = change;
   end
end
if nnodes < 1 | nnodes-floor(nnodes) ~= 0,
   disp(['nnodes = ' num2str(nnodes)])
   error('nnodes must be a positive integer')
end

% make sure to have parents as input
[parents children] = parentschildren(parents);

% add one superroot 
x = [0 row(x)]; parents = [0 parents+1]; nnodes = nnodes+1; n = n+1;

% Now, find the children
children = parentschildren(parents,'cell');

nnodeswithc = length(children); % nodes beyond nnodeswithc cannot possibly 
                                % have children
                                % (other nodes may or may not have children)
cc = 1;
maxmas = {};
trees = {};
active = ones(1,n); nactive = n;
level = 0; % throughout the iterations, the routine will maintain a path from
           % the top (root) of the tree down to leaf level
while nactive>1,
   % go down in tree, as far/as deep as possible
   while length(cc)>0, 
      level = level+1; p = cc(1);
      maxmas = {maxmas{:},x(p)}; % adds a subtree rooted at p with just p as
                                 % single node (to be extended)
      trees = {trees{:},{p}};
      if p>nnodeswithc, cc = []; else, cc = children{p}; end
   end
   % merge p and its parent (backtracking)
   % p and c may be single node trees, or roots of extended trees.
   c = p; p = parents(c);
   massp = maxmas{level-1}; massc0 = [0 maxmas{level}];
   subtreesp = trees{level-1};
   subtreesc = trees{level};
   ntp = length(massp); ntc = length(massc0)-1;
   ntpc = ntp+ntc; % ntpc maximum size of tree combining p and c
   masspc = zeros(1,ntpc); subtreespc = cell(1,ntpc);
   for s = 1:ntpc,
      % best s-subtree combining p with c. This subtree must contain at
      % least one node from p (the root p itself), otherwise, it cannot
      % be linked to nodes further up in the tree.
      s1 = max(1,s-ntc); s2 = min(ntp,s);
      addedmass = massp(s1:s2)+massc0(s-s1+1:-1:s-s2+1);
      [masspc(s) l] = max(addedmass); l = l+s1-1;
      subtreespc{s} = subtreesp{l};
      if l<s, subtreespc{s} = [subtreespc{s} subtreesc{s-l}]; end
   end
   % clean up after merging and go one level up in tree
   % cut off child (no longer needed), remove its entries in maxmas
   % and trees
   cc = setdiff(children{p},c); children{p} = cc;
   active(c) = 0; nactive = nactive-1;
   level = level-1;
   trees{level} = subtreespc;
   maxmas{level} = masspc;
   trees = trees(1:level);
   maxmas = maxmas(1:level);
   % Now jump back to the beginning of the loop. 
   % From there, we may either redescend into another subtree, or we may have 
   % to climb up and merge further, before we can go down again.
end

maxmas = maxmas{1};
trees = trees{1};
nnodes = nnodes-1;
maxmas = maxmas(2:nnodes+1);
trees1 = trees;
trees = cell(1,nnodes);
for k=1:nnodes,
   treesk = trees1{k+1};
   trees{k} = treesk(2:end)-1;
end


% Copyright (c) Maarten Jansen
% 
% This software is part of ThreshLab and is copyrighted material. 
