
function [Ghat KappaXhat lastproblem] = nodewiseLassoGCVCpmirror(X,varargin)

% nodewiseLassoGCVCpmirror - ThreshLab/Trees
%   Usage
%     [Ghat KappaXhat] = nodewiseLassoGCVCpmirror(X,<optional inputs>);
%   Inputs
%     X           m times n matrix; n observations of m-variate normal random
%                                   vector with zero mean
%   Optional inputs (in any order)
%     addmirror   boolean variable (true by default)
%     kmax        maximum size of selection
%     lassomethod string: 'LARS','ITST', or 'LARSmodified'
%     IC          cell of two strings, fixing the information criteria used
%                 throughout the calculations IC{1},IC{2} are 'Cp' or 'GCV'
%                 The second string variable fixes the finetuning of the final
%                 LS selection and estimation
%                 If the second string is 'none', then the lasso shrinkage
%                 approach of the first string variable is taken as final
%                 result.
%     cc          integer vector; list of nodes in which regression is carried
%                 out (default (1:m), with m the dimension of the random vector)
%     'sym'       imposes symmetric output
%   Outputs
%     Ghat        estimated graph matrix
%     KappaXhat   estimated concentration matrix
%   Description
%     applies Meinshausen-Bühlmann (AnStat, 2006) nodewise regression method.
%     LARS is used as lasso-solver, adopting IC{1} as initial model assessment
%     for variable selection with shrinkage. In the next step the selected
%     models are re-assessed using IC{2} with mirror correction for selection
%     without shrinkage, and -- if required by the expression of IC{2} -- with
%     explicit variance estimator based on the residuals of the previous step.
%   Remark
%     Symmetry in the output matrix is not guaranteed.
%     Set Ghat = Ghat*Ghat'; or Ghat = max(1,Ghat+Ghat') to impose symmetry

disp('This is nodewiseLassoGCVCpmirror. Execution may take a while ...')
%%% parse the input variables
IC = {'GCV','Cp'}; lassomethod = 'LARSmodified'; kmax = 100; addmirror = true;
dosym = false;
cc = NaN; kmaxfound = false;
nvarargin = length(varargin);
for k = 1:nvarargin, vark = varargin{k};
   if iscell(vark), if length(vark)==2, IC = vark; end
   elseif ischar(vark),
      switch(vark)
      case {'dosym','sym','symm'},
         dosym = true;
      otherwise
         lassomethod = vark;
      end
   elseif ismember(vark,[0 1]), addmirror = vark; 
   elseif isnumeric(vark)
      if length(vark)==1, 
         if floor(vark)==vark, 
            if kmaxfound==false,
               kmax = vark; kmaxfound = true;
            else
               cc = vark;
            end
         end
      else, cc = vark;
      end
   end
end

[m,n] = size(X);
kmax = min(kmax,n-1);
% from n observations, at most n edges (nonzero beta's) can be estimated
if isnan(cc), cc = (1:m); else, cc = cc(find(cc<m+1)); cc = cc(find(cc>0)); end
mc = length(cc); 

Graphmatrix = sparse(zeros(m,m));
KappaXhat = sparse(zeros(m,m));
benchmarks = 101;
if mc>100, benchmarks = [(10:10:90) 110]; end
if mc>1000, benchmarks = [1 5 (10:10:90) 95 99 110]; end
b = 1;
for l=1:mc,
   c = cc(l); notc = [1:c-1 c+1:m];
   if l>benchmarks(b)/100*mc, disp([num2str(floor(l/mc*100)) ' %']), b=b+1; end
   Yc = X(c,1:n)'; Xc = X(notc,1:n)';
   % column normalisation (needed for LARS)
   Dc = diag(sqrt(sum(Xc.^2))); Xc = Xc/Dc; Yc = Yc/norm(Yc);
   % by normalising the response as well, the betahat remains unchanged
   switch IC{1},
   case {'Cp'}
      % stdevchat = MAD1(Yc)/invcumgauss(3/4);
      stdevchat = 1/sqrt(n); % overestimates stdevc, as it ignores the
                             % regression of Yc for Xc
   otherwise
      stdevchat = NaN;
   end
   if stdevchat>0 | isnan(stdevchat),
      switch lassomethod
      case {'LARS'},
         [betachat muchat S estPElasso lambda Sopt] = LARS(Yc,Xc,stdevchat,kmax);
      % case {'ITST','itST'},
      %    betachat = iterativeST(Yc,Xc,
      otherwise
         [betachat muchat S estPElasso lambda Sopt] = LARSmodified(Yc,Xc,stdevchat,kmax);
      end
      % Use nodewise regression for the lasso/shrinkage estimation of the
      % concentration matrix entries
      betachat = sparse(Dc\column(betachat)); 
      sigma2chat = norm(Yc-Xc*betachat)^2/n;
      % check: norm(Yc-muchat)^2/n;
      KappaXhat(c,c) = 1/sigma2chat;
      KappaXhat(c,notc) = -KappaXhat(c,c)*betachat';
      stdevchat = norm(Yc-muchat)/sqrt(n);
      keepSopt = true;
      undoshr = true;
      switch IC{2},
      case {'Cp'}
         estPELS = CponSmodelsize(S,Yc,Xc,stdevchat);
         if addmirror==true,
            estmirror = empiricalmirror(S,Xc);
            estPELS = estPELS+2*estmirror*stdevchat^2;
         end
         [estPELSopt sopt] = min(estPELS);
         keepSopt = false;
      case {'GCV'}
         estPELS = GCVonSmodelsize(S,Yc,Xc);
         if addmirror==true,
            estmirror = empiricalmirror(S,Xc);
            estPELS = estPELS+2*estmirror*stdevchat^2;
         end
         [estPELSopt sopt] = min(estPELS);
         keepSopt = false;
      otherwise
         estPELS = NaN; undoshr = false;
      end
      if keepSopt==false, ii = [];
         for s = 1:sopt,
            i = S(s);
            if i > 0, ii = [ii i]; else, ii = setdiff(ii,i); end
         end
         Sopt = ii;
      end
      Sopt(Sopt>=c) = Sopt(Sopt>=c)+1;
      Graphmatrix(c,Sopt) = 1;
   end
end
if dosym==true,
   Ghat = Graphmatrix.*Graphmatrix';
else
   Ghat = Graphmatrix;
end
if undoshr,
   % SigmaXhat = X*X'/n;
   KappaXhat = nodewisereggraph(X,Ghat);
   lastproblem = {Yc,Xc,S,estPElasso,estPELS,lambda,Dc};
end
